

define(
  'touch/app/Runtime',[
    ''
  ],
  function Runtime(
    junk
  ) {

    var url = Url.parse(window.location.href);
    var runtimeType = url.host.replace(/.buxfer.com$/, '');
    var isDev = (runtimeType === 'dev');

    var parts = runtimeType.split('-');
    if (parts.length == 2) {
      isDev = true;
      runtimeType = parts[0];
    }

    var userAgent = window.navigator.userAgent;
    var isAndroid = userAgent.match(/Android/);

    return {

      isApp: false,
      isDev: isDev,
      isDemo: (runtimeType === 'demo'),
      isHarvest: (runtimeType === 'harvest'),
      isOffline: false,

      isAndroid: isAndroid,

      canWorkOffline: function canWorkOffline() {
        // return true;
        return this.isApp;
      },

      getBrandName: function getBrandName() {
        if (this.isHarvest) {
          return 'The Harvest Plan';
        }
        return 'Buxfer';
      }

    };

  }
);



define(
  'touch/network/Network',[
    'touch/app/Runtime',
    ''
  ],
  function Network(
    Runtime,
    junk
  ) {

    return {

      init: function init() {
        Backbone.ajax = FunctionUtils.bind(this, this.sendRequest);
      },

      sendRequest: function sendRequest(request) {
        if (Runtime.baseUrl) {
          request.url = Runtime.baseUrl + request.url;
        }

        var headers = request.headers || {};
        if (Runtime.isApp) {
          headers['BX-PLATFORM'] = window.device.platform;
          headers['BX-APP-VERSION'] = AppConfig.version;
        }
        request.headers = headers;

        Vendor.jQuery.ajax(request);
      }

    };

  }
);



define(
  'touch/core/Model',[
    'touch/app/Runtime',
    ''
  ],
  function Model(
    Runtime,
    junk
  ) {

    return {
      
      create: function create(attributes) {
        var ModelClass = this.createClass(attributes);
        return new ModelClass();
      },

      createCollection: function createCollection(attributes) {
        var ModelClass = this.createCollectionClass(attributes);
        return new ModelClass();
      },

      createClass: function createClass(attributes) {
        return Backbone.Model.extend(attributes);
      },

      createCollectionClass: function createCollectionClass(attributes) {
        return Backbone.Collection.extend(attributes);
      }

    };

  }
);




define(
  'touch/auth/Session',[
    'touch/core/Model',
    ''
  ],
  function Session(
    Model,
    junk
  ) {

    return Model.create({
      url: '/apix/session',

      isInitialized: false,

      getLoggedInUid: function getLoggedInUid() {
        return this.attributes.uid;
      },

      isLoggedIn: function isLoggedIn() {
        return typeof this.attributes.uid != 'undefined';
      },

      getEmail: function getEmail() {
        return this.attributes.email;
      },

      getPreference: function getPreference(name) {
        if (!this.attributes.preferences) {
          return null;
        }
        return this.attributes.preferences[name];
      },

      init: function init() {
        if (this.isInitialized) {
          return;
        }

        var didFetchSession = FunctionUtils.bind(this, function didFetchSession() {
          this.isInitialized = true;
          this.trigger('didInit');
        });

        var didSessionFetchFail = FunctionUtils.bind(this, function didSessionFetchFail() {
          this.trigger('didFailInit');
        });

        this.fetch({
          success: didFetchSession,
          error: didSessionFetchFail
        });
      },

      logout: function logout() {
        this.destroy(); // Server-side
        this.clear(); // Client-side
        this.trigger('refresh');
      }

    });

  }
);



define(
  'touch/app/NavigationController',[
    'touch/app/Runtime',
    'touch/auth/Session',
    ''
  ],
  function AppNavigationController(
    Runtime,
    Session,
    junk
  ) {

    var router, currentPage;
    var ROUTE_NOT_FOUND = 'notFound';
    var pageBody = DOM.findBySelector('body');
    var urlStack = [];
    var stackPosition = -1;

    return {

      getCurrentUrl: function getCurrentUrl(options) {
        options = options || {};
        var absoluteUrl = window.location.href;
        if (options.absolute) {
          return absoluteUrl;
        }
        var url = Url.parse(absoluteUrl);
        url.host = '';
        url.port = '';
        return url.toString();
      },

      filterCurrentUrl: function filterCurrentUrl(filters) {
        var url = Url.parse(this.getCurrentUrl());
        url.addQueryParams(filters);
        this.openUrl(url.toString());
      },

      navigateBack: function navigateBack() {
        if (stackPosition == 0) {
          this.setUrl('/dashboard');
        } else {
          history.back();
        }
      },

      reloadPage: function reloadPage(options) {
        options = options || {};

        this.openUrl(this.getCurrentUrl());

        if (!options.avoidSessionInit) {
          Session.init();
        }
      },

      pushUrl: function pushUrl(url, options) {
        this._openUrlWithPresentationMethod(url, options, 'push');
      },

      popUrl: function popUrl(url, options) {
        this._openUrlWithPresentationMethod(url, options, 'push');
      },

      setUrl: function setUrl(url, options) {
        this._openUrlWithPresentationMethod(url, options, 'appear');
      },

      _openUrlWithPresentationMethod: function _openUrlWithPresentationMethod(url, options, presentationMethod) {
        options = options || {};
        options.presentationMethod = presentationMethod;
        this.openUrl(url, options);
      },

      openUrl: function openUrl(url, options) {
        this.dismissPopup();
        this.dismissModal();
        this._routeUrl(url, options);
      },

      openExternalUrl: function openExternalUrl(url) {
        if (Runtime.isApp) {
          window.open(url, '_system');
        } else {
          window.location = url;
        }
      },

      isSidebarOpen: function isSidebarOpen() {
        return DOM.hasClass(pageBody, 'sidebarOpen');
      },

      isModalOpen: function isModalOpen() {
        return DOM.hasClass(pageBody, 'modalOpen');
      },

      isPopupOpen: function isPopupOpen() {
        return DOM.hasClass(pageBody, 'popupActive');
      },

      toggleSidebar: function toggleSidebar() {
        var sidebarContainer = DOM.findBySelector('.UISidebar');
        var isSidebarOpen = DOM.hasClass(pageBody, 'sidebarOpen');
        var windowSize = DOM.windowSize();

        sidebarContainer.css('width', windowSize.width);

        var targetLeft;
        if (isSidebarOpen) {
          targetLeft = -windowSize.width;
        } else {
          targetLeft = 0;
          sidebarContainer.css('left', -windowSize.width);
        }
        DOM.removeClass(pageBody, 'sidebarOpen');
        if (isSidebarOpen) {
          DOM.addClassIf(pageBody, 'sidebarClosing', isSidebarOpen);
        } else {
          DOM.addClassIf(pageBody, 'sidebarOpening', !isSidebarOpen);
        }

        this._runAnimation(
          sidebarContainer,
          {
            left: targetLeft
          },
          {
            complete: function() {
              if (isSidebarOpen) {
                DOM.removeClass(pageBody, 'sidebarClosing');
              } else {
                DOM.removeClass(pageBody, 'sidebarOpening');
                DOM.addClass(pageBody, 'sidebarOpen');
              }
            }
          }
        );
      },

      _configurePresentedComponent: function _configurePresentedComponent(component) {
        var browserTitle = component.getBrowserPageTitle ? component.getBrowserPageTitle() : '';
        if (browserTitle) {
          browserTitle = StringUtils.ucwords(browserTitle) + ' - ';
        }
        document.title = browserTitle+Runtime.getBrandName();
      },

      presentModal: function presentModal(modal) {
        modal = React.cloneElement(modal, {
          presentationMethod: 'modal'
        });

        this.dismissPopup({
          animated: false
        });

        var windowSize = DOM.windowSize();
        var isFirstModal = !this.isModalOpen();

        if (isFirstModal) {
          var scrollOffset = DOM.getPageScrollOffset();
          if (scrollOffset) {
            pageBody.attr('modal-scroll-offset', scrollOffset);
          }
          DOM.addClass(pageBody, 'modalOpening');
        }

        var modalContainer = DOM.createElement('div');
        DOM.addClass(modalContainer, 'modalContainer layerOverlay');
        DOM.findBySelector('.modalBackgroundContainer').append(modalContainer);
        var modalComponent = React.render(modal, modalContainer[0]);
        DOM.setAttribute(modalContainer, 'previousBrowserTitle', document.title);
        this._configurePresentedComponent(modalComponent);

        modalContainer.css('top', windowSize.height);
        this._runAnimation(
          modalContainer,
          {
            top: 0
          },
          {
            complete: function() {
              // Setting the scroll offset is required as the first thing in
              // this completion block to avoid jittery effects on potential
              // keyboard focus in didModalPresent
              DOM.setPageScrollOffset(0);

              DOM.removeClass(pageBody, 'modalOpening');
              DOM.addClass(pageBody, 'modalOpen');
              if (modalComponent.didModalPresent) {
                modalComponent.didModalPresent();
              }
            }
          }
        );
      },

      dismissModal: function dismissModal() {
        if (!this.isModalOpen()) {
          return;
        }

        var windowSize = DOM.windowSize();
        var allModalContainers = DOM.findAllBySelector('.modalContainer');

        var modalContainer = allModalContainers.last();
        document.title = modalContainer.attr('previousBrowserTitle');

        var isLastModal = (allModalContainers.length == 1);
        var scrollOffset = 0;
        if (isLastModal) {
          DOM.removeClass(pageBody, 'modalOpen');
          DOM.addClass(pageBody, 'modalClosing');
          scrollOffset = pageBody.attr('modal-scroll-offset');
          DOM.removeAttribute(pageBody, 'modal-scroll-offset');
        }

        // Needed to put the "Add Transaction" button back to it's fixed
        // position if the modal dismissal also causes keyboard dismissal.
        // Test case: Tap "Add Transaction" > Focus Description > Dismiss
        DOM.setPageScrollOffset(scrollOffset);

        this._runAnimation(
          modalContainer,
          {
            top: windowSize.height
          },
          {
            complete: function() {
              DOM.removeClass(pageBody, 'modalClosing');
              React.unmountComponentAtNode(modalContainer[0]);
              modalContainer.remove();
            }
          }
        );
      },

      presentPopup: function prepsentPopup(component, options) {
        component = React.cloneElement(component, {
          presentationMethod: 'popup'
        });
        options = options || {};

        // Class needs to be added to the body first, so any setup in
        // `componentDidMount` (as in UITransactionInput) works.
        DOM.addClass(pageBody, 'popupActive');
        var container = DOM.findBySelector('.popupContainer');
        container.css('opacity', 0);

        React.render(component, container[0]);

        var popupBackgroundContainer = DOM.findBySelector('.popupBackgroundContainer');
        var screenSize = DOM.size(popupBackgroundContainer);
        var containerSize = DOM.size(container);

        var popupWidth = options.width || '90%';
        if (StringUtils.endsWith(popupWidth, '%')) {
          var widthFraction = parseInt(StringUtils.stripSuffix(popupWidth, '%')) / 100;
          popupWidth = widthFraction * screenSize.width;
        }
        container.css('width', popupWidth);

        popupWidth = Math.min(0.9 * screenSize.width, popupWidth);

        // Vertical positioning rules:
        // - Popup taller than screen => Small fixed top offset.
        // - Popup tall, but fits in screen => Center vertically.
        // - Popup small => Larger fixed top offset.
        var popupTop = (screenSize.height - containerSize.height) / 2;
        popupTop = Math.min(100, popupTop);
        popupTop = Math.max(20, popupTop);

        var containerPosition = {
          left: (screenSize.width - popupWidth) / 2,
          top: popupTop
        };
        container.css(containerPosition);

        this._runAnimation(container, {
          opacity: 1
        });
      },

      _runAnimation: function _runAnimation(container, props, options) {
        options = options || {};

        // options.duration = 10000;

        var onComplete = options.complete;
        options.complete = function() {
          DOM.removeClass(pageBody, 'transitionActive');
          onComplete && onComplete();
        };

        DOM.addClass(pageBody, 'transitionActive');
        container.velocity(props, options);
      },

      dismissPopup: function dismissPopup(options) {
        options = options || {};

        if (!this.isPopupOpen()) {
          return;
        }

        var container = DOM.findBySelector('.popupContainer');

        var onAnimationComplete = function onAnimationComplete() {
          React.unmountComponentAtNode(container[0]);
          DOM.removeClass(pageBody, 'popupActive');
        };

        if (options.animated === false) {
          container.css('opacity', 0);
          onAnimationComplete();
        } else {
          this._runAnimation(
            container,
            {
              opacity: 0
            },
            {
              complete: onAnimationComplete
            }
          );
        }
      },

      initWithRouteConfig: function initWithRouteConfig(routeConfig) {
        this.routeConfig = routeConfig;

        var routeMap = {};
        ArrayUtils.each(this.routeConfig.routes, function(route, info) {
          var routeKey = StringUtils.stripPrefix(route, '/');
          routeMap[routeKey] = route;
        });
        routeMap['*notFound'] = ROUTE_NOT_FOUND;
        var AppRouter = Backbone.Router.extend({
          routes: routeMap
        });
        router = new AppRouter();

        // Needed for browser-based back/forward navigation, and also navigating
        // to the initial url when loading '/'
        router.on('route', FunctionUtils.bind(this, function(route, params) {
          if (route === ROUTE_NOT_FOUND) {
            route = params[0];
            params = params.slice(1);
          }

          if (!route || !route.length) {
            return;
          }

          // TODO: Navigation from logged-in page to logged-out page should not
          // be allowed. Not sure how Android apps deal with it.
          if (StringUtils.endsWith(route, 'index.html')) {
            navigator.app.exitApp();
            return;
          }

          var url = Url.createWithPath(route);
          if (params.length) {
            url.setQueryString(params[0]);
          }

          this.openUrl(url, {
            fromBrowserNavigation: true
          });
        }));

        Backbone.history.start({
          pushState: true,
          // Initial routing needs to be explicitly handled by AppInit
          silent: true
        });

        this._initSwipeHandler();
      },

      _initSwipeHandler: function _initSwipeHandler() {
        var hammer = new Hammer(pageBody[0]);
        hammer.get('swipe').set({ velocity: 0.2 });
        hammer.on('swipeleft swiperight', FunctionUtils.bind(this, function(event) {
          if (!Session.isLoggedIn() || this.isPopupOpen() || this.isModalOpen()) {
            return;
          }

          var isSidebarOpen = DOM.hasClass(pageBody, 'sidebarOpen');
          var toggleDirection = isSidebarOpen ? 'swipeleft' : 'swiperight';
          if (event.type != toggleDirection) {
            return;
          }

          if (!isSidebarOpen) {
            var startX = event.pointers[0].pageX - event.distance;
            if (startX > 50) {
              return;
            }
          }

          this.toggleSidebar();
          Event.stop(event.srcEvent);
        }));
      },
    
      _routeUrl: function _routeUrl(url, options) {
        options = options || {};
        var urlString;
        if (Object.isString(url)) {
          urlString = url;
          url = Url.parse(urlString);
        } else {
          urlString = url.toString();
        }

        if (!urlString) {
          return;
        }

        var route = url.path;
        var info = this.routeConfig.routes[route];
        if (!info) {
          this._handle404(url);
          return;
        }

        var ComponentClass = info.component;
        if (!ComponentClass) {
          WARNING('Attempted to open invalid url: '+urlString);
          return;
        }

        var presentationMethod = ComponentClass.presentationMethod || info.presentationMethod;
        var componentFactory = React.createElement(ComponentClass, {
          url: url,
          navigationCategory: info.navigationCategory,
          params: url.queryParams
        });

        try {
          if (presentationMethod === 'modal') {
            this.presentModal(componentFactory);
            return;
          }

          this._presentPageAtUrl(componentFactory, url, options);

        } catch (e) {
          ErrorManager.handleException(e, 'Error rendering page');
        }
      },

      chromeDidInit: function chromeDidInit() {
        currentPage = null;
      },

      _presentPageAtUrl: function _presentPageAtUrl(componentFactory, url, options) {
        var urlString = url.toString();
        var appContainer = DOM.findBySelector('.appContainer');
        DOM.setPageScrollOffset(0);

        var onNewPageLoad = FunctionUtils.bind(this, function onNewPageLoad(component) {
          this._configurePresentedComponent(component);
          currentPage = component;
          if (!options.fromBrowserNavigation) {
            router.navigate(url.toString());
          }
        });

        var presentationMethod;
        if (options.fromBrowserNavigation) {
          if (
            stackPosition === urlStack.length - 1
            || (stackPosition > 0 && window.location.pathname === urlStack[stackPosition - 1].path)
          ) {
            presentationMethod = 'pop';
            --stackPosition;
          } else {
            presentationMethod = 'push';
            ++stackPosition;
          }
        } else if (
          options.presentationMethod === 'appear'
          || !currentPage
          || window.location.pathname === url.path
        ) {
          presentationMethod = 'appear';
          urlStack = [url];
          stackPosition = 0;
        } else {
          presentationMethod = 'push';
          urlStack.push(url);
          ++stackPosition;
        }

        // console.log('Presenting page', url.path, presentationMethod, stackPosition);
        componentFactory = React.cloneElement(componentFactory, {
          navigationDepth: stackPosition
        });

        appContainer.append('<div class="pageContainer">');

        if (!currentPage) {
          var pageContainer = DOM.findBySelector('.appContainer .pageContainer');
          var component = React.render(componentFactory, pageContainer[0]);
          onNewPageLoad(component);
          return;
        }

        var allPageContainers = DOM.findAllBySelector('.appContainer .pageContainer');
        var newPageContainer = allPageContainers.last();
        var oldPageContainer = allPageContainers.first();

        var windowSize = DOM.windowSize();

        var pageToAnimate, animationProperties;
        if (presentationMethod === 'push') {
          pageToAnimate = newPageContainer;
          pageToAnimate.css('left', windowSize.width);
          animationProperties = {
            left: 0
          };
        } else {
          pageToAnimate = oldPageContainer;
          animationProperties = {
            left: windowSize.width
          };
        }

        var needsAnimation = (presentationMethod !== 'appear');
        React.enableNetworking = !needsAnimation;
        var component = React.render(componentFactory, newPageContainer[0]);

        var onPresentComplete = FunctionUtils.bind(this, function onPresentComplete() {
          React.unmountComponentAtNode(oldPageContainer[0]);
          oldPageContainer.remove();
          onNewPageLoad(component);
          DOM.removeClass(pageToAnimate, 'activePage');

          if (!React.enableNetworking) {
            React.enableNetworking = true;
            React.unmountComponentAtNode(newPageContainer[0]);
            component = React.render(componentFactory, newPageContainer[0]);
          }
        });

        if (!needsAnimation) {
          onPresentComplete();
          return;
        }

        DOM.addClass(pageToAnimate, 'activePage');
        this._runAnimation(
          pageToAnimate,
          animationProperties,
          {
            // duration: 10000,
            complete: onPresentComplete
          }
        );
      },

      setNetworkAvailable: function setNetworkAvailable(isAvailable) {
        if (currentPage) {
          currentPage.setNetworkAvailable(isAvailable);
        }
      },

      _handle404: function _handle404(url) {
        var startUrl = Session.isLoggedIn() ? '/dashboard' : '/signup';
        if (url !== startUrl) {
          this.openUrl(startUrl);
        }
      },

      didAppEnterForeground: function didAppEnterForeground() {
      },

      didPressBackButton: function didPressBackButton(event) {
        if (this.isPopupOpen()) {
          this.dismissPopup();
          return;
        }

        if (this.isModalOpen()) {
          this.dismissModal();
          return;
        }

        navigator.app.backHistory();
      },

      didAppOpenUrl: function didAppOpenUrl(url) {
        if (url.host === 'app') {
          var url = url.queryParams.url;
          if (url && StringUtils.startsWith(url, '/')) {
            this.openUrl(url);
          }
        }
      },

      startInAppDemo: function startInAppDemo(options) {
        Runtime.isDemo = true;
        Runtime.originalBaseUrl = Runtime.baseUrl;
        Runtime.baseUrl = 'https://demo.buxfer.com';

        Session.clear();
        Session.save(
          {
            demo: true
          },
          options
        );
      },

      endInAppDemo: function endInAppDemo() {
        Session.logout();

        Runtime.isDemo = false;
        Runtime.baseUrl = Runtime.originalBaseUrl;
        delete Runtime.originalBaseUrl;
      }
    };
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIGrid',[
    ''
  ],
  function UIGrid(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var children = [];
        ArrayUtils.each(this.props.children, function(junk, childNode) {
          if (childNode) {
            children.push(childNode);
          }
        });

        var numChildren = children.length;
        var numColumns = this.props.numColumns || numChildren;
        var widths = this.props.widths || [];

        var tableRows = [];
        var columnIndex = 0;
        var rowIndex = 0;
        var currentRowCells = [];
        var numRows = Math.ceil(numChildren / numColumns);
        ArrayUtils.each(children, FunctionUtils.bind(this, function(junk, childNode) {
          var classNames = [];
          if (columnIndex == 0) {
            classNames.push('firstColumn');
          } else if (columnIndex == (numColumns - 1)) {
            classNames.push('lastColumn');
          }

          if (rowIndex == 0) {
            classNames.push('firstRow');
          } else if (rowIndex === numRows - 1) {
            classNames.push('lastRow');
          }

          var cell;
          if (childNode.constructor.displayName == 'UIGridCell') {
            cell = childNode;
            classNames.push(cell.props.className);
          } else {
            cell = React.createElement("td", null, childNode);
          }
          var cellProps = {
            key: (childNode.props && childNode.props.key) || 'cell-'+currentRowCells.length,
            className: classNames.join(' ')
          };

          if (this.props.hasEqualWidthCells) {
            cellProps.width = (100 / numChildren)+'%';
          } else if (widths[columnIndex]) {
            cellProps.width = widths[columnIndex];
          }

          cell = React.cloneElement(cell, cellProps);
          currentRowCells.push(cell);
          ++columnIndex;

          if (columnIndex == numColumns) {
            tableRows.push(
              React.createElement("tr", {key: tableRows.length}, 
                currentRowCells
              )
            );
            columnIndex = 0;
            currentRowCells = [];
            ++rowIndex;
          }
        }));

        if (currentRowCells.length) {
          tableRows.push(
            React.createElement("tr", {key: tableRows.length}, 
            currentRowCells
            )
          );
        }

        var classNames = ['UIGrid'];
        if (this.props.isCentered) {
          classNames.push('centered');
        }
        if (this.props.isFullWidth) {
          classNames.push('fullWidth');
        }

        return (
          React.createElement("table", {className: classNames.join(' ')}, 
            React.createElement("tbody", null, 
              tableRows
            )
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIHorizontalList',[
    'touch/ui/UIGrid',
    ''
  ],
  function UIHorizontalList(
    UIGrid,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement(UIGrid, {numRows: "1"}, 
            this.props.children
          )
        );
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIRadioSelector',[
    'touch/ui/UIHorizontalList',
    ''
  ],
  function UIRadioSelector(
    UIHorizontalList,
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          selectedValue: this.props.value || null
        };
      },

      compose: function compose() {
        ASSERT_TRUTHY(this.props.options, '[UIRadioSelector] Expected non-empty options');

        var cells = [];
        var isSelectedValueValid = false;
        ArrayUtils.each(this.props.options, FunctionUtils.bind(this, function(value, label) {
          var classNames = ['optionContainer'];
          if (value === this.state.selectedValue) {
            classNames.push('selected');
            isSelectedValueValid = true;
          }

          var didClickOption = FunctionUtils.bind(this, function() {
            this.setState({
              selectedValue: value
            });
            if (this.props.onSelect) {
              this.props.onSelect(value);
            }
          });

          cells.push(
            React.createElement("div", {className: classNames.join(' '), onClick: didClickOption}, 
              label
            )
          );
        }));

        var hiddenInput = null;
        if (this.props.name && isSelectedValueValid) {
          hiddenInput = React.createElement("input", {type: "hidden", name: this.props.name, value: this.state.selectedValue});
        }

        return (
          React.createElement("div", {className: "UIRadioSelector"}, 
            React.createElement(UIHorizontalList, {isFullWidth: this.props.isFullWidth, hasEqualWidthCells: true}, 
              cells
            ), 
            hiddenInput
          )
        );
      }


    });


  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UILeftRight',[
    ''
  ],
  function UILeftRight(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var childNodes = this.props.children;
        ASSERT_EQUALS(2, childNodes.length, '[UILeftRight] Expected two child nodes');
        return (
          React.createElement("div", null, 
            React.createElement("div", {className: "lF"}, childNodes[0]), 
            React.createElement("div", {className: "rF"}, childNodes[1]), 
            React.createElement("div", {className: "cF"})
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIButton',[
    ''
  ],
  function UIButton(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var titleContainer =
          React.createElement("a", {className: "controlTitleContainer buttonTitleContainer", href: this.props.href, target: this.props.target}, 
            this.props.title
          );

        return (
          React.createElement("div", null, 
            titleContainer
          )
        );
      },

      getClassNames: function getClassNames() {
        var classNames = ['UIButton UIControl'];
        if (this.props.isDestructive) {
          classNames.push('redButton');
        } else {
          classNames.push('styleDefault');
        }
        classNames.push(this.props.isDisabled ? 'disabled' : 'enabled');
        classNames.push(this.props.isSelected ? 'selected' : 'unselected');
        if (this.props.isFullWidth) {
          classNames.push('fullWidthControl');
        }
        if (
          (this.props.isHoverable || !('isHoverable' in this.props))
          && !this.props.isDestructive
        ) {
          classNames.push('hoverable');
        }
        if (this.props.type === 'submit') {
          classNames.push('submitButton');
        }
        return classNames;
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIPopup',[
    'touch/app/NavigationController',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIButton',
    ''
  ],
  function UIPopup(
    AppNavigationController,
    UIHorizontalList,
    UIButton,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        ASSERT_TRUTHY(this.props.title, '[UIPopup] Expected non-empty title');

        return (
          React.createElement("div", {className: "UIPopup"}, 
            React.createElement(UIHorizontalList, {widths: [1], className: "headerContainer", isFullWidth: true}, 
              React.createElement("div", {className: "closeIconContainer", onClick: this._didClickClose}), 
              React.createElement("div", {className: "titleContainer"}, 
                this.props.title
              )
            ), 
            React.createElement("div", {className: "contentContainer"}, 
              this.props.children
            ), 
            this._maybeRenderCloseButton()
          )
        );
      },

      _maybeRenderCloseButton: function _maybeRenderCloseButton() {
        if (!this.props.showCloseButton) {
          return null;
        }

        return (
          React.createElement(UIButton, {
            className: "closeButton", 
            isFullWidth: true, 
            isHoverable: false, 
            onClick: this._didClickClose, 
            title: UIStaticText('CLOSE')}
          )
        );
      },

      _didClickClose: function _didClickClose() {
        AppNavigationController.dismissPopup();
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/errors/UIPopup',[
    'touch/ui/UIPopup',
    ''
  ],
  function UIErrorPopup(
    UIPopup,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement(UIPopup, {className: "UIErrorPopup", title: UIStaticText('ERROR'), showCloseButton: true}, 
            this.props.message
          )
        );
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/errors/Manager',[
    'touch/errors/UIPopup',
    'touch/core/Model',
    ''
  ],
  function ErrorManager(
    UIErrorPopup,
    Model,
    junk
  ) {

    var SYS_ERROR_MESSAGE = UIStaticText("Sorry, the system encountered an error. Please try again later");
    var NETWORK_ERROR_MESSAGE = UIStaticText("A network error occurred. Please check your internet connection and try again.");

    return {
    
      init: function init() {
        setupWindowErrorHandler(this);
      },

      handleAPIError: function handleAPIError(response) {
        var errorMessage;
        if (!response || !response.error) {
          errorMessage = NETWORK_ERROR_MESSAGE;
        } else {
          errorMessage = response.error.message;
        }
        this.renderError(errorMessage);
      },

      renderUserError: function renderUserError(message) {
        this.renderError(message);
      },

      logException: function logException(e, message, options) {
        options = options || {};
        this.logError(message, Object.extend(options, {
          exception: e
        }));
      },

      handleException: function handleException(e, message, options) {
        options = options || {};
        this.handleSystemError(message, Object.extend(options, {
          exception: e,
          isSystemError: true
        }));
      },

      handleSystemError: function handleSystemError(message, options) {
        this.renderSystemError();
        this.logError(message, options);
      },

      renderSystemError: function renderSystemError() {
        this.renderError(SYS_ERROR_MESSAGE);
      },

      renderError: function renderError(message) {
        AppNavigationController.presentPopup(
          React.createElement(UIErrorPopup, {message: message})
        );
      },

      logError: function logError(message, options) {
        options = options || {};
        var exception = options.exception;

        if (exception) {
          message += ' ['+exception.message+']';
        }

        var trace = '';
        if (options.trace) {
          trace = options.trace;
        } else if (exception && exception.stack) {
          trace = exception.stack;
          trace = this._prettifyTrace(trace);
        } else {
          trace = getCurrentStackTrace();
        }

        if (window.console) {
          console.warn(message);
          console.warn(trace);
        }

        var error = Model.create({
          url: '/apix/error'
        });
        error.set({
          message: message,
          isSystemError: options.isSystemError || false,
          currentPageUrl: AppNavigationController.getCurrentUrl(),
          trace: trace
        });
        error.save();
      },

      _prettifyTrace: function _prettifyTrace(trace) {
        // file:///Users/shashank/Library/Developer/CoreSimulator/Devices/F912C47F-2F18-4C6D-8045-E52A3C646603/data/Containers/Bundle/Application/276E78F3-5829-4DFA-A821-AADC1DD903E1/Buxfer.app
        return trace.replace(/file:\/\/\/.*\/Buxfer.app/g, '');
      }
    };
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIForm',[
    'touch/core/Model',
    'touch/errors/Manager',
    ''
  ],
  function UIForm(
    Model,
    ErrorManager,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement("form", {className: "UIForm", onSubmit: this._didSubmitForm, onKeyPress: this._didKeyPress, onClick: this._didClickForm}, 
            this.props.children
          )
        );
      },

      _didClickForm: function _didClickForm(event) {
        var button = DOM.maybeFindParentButton(event.target);
        if (
          !button
          || DOM.hasClass(button, 'disabled')
          || !DOM.isSubmitButton(button)
        ) {
          return;
        }

        this.loadingElement = button;
        this.submit();
      },

      _didKeyPress: function _didKeyPress(event) {
        if (DOM.hasClass(event.target, 'disabled')) {
          return;
        }

        if (event.charCode !== Key.ENTER || !DOM.shouldSubmitOnEnter(event.target)) {
          return;
        }

        this.loadingElement = event.target;
        this.submit();
      },

      _didSubmitForm: function _didSubmitForm(event) {
        this.submit();
        event.stopPropagation();
        event.preventDefault();
      },

      submit: function submit() {
        if (this.props.shouldSubmit && !this.props.shouldSubmit()) {
          return;
        }

        if (this.state.isLoading) {
          return;
        }

        this.setState({
          isLoading: true
        });
        if (this.loadingElement) {
          DOM.addClass(this.loadingElement, 'disabled');
          DOM.setLoading(this.loadingElement, true);
        }

        var model = this.props.model;
        if (!model) {
          ASSERT_TRUTHY(this.props.action, '[UIForm] Expected non-empty model or action');
          model = Model.create({
            url: this.props.action
          });
        }

        var formParams = this._getParams();
        model.save(formParams, {
          success: FunctionUtils.bind(this, this._onSubmitSuccess),
          error: FunctionUtils.bind(this, this._onSubmitError)
        });

        this._notifyDelegate('formWillSubmit', [formParams]);
      },

      _notifyDelegate: function _notifyDelegate(eventName, args) {
        if (!this.props.delegate || !this.props.delegate[eventName]) {
          return;
        }
        args = ArrayUtils.prepend(args, this);
        return this.props.delegate[eventName].apply(this.props.delegate, args);
      },

      _onSubmitSuccess: function _onSubmitSuccess(model, request, options) {
        this._onRequestComplete();
        this._notifyDelegate('formDidSubmit', [model]);
      },

      _onSubmitError: function _onSubmitError(model, request, options) {
        this._onRequestComplete();
        var delegateResponse = this._notifyDelegate('formDidFailSubmit', [model, request.responseJSON]);

        if (delegateResponse === EventDelegater.STOP) {
          return;
        }

        ErrorManager.handleAPIError(request.responseJSON);
      },

      _onRequestComplete: function _onRequestComplete() {
        this.setState({
          isLoading: false
        });
        if (this.loadingElement) {
          DOM.setLoading(this.loadingElement, false);
          DOM.removeClass(this.loadingElement, 'disabled');
          this.loadingElement = null;
        }
      },

      _getParams: function _getParams() {
        var params = {};
        var form = DOM.findById(this.getDOMNode());
        ArrayUtils.each(['textarea', 'input', 'select', '.UICheckbox'], function(key, tagName) {
          var inputs = form.find(tagName);
          ArrayUtils.each(inputs, function(key, input) {
            input = DOM.findById(input);
            var inputName = DOM.getAttribute(input, 'name');

            if (!inputName || inputName.match (/^__/)) {
              return true;
            }

            var type = DOM.getAttribute(input, 'type');
            if (type == 'button' || type == 'submit' || (type == "radio" && !input.prop('checked'))) {
              return true;
            }

            var inputValue;
            if (DOM.isCheckbox(input)) {
              inputValue = DOM.isCheckboxSelected(input) ? "on" : "off";
            } else {
              inputValue = input.val();
            }

            params[inputName] = inputValue;
          });
        });
        return params;
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/auth/UIThirdPartyLogin',[
    'touch/app/Runtime',
    'touch/ui/UIHorizontalList',
    'touch/auth/Session',
    ''
  ],
  function UIThirdPartyLogin(
    Runtime,
    UIHorizontalList,
    Session,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var sources = [
          'google',
          'facebook',
          'microsoft',
          'yahoo'
        ];
        var cells = [];
        ArrayUtils.each(sources, FunctionUtils.bind(this, function(index, source) {
          var url = this._getThirdPartyLoginUrlForSource(source);
          if (Runtime.isApp) {
            cells.push(
              React.createElement("a", {className: 'thirdPartyButton '+source, onClick: FunctionUtils.bind(this, this._didClickThirdPartyLoginButton, url)})
            );
          } else {
            cells.push(
              React.createElement("a", {className: 'thirdPartyButton '+source, href: url, target: "_top"})
            );
          }
        }));

        return (
          React.createElement("div", {className: "UIThirdPartyLogin"}, 
            React.createElement(UIHorizontalList, {className: "thirdPartyButtonsContainer", isFullWidth: true, hasEqualWidthCells: true}, 
              cells
            )
          )
        );
      },

      _getThirdPartyLoginUrlForSource: function _getThirdPartyLoginUrlForSource(source) {
        var url = Url.createWithPath('/login/thirdparty');
        url.addQueryParam('source', source);

        if (Runtime.isApp) {
          url.addQueryParam('flow', 'app');
          url = AppConfig.baseUrl + url.toString();
        }

        return url;
      },

      _didClickThirdPartyLoginButton: function _didClickThirdPartyLoginButton(url) {
        var authWindow = window.open(url, '_blank', 'location=no,toolbar=yes,closebuttoncaption=Cancel,toolbarposition=top');
        $(authWindow).on('loadstart', FunctionUtils.bind(this, this._didLoadStart, authWindow));
      },

      _didLoadStart: function _didLoadStart(authWindow, e) {
        var url = e.originalEvent.url;

        var parsedUrl = Url.parse(url);

        if (parsedUrl.host !== 'localhost') {
          return;
        }

        authWindow.close();
        var token = ArrayUtils.get(parsedUrl.queryParams, 'token');

        var _onSuccess = FunctionUtils.bind(this, function(model) {
          Session.trigger('refresh');
          AppNavigationController.openUrl('/dashboard');
        });

        var _onError = FunctionUtils.bind(this, function(model, request) {
          ErrorManager.handleAPIError(request);
          AppNavigationController.openUrl('/signup');
        });

        Session.clear();
        Session.save(
          {
            thirdpartyToken: token
          },
          {
            success: _onSuccess,
            error: _onError
          }
        );
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/auth/UIAuth',[
    'touch/app/Runtime',
    'touch/core/Model',
    'touch/ui/UIRadioSelector',
    'touch/ui/UILeftRight',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIForm',
    'touch/ui/UIButton',
    'touch/auth/UIThirdPartyLogin',
    'touch/auth/Session',
    'touch/app/NavigationController',
    ''
  ],
  function UILogin(
    Runtime,
    Model,
    UIRadioSelector,
    UILeftRight,
    UIHorizontalList,
    UIForm,
    UIButton,
    UIThirdPartyLogin,
    Session,
    AppNavigationController,
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          mode: this.props.mode || 'signup'
        };
      },

      compose: function compose() {
        var formTitle, authModel, emailLabel, passwordLabel, showForgotPasswordLink = false, submitButtonLabel;
        switch (this.state.mode) {
          case 'login':
            formTitle = UIStaticText('LOGIN TO YOUR ACCOUNT');
            authModel = Session;
            emailLabel = UIStaticText('EMAIL ADDRESS');
            passwordLabel = UIStaticText('PASSWORD');
            // showForgotPasswordLink = true;
            submitButtonLabel = UIStaticText('LOGIN');
            break;

          case 'signup':
            formTitle = UIStaticText('CREATE A FREE ACCOUNT');
            authModel = Model.create({
              url: '/apix/user'
            });
            emailLabel = UIStaticText('ENTER EMAIL ADDRESS');
            passwordLabel = UIStaticText('CREATE PASSWORD');
            submitButtonLabel = UIStaticText('CREATE ACCOUNT');
            break;

          default:
            BUG('Invalid auth mode ['+this.state.mode+']');
        }

        var forgotLink;
        if (showForgotPasswordLink) {
          forgotLink = (
            React.createElement("a", {className: "bgText smallFont", href: "/forgotpassword", target: "_top", tabIndex: "-1"}, 
              UIStaticText('FORGOT?')
            )
            );
        }

        return (
          React.createElement("div", {className: "UIAuth"}, 
            React.createElement("div", {className: "UISection padded"}, 
              React.createElement("div", {className: "smallFont bFontWeight"}, 
                UIStaticText('START WITH AN EXISTING ACCOUNT')
              ), 
              React.createElement("div", {className: "ttMargin"}, 
                React.createElement(UIThirdPartyLogin, null)
              )
            ), 
            this._maybeRenderDemoButton(), 
            React.createElement("div", {className: "smallFont bgText cAlign bbMargin"}, 
              UIStaticText('OR')
            ), 
            React.createElement("div", {className: "UISection padded"}, 
              this._renderModeSelector(), 
              React.createElement("div", {className: "smallFont tttMargin bFontWeight"}, 
                formTitle
              ), 
              React.createElement(UIForm, {model: authModel, delegate: this}, 
                React.createElement("div", {className: "tttMargin"}, 
                  emailLabel
                ), 
                React.createElement("div", {className: "tMargin"}, 
                  React.createElement("input", {className: "fullWidth", type: "email", name: "email"})
                ), 
                React.createElement(UILeftRight, {className: "tttMargin"}, 
                  React.createElement("div", null, 
                    passwordLabel
                  ), 
                  forgotLink
                ), 
                React.createElement("div", {className: "tMargin"}, 
                  React.createElement("input", {className: "fullWidth", type: "password", name: "password"})
                ), 
                this._maybeRenderRememberInput(), 
                React.createElement(UIButton, {
                  isHoverable: false, 
                  className: "ttMargin submitButton", 
                  title: submitButtonLabel, 
                  isFullWidth: true, 
                  type: "submit"}
                )
              ), 
              this._maybeRenderLegalese()
            )
          )
        );
      },

      _maybeRenderDemoButton: function _maybeRenderDemoButton() {
        if (Runtime.isHarvest) {
          return;
        }

        var demoButton;
        if (Runtime.isApp) {
          demoButton = (
            React.createElement(UIButton, {
              isHoverable: false, 
              className: "demoButton", 
              title: UIStaticText('TRY A DEMO'), 
              isFullWidth: true, 
              onClick: this._didClickDemoButton}
            )
          );
        } else {
          demoButton = (
            React.createElement(UIButton, {
              isHoverable: false, 
              href: "/demo", 
              className: "demoButton", 
              title: UIStaticText('TRY A DEMO'), 
              isFullWidth: true, 
              target: "_top"}
            )
          );
        }

        return (
          React.createElement("div", null, 
            React.createElement("div", {className: "smallFont bgText cAlign bbMargin"}, 
              UIStaticText('OR')
            ), 
            React.createElement("div", {className: "UISection padded"}, 
              demoButton
            )
          )
        );
      },

      _didClickDemoButton: function _didClickDemoButton(event) {
        DOM.setLoading(event.target, true);
        AppNavigationController.startInAppDemo({
          success: this._finishLogin
        });
      },

      _renderModeSelector: function _renderModeSelector() {
        var options = {
          signup: UIStaticText('NEW USER'),
          login: UIStaticText('EXISTING USER')
        };
        return React.createElement(UIRadioSelector, {value: this.state.mode, isFullWidth: true, name: "authMode", options: options, onSelect: this._didSelectMode});
      },

      _maybeRenderRememberInput: function _maybeRenderRememberInput() {
        if (Runtime.isApp || true) {
          return React.createElement("input", {type: "hidden", name: "remember", value: "on"});
        }

        return (
          React.createElement("div", {className: "ttMargin"}, 
            React.createElement("input", {type: "checkbox", id: "remember-me-checkbox", name: "remember", checked: "true"}), 
            React.createElement("label", {className: "llMargin smallFont bgText clickable noWrap", htmlFor: "remember-me-checkbox"}, 
              UIStaticText('STAY LOGGED IN')
            )
          )
        );
      },

      _maybeRenderLegalese: function _maybeRenderLegalese() {
        if (this.state.mode !== 'signup') {
          return;
        }

        return (
          React.createElement("div", {className: "legaleseContainer"}, 
            UIStaticText(
              'By clicking \'Create Account\', you are indicating that you have read and agree to the '
            ), 
            React.createElement("a", {className: "hoverable", target: "_blank", href: "/tos"}, 
              UIStaticText('Terms of Service')
            ), 
            UIStaticText(' and '), 
            React.createElement("a", {className: "hoverable", target: "_blank", href: "/privacy"}, 
              UIStaticText('Privacy Policy')
            )
          )
          );
      },

      _didSelectMode: function _didSelectMode(mode) {
        this.setState({
          mode: mode
        });
      },

      formDidSubmit: function formDidSubmit(form, model) {
        this._finishLogin(model);
      },

      _finishLogin: function _finishLogin(model) {
        delete model.password;
        delete model.remember;

        Session.set(model.attributes);
        Session.trigger('refresh');
        AppNavigationController.openUrl('/dashboard');
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/categories/UINavigation',[
    'touch/core/Model',
    'touch/ui/UIHorizontalList',
    ''
  ],
  function UICategoryNavigation(
    Model,
    UIHorizontalList,
    junk
  ) {

    var CategoryNavigationModel = Model.createClass({

      url: function url() {
        var url = Url.createWithPath('/apix/categories/navigation');
        url.addQueryParam('tagId', this.attributes.tagId);
        return url;
      }

    });

    return React.createClass({

      getModelsToFetch: function getModelsToFetch() {
        if (!this.props.tagId) {
          return {};
        }

        return {
          navigation: new CategoryNavigationModel({
            tagId: this.props.tagId
          })
        };
      },

      compose: function compose() {
        if (!this.props.tagId) {
          return (
            React.createElement("div", {className: "UICategoryNavigation"}, 
              React.createElement("div", {className: "categoryLabel"}, UIStaticText('All tags'))
            )
          );
        }

        if (!this.state.models.navigation) {
          return React.createElement("div", null);
        }

        var url = this.props.href;
        var cells = [];

        url.addQueryParam('tags', null);
        cells.push(
          React.createElement("a", {className: "categoryLabel", href: url.toString()}, UIStaticText('All'))
        );

        var tags = this.state.models.navigation.attributes.tags;
        if (tags.length > 2) {
          tags = tags.slice(-2);
          cells.push(
            React.createElement("span", null, 
              React.createElement("span", {className: "categorySeparator"}), 
              React.createElement("span", {className: "categoryLabel"}, "...")
            )
          );
        }
        var numTags = tags.length;
        ArrayUtils.each(tags, FunctionUtils.bind(this, function(index, tag) {
          var classNames = ['categoryLabel'];
          if (index == numTags - 1) {
            classNames.push('selectedCategory');
          }
          classNames = classNames.join(' ');
        
          url.addQueryParam('tags', tag.id);
          cells.push(
            React.createElement("div", {className: "categorySeparator"}, " ")
          );
          cells.push(
            React.createElement("a", {href: url.toString(), className: classNames}, tag.name)
          );
        }));

        return (
          React.createElement("div", {className: "UICategoryNavigation"}, 
            React.createElement(UIHorizontalList, null, 
              cells
            )
          )
        );
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIPicker',[
    'touch/ui/UIHorizontalList',
    ''
  ],
  function UIPicker(
    UIHorizontalList,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement(UIHorizontalList, {className: "UIPicker"}, 
            React.createElement("div", {className: "contentContainer"}, 
              this.props.children
            ), 
            React.createElement("div", {className: "arrowContainer"}, " ")
          )
        );
      }
    });

  }
);



define(
  'touch/core/Period',[
    ''
  ],
  function Period(
    junk
  ) {

    var periodDescriptions = {
      'day_last_30': UIStaticText('Last 30 Days'),
      'week_current': UIStaticText('This Week'),
      'month_last': UIStaticText('Last Month'),
      'month_last_3': UIStaticText('Last 3 Months'),
      'month_last_12': UIStaticText('Last 12 Months'),
      'month_current': UIStaticText('This Month'),
      'month_next_3': UIStaticText('Next 3 Months'),
      'month_next_6': UIStaticText('Next 6 Months'),
      'year_last': UIStaticText('Last Year'),
      'year_next': UIStaticText('Next Year'),
      'year_current': UIStaticText('This Year')
    };

    return {

      getDescriptionForType: function getDescriptionForType(periodType) {
        return ASSERT_TRUTHY(periodDescriptions[periodType], 'Invalid period type ['+periodType+']');
      }
    };


  }
);

/** @jsx React.DOM */


define(
  'touch/app/UISectionRow',[
    ''
  ],
  function UISectionRow(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var children = ArrayUtils.toArray(this.props.children);

        if (this.props.isLoading) {
          var loadingIndicator = React.createElement("div", {key: "loading-indicator", className: "sectionRowLoadingContainer"}, " ");
          children = ArrayUtils.prepend(children, loadingIndicator);
        }

        var isClickable = this.props.href || this.props.isClickable || this.props.onClick;

        if (!isClickable) {
          return (
            React.createElement("div", {className: "UISectionRow"}, 
              children
            )
          );
        }

        if (this.props.isNavigable) {
          children = (
            React.createElement("div", {className: "navigableContentContainer"}, 
              children
            )
          );
        }

        return (
          React.createElement("a", {className: "UISectionRow block", href: this.props.href}, 
            children
          )
        );
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIPeriodPicker',[
    'touch/app/NavigationController',
    'touch/ui/UIPopup',
    'touch/ui/UIPicker',
    'touch/core/Period',
    'touch/app/UISectionRow',
    ''
  ],
  function UIPeriodPicker(
    AppNavigationController,
    UIPopup,
    UIPicker,
    Period,
    UISectionRow,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement(UIPicker, {className: "UIPeriodPicker clickable", onClick: this._didClickPicker}, 
            this._renderPeriodDescription()
          )
        );
      },

      _didClickPicker: function _didClickPicker() {
        AppNavigationController.presentPopup(this._renderPopup());
      },

      _renderPopup: function _renderPopup() {
        var cells = [];
        var periodTypes;
        if (this.props.future) {
          periodTypes = [
            'month_next_3',
            'month_next_6',
            'year_next'
          ];
        } else {
          periodTypes = [
            'week_current',
            'month_current',
            'month_last',
            'year_current',
            'year_last'
          ];
        }
        ArrayUtils.each(periodTypes, FunctionUtils.bind(this, function(index, periodType) {
          var description = Period.getDescriptionForType(periodType);
          var didSelectPeriodType = FunctionUtils.bind(this, function() {
            this.props.onSelect({
              periodType: periodType
            });
          });
          var classNames = [];
          if (periodType === this.props.filters.periodType) {
            classNames.push('selected');
          }
          cells.push(
            React.createElement(UISectionRow, {className: classNames.join(' '), onClick: didSelectPeriodType, key: 'period-'+periodType}, 
              description
            )
            );
        }));

        return (
          React.createElement(UIPopup, {title: UIStaticText('SELECT PERIOD')}, 
            cells
          )
        );
      },

      _renderPeriodDescription: function _renderPeriodDescription() {
        var filters = this.props.filters;

        if (filters.periodType) {
          return Period.getDescriptionForType(filters.periodType);
        }

        return UIStaticText('All Time');
      }

    });

  }
);



define(
  'touch/accounts/Model',[
    'touch/core/Model',
    ''
  ],
  function AccountModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/account?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        return '/account?id='+this.id;
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/accounts/picker/UIPicker',[
    'touch/ui/UIPicker',
    'touch/accounts/Model',
    'touch/app/NavigationController',
    'touch/ui/UIPopup',
    ''
  ],
  function UIAccountPicker(
    UIPicker,
    AccountModel,
    AppNavigationController,
    UIPopup,
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          accountId: this.props.accountId
        };
      },

      getModelsToFetch: function getModelsToFetch() {
        if (!this.state.accountId) {
          return {};
        }

        return {
          account: new AccountModel({
            id: this.state.accountId
          })
        };
      },

      compose: function compose() {
        var child = null;

        if (this.state.accountId === 0) {
          child = UIStaticText('No Account');
        } else if (!this.state.accountId) { // null or undefined
          child = UIStaticText('All Accounts');
        } else if (this.state.models.account) {
          child = this.state.models.account.attributes.name;
        } else {
          child = UIStaticText('Loading ...');
        }

        return (
          React.createElement(UIPicker, {className: "UIAccountPicker", onClick: this._didClickPicker}, 
            child, 
            React.createElement("input", {type: "hidden", name: "accountId", value: this.state.accountId})
          )
        );
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('SELECT ACCOUNT');
      },

      _didClickPicker: function _didClickPicker() {
        var accountId = this.state.accountId;
        var didSelectAccount = FunctionUtils.bind(this, this._didSelectAccount);
        var enableAllAccountsOption = this.props.enableAllAccountsOption;
        // Avoid circular dependency between UIPage and UIAccountPickerList
        require(
          ['touch/accounts/picker/UIPage'],
          function (UIAccountPickerListPage) {
            AppNavigationController.presentModal(
              React.createElement(UIAccountPickerListPage, {accountId: accountId, enableAllAccountsOption: enableAllAccountsOption, onSelect: didSelectAccount})
            );
          }
        );
      },

      _didSelectAccount: function _didSelectAccount(account) {
        this.setState({
          accountId: account.id,
          models: {
            account: account
          }
        });
        this.props.onSelect(account);
      }

    });
  }
);



define(
  'touch/membership/Feature',[
    ''
  ],
  function Feature(
    junk
  ) {

    return {
      FORECAST: 'forecast',
      TIMELINE: 'timeline',

      getName: function getName(feature) {
        var featureNames = {};
        featureNames[this.FORECAST] = 'Forecast';
        featureNames[this.TIMELINE] = 'Timeline';
        featureNames[this.PAYMENTS] = 'Payments';
        featureNames[this.DEALS] = 'Deals';
        featureNames[this.ADVANCED_REPORTS] = 'Advanced Reports';
        featureNames[this.TOP_MOVERS] = 'Top Movers';
        featureNames[this.ALLOCATION] = 'Allocation';
        featureNames[this.AUTOMATIC_BACKUP] = 'Automatic Backup'
        return ASSERT_TRUTHY(featureNames[feature], 'Invalid feature ['+feature+']');
      }
    };
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIList',[
    ''
  ],
  function UIList(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var childNodes = [];
        ArrayUtils.each(this.props.children, function(index, child) {
          childNodes.push(
            React.createElement("li", {key: 'list-'+index}, child)
          );
        });
        return React.createElement("ul", {className: "UIList square"}, childNodes)
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/membership/UIBlock',[
    'touch/app/NavigationController',
    'touch/membership/Feature',
    'touch/ui/UIList',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIButton',
    ''
  ],
  function UIMembershipBlock(
    AppNavigationController,
    Feature,
    UIList,
    UIHorizontalList,
    UIButton,
    junk
  ) {

    return React.createClass({
      compose: function compose() {
        ASSERT_TRUTHY(this.props.feature, '[UIMembershipUpsell] Expected non-empty feature');
        var featureName = Feature.getName(this.props.feature);

        var referer = this.props.feature.toLowerCase()+'_upsell';
        var paymentUrl = Url.createWithPath('/membership/payment');
        paymentUrl.addQueryParam('ref', referer);

        var plansUrl = Url.createWithPath('/membership/plans');
        plansUrl.addQueryParam('ref', referer);

        return (
          React.createElement("div", {className: "UIMembershipBlock UISection padded"}, 
            React.createElement("div", {className: "titleContainer"}, 
              this._renderTitle()
            ), 
            React.createElement("div", {className: "descriptionContainer"}, 
              this._renderShortDescription()
            ), 
            React.createElement("div", {className: "bigTMargin"}, 
              UIStaticText('To try '+featureName+', please upgrade to a PRO plan.')
            ), 
            React.createElement(UIHorizontalList, {className: "actionsContainer"}, 
              React.createElement(UIButton, {title: UIStaticText('UPGRADE TO PRO'), href: paymentUrl.toString()}), 
              React.createElement("a", {className: "seeAllPlansLink", href: plansUrl}, UIStaticText('SEE ALL PLANS'))
            )
          )
        );
      },

      _renderTitle: function _renderTitle() {
        if (this.props.feature === Feature.FORECAST) {
          return UIStaticText(
            'Use Forecast to save for your future goals'
          );
        }

        if (this.props.feature === Feature.TIMELINE) {
          return UIStaticText(
            'Use Timeline to easily spot unusual spending'
          );
        }

        featureName = Feature.getName(this.props.feature);
        return UIStaticText('Try '.featureName);
      },

      _renderShortDescription: function _renderShortDescription() {
        if (this.props.feature === Feature.FORECAST) {
          return (
            React.createElement(UIList, null, 
            UIStaticText('Plan for upcoming expenses such as a vacation, buying a new house or a wedding.'), 
            UIStaticText('Preview how your account balances are going to change with time.'), 
            UIStaticText('Avoid overdraft fees.'), 
            UIStaticText('Avoid going over your credit card limit.')
            )
          );
        }

        if (this.props.feature === Feature.TIMELINE) {
          return (
            React.createElement(UIList, null, 
            UIStaticText('Visually compares day-to-day spending with the last month.'), 
            UIStaticText('Easily spot deviations from regular spending patterns.'), 
            UIStaticText('Predict if you are on track to stay within your budget limit.')
            )
          );
        }

        BUG('Not implemented');
      },

      _renderDescription: function _renderDescription() {
        if (this.props.feature === Feature.FORECAST) {
          return (
            React.createElement("div", null, 
            UIStaticText(
              'Forecast estimates your future income and expenses based on your past spending, upcoming transactions, bill reminders and budgets.'
            ), 
            React.createElement("div", {className: "ttMargin"}, 
            UIStaticText('With Forecast, you can:')
            ), 
            React.createElement(UIList, null, 
            UIStaticText('Plan for upcoming expenses such as a vacation, buying a new house or a wedding.'), 
            UIStaticText('Preview how your account balances are going to change with time.'), 
            UIStaticText('Avoid overdraft fees.'), 
            UIStaticText('Avoid going over your credit card limit.')
            )
            )
          );
        }

        BUG('Not implemented');
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/membership/UIUpsell',[
    'touch/app/NavigationController',
    'touch/membership/Feature',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIButton',
    ''
  ],
  function UIMembershipUpsell(
    AppNavigationController,
    Feature,
    UIHorizontalList,
    UIButton,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        ASSERT_TRUTHY(this.props.feature, '[UIMembershipUpsell] Expected non-empty feature');
        ASSERT_TRUTHY(this.props.numPreviewDaysRemaining, '[UIMembershipUpsell] Expected non-empty preview days remaining');

        var referer = this.props.feature.toLowerCase()+'_upsell';
        var paymentUrl = Url.createWithPath('/membership/payment');
        paymentUrl.addQueryParam('ref', referer);

        var plansUrl = Url.createWithPath('/membership/plans');
        plansUrl.addQueryParam('ref', referer);

        return (
          React.createElement("div", {className: "UIMembershipUpsell aaPadding"}, 
            React.createElement("div", {className: "noticeTextContainer"}, 
              this._renderNotice()
            ), 
            React.createElement(UIHorizontalList, {className: "fullWidth ttMargin"}, 
              React.createElement(UIButton, {className: "upgradeButton", title: UIStaticText('UPGRADE TO PRO'), href: paymentUrl.toString()}), 
              React.createElement("a", {className: "seeAllPlansLink", href: plansUrl.toString()}, UIStaticText('SEE ALL PLANS'))
            )
          )
        );
      },

      _renderNotice: function _renderNotice() {
        var featureName = Feature.getName(this.props.feature);
        var s = this.props.numPreviewDaysRemaining > 1 ? 's' : '';
        return UIStaticText(
          featureName+' needs a PRO membership. This free preview is available to you for '
          +'the next '+this.props.numPreviewDaysRemaining+' day'+s+' only.'
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIText',[
    ''
  ],
  function UIText(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var text = ASSERT_STRING(this.props.children);

        // TODO(touch): This could be improved
        if (this.props.maxLength && this.props.maxLength > 0 && text.length > this.props.maxLength) {
          text = text.substr(0, this.props.maxLength)+'...';
        }

        return (
          React.createElement("span", null, text)
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/input/UIButton',[
    ''
  ],
  function UITransactionInputButton(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        // Using a UIButton here causes strange layout issues on Chrome Android
        return (
          React.createElement("a", {className: "block layerDetail UITransactionInputButton", href: "/transaction/add"}, 
            UIStaticText('ADD TRANSACTION')
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/app/UIPage',[
    'touch/categories/UINavigation',
    'touch/ui/UIPeriodPicker',
    'touch/ui/UIHorizontalList',
    'touch/accounts/picker/UIPicker',
    'touch/app/NavigationController',
    'touch/auth/Session',
    'touch/app/Runtime',
    'touch/ui/UIButton',
    'touch/membership/UIBlock',
    'touch/membership/UIUpsell',
    'touch/ui/UIText',
    'touch/transactions/input/UIButton',
    ''
  ],
  function UIAppPage(
    UICategoryNavigation,
    UIPeriodPicker,
    UIHorizontalList,
    UIAccountPicker,
    AppNavigationController,
    Session,
    Runtime,
    UIButton,
    UIMembershipBlock,
    UIMembershipUpsell,
    UIText,
    UITransactionInputButton,
    junk
  ) {

    return {

      getPageClasses: function getPageClasses() {
        return [];
      },

      hasTopLevelAccessControl: function hasTopLevelAccessControl() {
        return false;
      },

      canWorkOffline: function canWorkOffline() {
        return false;
      },

      compose: function compose() {
        var classNames = this.getPageClasses();
        classNames.push('UIAppPage');

        if (this.props.presentationMethod === 'modal') {
          classNames.push('modal');
        } else if (this.props.navigationCategory) {
          classNames.push(this.props.navigationCategory+'Page');
        }

        var accessControl = this.state.accessControl || {};
        var content;
        if (Runtime.isOffline && !Runtime.canWorkOffline()) {
          content = this._renderNetworkUnavailableContent();
        } else if (this.hasTopLevelAccessControl() && accessControl.block) {
          content = React.createElement(UIMembershipBlock, {feature: accessControl.feature});
        } else {
          content = this.renderPageContent();
          var filtersUI = this._maybeRenderFilters();
          var upsellUI = null;
          if (this.hasTopLevelAccessControl() && accessControl.preview) {
              upsellUI = React.createElement(UIMembershipUpsell, {className: "pageNotice", feature: accessControl.feature, numPreviewDaysRemaining: accessControl.numPreviewDaysRemaining});
          }
          if (filtersUI || upsellUI) {
            content = (
              React.createElement("div", null, 
                upsellUI, 
                filtersUI, 
                content
              )
            );
          }
        }

        return (
          React.createElement("div", {className: classNames.join(' ')}, 
            this._renderHeader(), 
            this._maybeRenderDemoNotice(), 
            content, 
            this._maybeRenderFixedActions()
          )
        );
      },

      _maybeRenderFixedActions: function _maybeRenderFixedActions() {
        if (!Session.isLoggedIn() || this.props.presentationMethod === 'modal') {
          return null;
        }
        
        var actionsUI = this.renderFixedActions();

        if (!actionsUI) {
          return actionsUI;
        }

        return React.createElement("div", {className: "fixedActionsContainer"}, actionsUI);
      },

      renderFixedActions: function renderFixedActions() {
        return React.createElement(UITransactionInputButton, null);
      },

      getPageContentCards: function getPageContentCards() {
        return [];
      },

      renderPageContent: function renderPageContent() {
        var cards = this.getPageContentCards();
        ASSERT_TRUE(cards.length > 0, '[UIAppPage] Expected non-empty cards');

        var contentNodes = [];
        ArrayUtils.each(cards, FunctionUtils.bind(this, function(index, card) {
          contentNodes.push(React.cloneElement(card, {
            key: 'card-'+index,
            delegate: this,
            hideAccessControl: this.hasTopLevelAccessControl()
          }));
        }));
        return React.createElement("div", null, contentNodes);
      },

      componentDidUpdateAccessControl: function componentDidUpdateAccessControl(component, accessControl) {
        this.setState({
          accessControl: accessControl
        });
      },

      _renderPageLoadingIndicator: function _renderPageLoadingIndicator() {
        return (
          React.createElement("div", {className: "pageLoadingIndicator smallFont bFontWeight"}, 
            UIStaticText('LOADING')
          )
        );
      },

      _renderNetworkUnavailableContent: function _renderNetworkUnavailableContent() {
         return (
           React.createElement("div", {className: "networkErrorContainer"}, 
             React.createElement("div", {className: "bigFont"}, 
               UIStaticText('We could not fetch your data')
             ), 
             React.createElement("div", {className: "ttMargin bgText"}, 
               UIStaticText('Please check your internet connection.')
             ), 
             React.createElement(UIButton, {
               className: "bigTMargin", 
               isFullWidth: true, 
               onClick: this._didRetryNetworkError, 
               title: UIStaticText('TRY AGAIN')}
             )
           )
         );
      },

      _didRetryNetworkError: function _didRetryNetworkError() {
        AppNavigationController.reloadPage();
      },

      componentDidUpdate: function componentDidUpdate() {
        if (this.hasDynamicPageTitle && this.hasDynamicPageTitle()) {
          document.title = this.getBrowserPageTitle();
        }
      },

      _renderHeader: function _renderHeader() {
        ASSERT_TRUTHY(this.renderPageTitle, '[UIAppPage] Missing definition for `renderPageTitle`');
        var titleUI = this.renderPageTitle();
        if (Object.isString(titleUI)) {
          titleUI = React.createElement(UIText, {maxLength: "20"}, titleUI)
        }

        var classNames = ['navIconContainer clickable'];
        if (this.props.navigationDepth > 0) {
          classNames.push('backButton');
        } else {
          classNames.push('menuButton');
        }

        return (
          React.createElement("div", {className: "UIAppHeader"}, 
            React.createElement("div", {className: classNames.join(' '), onClick: this._didClickHeaderMenu}), 
            React.createElement("div", {className: "actionsContainer"}, 
              this._maybeRenderActions()
            ), 
            React.createElement("div", {className: "titleContainer"}, 
              titleUI
            )
          )
        );
      },

      _didClickHeaderMenu: function _didClickHeaderMenu() {
        if (this.props.presentationMethod === 'modal') {
          AppNavigationController.dismissModal(this);
          return;
        }

        if (!Session.isLoggedIn()) {
          return;
        }
      
        if (Runtime.isOffline && !Runtime.canWorkOffline()) {
          return;
        }

        if (this.props.navigationDepth === 0) {
          AppNavigationController.toggleSidebar();
          return;
        }

        AppNavigationController.navigateBack();
      },

      _maybeRenderFilters: function _maybeRenderFilters() {
        var filtersToRender = (this.getPageFiltersToRender && this.getPageFiltersToRender());

        if (!filtersToRender) {
          return;
        }

        var filters = this.getFilters();
        var filtersToRenderMap = ArrayUtils.fillKeys(filtersToRender, 1);

        var childNodes = [];

        if (filtersToRenderMap.period) {
          var shouldSelectFuturePeriod = this.shouldSelectFuturePeriod && this.shouldSelectFuturePeriod();
          childNodes.push(
            React.createElement(UIPeriodPicker, {
              filters: filters, 
              onSelect: this._didSelectPeriodFilter, 
              future: shouldSelectFuturePeriod, 
              key: "filter-period"}
            )
          );
        }

        if (filtersToRenderMap.tagId && filters.tags) {
          childNodes.push(
            React.createElement(UICategoryNavigation, {tagId: filters.tags, href: this.props.url, key: "filter-category"})
          );
        }

        if (filtersToRenderMap.accountId) {
          childNodes.push(
            React.createElement(UIAccountPicker, {
              accountId: filters.accountId, 
              enableAllAccountsOption: true, 
              onSelect: this._didSelectAccountFilter, 
              key: "filter-account"}
            )
          );
        }

        return (
          React.createElement("div", {className: "UIAppFilters"}, 
            React.createElement(UIHorizontalList, {className: "fullWidth", widths: [1]}, 
              childNodes
            )
          )
        );
      },

      _didSelectPeriodFilter: function _didSelectPeriodFilter(periodParams) {
        AppNavigationController.filterCurrentUrl(periodParams);
      },

      _didSelectAccountFilter: function _didSelectAccountFilter(account) {
        AppNavigationController.filterCurrentUrl({
          accountId: account.id
        });
      },

      _maybeRenderActions: function _maybeRenderActions() {
        if (Runtime.isOffline && !this.canWorkOffline()) {
          return null;
        }

        if (this.renderCustomActions) {
          return this.renderCustomActions();
        }

        return null;
      },

      getFilters: function getFilters() {
        var filterKeys = [
          'tags',
          'accountId',
          'contacts',
          'groupId',
          'startDate',
          'endDate',
          'periodType',
          'keywords',
          'transactionType',
          'amountType',
          'sortField',
          'sortOrder',
          'tids'
        ];
        var implicitParams = {};
        if (this.getImplicitFilterParams) {
          implicitParams = this.getImplicitFilterParams();
        }
        var filterParams = _.extend(implicitParams, this.getAllRequestParams());
        if (filterParams.periodType === 'custom') {
          delete filterParams.periodType;
        }
        return _.pick(filterParams, filterKeys);
      },

      getAllRequestParams: function getAllRequestParams() {
        return this.props.params || {};
      },

      getRequestParam: function getRequestParam(paramName, defaultValue) {
        var allParams = this.getAllRequestParams();
        
        if (paramName in allParams) {
          return allParams[paramName];
        }

        if (typeof defaultValue != 'undefined') {
          return defaultValue;
        }

        if (this.getDefaultParamValues) {
          var defaults = this.getDefaultParamValues();
          return defaults[paramName];
        }

        return null;

      },

      _maybeRenderDemoNotice: function _maybeRenderDemoNotice() {
        if (!Runtime.isDemo || this.props.presentationMethod === 'modal') {
          return;
        }

        var signupButton;
        if (Runtime.isApp) {
          signupButton = (
            React.createElement(UIButton, {
              className: "orangeButton", 
              title: UIStaticText('SIGNUP FOR A FREE ACCOUNT'), 
              onClick: this._didClickEndDemo}
            )
          );
        } else {
          signupButton = (
            React.createElement(UIButton, {
              className: "orangeButton", 
              href: "https://www.buxfer.com/signup", 
              target: "_top", 
              title: UIStaticText('SIGNUP FOR A FREE ACCOUNT')}
            )
          );
        }

        return (
          React.createElement("div", {className: "pageNotice smallFont cAlign"}, 
            UIStaticText('This is a computer-generated demo account.'), 
            React.createElement("div", {className: "ttMargin"}, 
              signupButton
            )
          )
        );
      },

      _didClickEndDemo: function _didClickEndDemo() {
        AppNavigationController.endInAppDemo();
        AppNavigationController.openUrl('/signup');
      }
    };
  }
);

/** @jsx React.DOM */


define(
  'touch/auth/UILogin',[
    'touch/app/Runtime',
    'touch/auth/UIAuth',
    'touch/app/UIPage',
    ''
  ],
  function UILogin(
    Runtime,
    UIAuth,
    UIAppPage,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getBrowserPageTitle: function getBrowserPageTitle() {
        return UIStaticText('Welcome');
      },

      renderPageTitle: function renderPageTitle() {
        if (Runtime.isHarvest) {
          return UIStaticText('WELCOME TO THE HARVEST PLAN');
        }
        return UIStaticText('WELCOME TO BUXFER');
      },

      getPageClasses: function getPageClasses() {
        return ['UILogin'];
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement(UIAuth, {mode: "login"})
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/auth/UISignup',[
    'touch/app/Runtime',
    'touch/auth/UIAuth',
    'touch/app/UIPage',
    ''
  ],
  function UISignup(
    Runtime,
    UIAuth,
    UIAppPage,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getBrowserPageTitle: function getBrowserPageTitle() {
        return UIStaticText('Welcome');
      },

      renderPageTitle: function renderPageTitle() {
        if (Runtime.isHarvest) {
          return UIStaticText('WELCOME TO THE HARVEST PLAN');
        }
        return UIStaticText('WELCOME TO BUXFER');
      },

      getPageClasses: function getPageClasses() {
        return ['UISignup'];
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement(UIAuth, {mode: "signup"})
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIAmount',[
    ''
  ],
  function UIAmount(
    junk
  ) {
    return React.createClass({

      compose: function compose() {
        var amount = ASSERT_NOT_NULL(this.props.amount, '[UIAmount] Expected non-null amount');
        var delimiter = this.props.delimiter || ',';
        var decimalSymbol = this.props.decimalSymbol || '.';
        var showZeroDecimals = this.props.showZeroDecimals;
        if (typeof showZeroDecimals == 'undefined') {
          showZeroDecimals = true;
        }

        var sign = (amount >= 0) ? '+' : '-';
        var isPositiveSign = (amount >= 0);
        var hasColor = (this.props.amountType !== 'none');
        switch (this.props.amountType) {
          case 'expense':
            sign = '-';
            isPositiveSign = false;
            break;

          case 'income':
          case 'refund':
            sign = '+';
            isPositiveSign = true;
            break;

          case 'debt':
            sign = (
              React.createElement("span", {className: "debtSign"}, 
                (amount >= 0) ? UIStaticText('GET') : UIStaticText('OWE')
              )
            );
            break;

          case 'settlement':
            isPositiveSign = (amount <= 0);
            sign = (
              React.createElement("span", {className: "debtSign"}, 
                UIStaticText('SETTLED')
              )
            );
            break;

          case 'transfer':
            sign = (
              React.createElement("span", {className: "transferSign"}, " ")
            );
            hasColor = false;
            break;

          case 'transfer_one_way':
            sign = (
              React.createElement("span", {className: (amount >= 0) ? 'transferInSign' : 'transferOutSign'}, " ")
            );
            break;
        }

        amount = Math.abs(amount)+'';

        var formattedAmount = '';
        var decimalPos = amount.indexOf(decimalSymbol);
        if (decimalPos == -1) {
          formattedAmount = showZeroDecimals ? decimalSymbol+'00' : '';
        } else {
          formattedAmount = decimalSymbol + amount.substr(decimalPos + 1);
          amount = amount.substr(0, decimalPos);
          if (formattedAmount.length == 2) {
            formattedAmount += '0';
          }
        }

        while (amount.length > 3) {
          formattedAmount = delimiter + amount.substr(-3) + formattedAmount;
          amount = amount.substr(0, amount.length-3);
        }
        formattedAmount = amount + formattedAmount;

        var classNames = ['UIAmount'];

        var spacer;
        if (this.props.shouldDisplaySign === false || this.props.amountType === 'none' || this.props.isMovement) {
          sign = '';
          spacer = '';
        } else {
          spacer = ' ';
        }

        if (this.props.isMovement) {
          var isGoodMovement, movementDirection;
          if (this.props.amountType === 'expense') {
            isGoodMovement = (amount <= 0);
            movementDirection = (amount <= 0) ? 'moveDown' : 'moveUp';
          } else {
            isGoodMovement = (amount >= 0);
            movementDirection = (amount >= 0) ? 'moveUp' : 'moveDown';
          }
          classNames.push('movement');
          classNames.push(movementDirection);
          classNames.push(isGoodMovement ? 'moveGood' : 'moveBad');
          isPositiveSign = isGoodMovement;
        }

        if (hasColor) {
          if (isPositiveSign) {
            classNames.push('greenAmount');
          } else {
            classNames.push('redAmount');
          }
        }

        classNames = classNames.join(' ');
        return (
          React.createElement("div", {className: classNames}, 
            sign, spacer, formattedAmount
          )
        );
      }

    });
  }
);


/** @jsx React.DOM */


define(
  'touch/app/UISection',[
    ''
  ],
  function UISection(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var classNames = ['UISection'];
        if (this.props.isPadded) {
          classNames.push('padded');
        }
        if (!this.props.label) {
          classNames.push('noLabel');
        }

        classNames = classNames.join(' ');

        return (
          React.createElement("div", {className: classNames}, 
            this._maybeRenderLabel(), 
            this.props.children
          )
        );
      },

      _maybeRenderLabel: function _maybeRenderLabel() {
        var labelUI = null;
        if (this.props.label) {
          labelUI = (
            React.createElement("span", null, 
              React.createElement("span", {className: "bFontWeight"}, 
                this.props.label
              ), 
              this.props.labelMetadata
            )
          );
        } else if (!this.props.isLoading) {
          return null;
        }

        return (
          React.createElement("div", {className: "sectionLabel"}, 
            this._maybeRenderLoadingIndicator(), 
            labelUI
          )
        );
      },

      _maybeRenderLoadingIndicator: function _maybeRenderLoadingIndicator() {
        if (!this.props.isLoading) {
          return null;
        }
        return React.createElement("div", {className: "loadingIndicator"}, " ");
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/app/UICard',[
    'touch/app/Runtime',
    'touch/app/NavigationController',
    'touch/app/UISection',
    'touch/ui/UIHorizontalList',
    'touch/membership/UIBlock',
    'touch/membership/UIUpsell',
    ''
  ],
  function UICard(
    Runtime,
    AppNavigationController,
    UISection,
    UIHorizontalList,
    UIMembershipBlock,
    UIMembershipUpsell,
    junk
  ) {

    return {

      canWorkOffline: function canWorkOffline() {
        return false;
      },

      compose: function compose() {
        ASSERT_TRUTHY(this.renderCard, '[UICard] Missing definition for renderCard');

        var accessControl = this.state.accessControl || {};
        if (accessControl.block) {
          if (this.props.hideAccessControl) {
            return React.createElement("div", null);
          }
          return React.createElement(UIMembershipBlock, {feature: accessControl.feature});
        }

        var cardUI;
        if (
          this.state.numFailedModels
          || (Runtime.isOffline && (!Runtime.canWorkOffline() || !this.canWorkOffline()))
        ) {
          cardUI = (
            React.createElement(UISection, {className: "loadingError"}, 
              React.createElement("div", {className: "errorMessageContainer"}, 
                React.createElement(UIHorizontalList, {isFullWidth: true, widths: [null, 1]}, 
                  UIStaticText('Error loading data'), 
                  React.createElement("a", {className: "retryLink smallFont semiBold", onClick: this._didClickRetry}, UIStaticText('RETRY'))
                )
              )
            )
          );
        } else {
          cardUI = this.renderCard();
          if (!this.props.hideAccessControl && accessControl.preview) {
            var children = cardUI.props.children;
            if (!Object.isArray(children)) {
              children = [children];
            }
            children.push(
              React.createElement(UIMembershipUpsell, {
                feature: accessControl.feature, 
                numPreviewDaysRemaining: accessControl.numPreviewDaysRemaining}
              )
            );
            cardUI = React.cloneElement(cardUI, {}, children);
          }
        }

        if (cardUI) {
          cardUI = React.cloneElement(cardUI, {
            label: this.renderCardLabel(),
            labelMetadata: this.renderCardLabelMetadata()
          });
        }

        return cardUI;
      },

      renderCardLabel: function renderCardLabel() {
        return null;
      },

      renderCardLabelMetadata: function renderCardLabelMetadata() {
        return null;
      },

      _didClickRetry: function _didClickRetry() {
        AppNavigationController.reloadPage();
      }

    };

  }
);

/** @jsx React.DOM */


define(
  'touch/UIQuickSummary',[
    'touch/core/Model',
    'touch/ui/UIAmount',
    'touch/ui/UILeftRight',
    'touch/app/UISection',
    'touch/app/UISectionRow',
    'touch/app/UICard',
    ''
  ],
  function UIQuickSummary(
    Model,
    UIAmount,
    UILeftRight,
    UISection,
    UISectionRow,
    UICard,
    junk
  ) {
    return React.createClass({

      mixins: [UICard],

      getModelsToFetch: function getModelsToFetch() {
        if (!this.props.dataUrl) {
          return {};
        }

        return {
          summary: Model.create({
            url: this.props.dataUrl
          })
        };
      },

      renderCard: function renderCard() {
        return (
          React.createElement(UISection, {className: "UIQuickSummary"}, 
            this._renderSummaryItems()
          )
        );
      },

      _renderSummaryItems: function _renderSummaryItems() {
        var items, isLoading = false;
        if (this.props.items) {
          items = this.props.items;
        } else if (this.props.dataUrl && this.state.models.summary) {
          items = this.state.models.summary.attributes.summaryItems;
        } else {
          var labels = ASSERT_TRUTHY(this.props.labels, '[UIQuickSummary] Expected non-empty labels');
          isLoading = true;
          items = [];
          ArrayUtils.each(labels, function(index, label) {
            items.push({
              title: label,
              amountUI: React.createElement("div", null)
            });
          });
        }

        var hrefMap = this.props.hrefs || {};

        var childNodes = [];
        ArrayUtils.each(items, FunctionUtils.bind(this, function(index, summaryItem) {
          var amountUI = summaryItem.amountUI;
          if (!amountUI) {
            amountUI = (
              React.createElement(UIAmount, {
                amount: summaryItem.amount, 
                amountType: summaryItem.amountType, 
                shouldDisplaySign: summaryItem.shouldDisplaySign, 
                isMovement: summaryItem.isMovement}
              )
            );
          }
          var itemUI = (
            React.createElement(UILeftRight, null, 
              summaryItem.title, 
              amountUI
            )
          );

          var itemKey = 'summary-' + (summaryItem.type || index);

          var href = summaryItem.href;
          if (!href && summaryItem.type && hrefMap[summaryItem.type]) {
            href = hrefMap[summaryItem.type];
          }

          if (href && this.props.filters) {
            href = Url.parse(href);
            href.addQueryParams(this.props.filters);
            href = href.toString();
          }

          itemUI = (
            React.createElement(UISectionRow, {href: href, key: itemKey, isLoading: isLoading}, 
              itemUI
            )
          );

          childNodes.push(itemUI);
        }));
        return childNodes;
      }

    });
  }
);



define(
  'touch/lists/Model',[
    'touch/core/Model',
    ''
  ],
  function ListModel(
    Model,
    junk
  ) {

    return {

      createClass: function createClass(definition) {
        // Do not define `attributes` on the definition here. Somehow it gets
        // copied onto each model and persists any previously saved values,
        // causing navStart's to incorrectly being propagated between subsequent
        // instantiations of the list model.
        var thisCopy = _.clone(this);
        delete thisCopy.createClass;
        Object.extend(thisCopy, definition);
        return Model.createCollectionClass(thisCopy);
      },

      isEmpty: function isEmpty() {
        return this.models.length == 0;
      },

      hasMoreItems: function hasMoreItems() {
        return !this.attributes || (this.attributes.navStart + this.attributes.listSize < this.attributes.numTotalItems);
      },

      fetch: function fetch(params) {
        params = params || {};

        var navStart;
        if (!this.attributes) {
          navStart = 1;
        } else {
          navStart = this.attributes.navStart;
        }
        
        if (params.isTailLoad) {
          navStart += this.attributes.listSize;
        }

        var data = {
          navStart: navStart,
        };
        var fieldsToTransfer = ['disablePagination', 'sortField', 'sortOrder'];
        ArrayUtils.each(fieldsToTransfer, FunctionUtils.bind(this, function(index, fieldName) {
          if (typeof this[fieldName] != 'undefined') {
            data[fieldName] = this[fieldName];
          }
        }));
        Object.extend(params, {
          update: true,
          remove: false,
          data: data
        });
        Backbone.Collection.prototype.fetch.call(this, params);
      },

      parse: function parse(response) {
        if (!this.attributes) {
          this.attributes = {};
        }
        this.attributes.numTotalItems = response.numTotalItems;
        this.attributes.listSize = response.listSize;
        this.attributes.navStart = response.navStart;
        this.attributes.accessControl = response.accessControl;

        if (this.parseCustomResponse) {
          this.parseCustomResponse(response);
        }

        return response.items;
      }

    };

  }
);



define(
  'touch/transactions/Model',[
    'touch/core/Model',
    ''
  ],
  function TransactionModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function() {
        var url = Url.createWithPath('/apix/transaction');
        if (this.id) {
          url.addQueryParam('id', this.id);
        }
        return url.toString();
      },

      isPersonalTransaction: function isPersonalTransaction() {
        var personalTypes = {
          expense: 1,
          income: 1,
          transfer: 1,
          refund: 1
        }
        return personalTypes[this.attributes.transactionType] === 1;
      }

    });

  }
);



define(
  'touch/transactions/list/Model',[
    'touch/lists/Model',
    'touch/transactions/Model',
    ''
  ],
  function TransactionListModel(
    ListModel,
    TransactionModel,
    junk
  ) {

    return ListModel.createClass({

      url: function() {
        var url = Url.createWithPath('/apix/transactions');
        if (this.filters) {
          url.addQueryParams(this.filters);
        }
        return url.toString();
      },

      model: TransactionModel
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/lists/UIList',[
    'touch/app/UISection',
    'touch/app/UISectionRow',
    'touch/ui/UIButton',
    'touch/app/UICard',
    'touch/ui/UIText',
    'touch/ui/UILeftRight',
    'touch/ui/UIAmount',
    ''
  ],
  function UIModelList(
    UISection,
    UISectionRow,
    UIButton,
    UICard,
    UIText,
    UILeftRight,
    UIAmount,
    junk
  ) {

    var base = {};
    Object.extend(base, UICard);

    return Object.extend(base, {

      componentDidMount: function componentDidMount() {
        EventNotificationManager.subscribe(EventType.PAGE_DID_SCROLL, FunctionUtils.bind(this, this._pageDidScroll));
      },

      _pageDidScroll: function _pageDidScroll(eventData) {
        if (!this.state.models.list) {
          return;
        }

        var windowSize = DOM.windowSize();
        var documentSize = DOM.documentSize();
        if (windowSize.height + eventData.scrollTop > documentSize.height - 10) {
          this._loadMoreItems();
        }
      },

      getModelsToFetch: function getModelsToFetch() {
        return {
          list: this.createListModel()
        };
      },

      renderCard: function renderCard() {
        var isLoading = (!this.state.models.list);

        var content;
        if (isLoading) {
          content = null;
        } else if (this.state.models.list.isEmpty()) {
          content = this.renderEmptyState();
        } else {
          content = this._renderListItems();
        }

        return (
          React.createElement(UISection, {className: "UIModelList", isLoading: isLoading}, 
            content
          )
        );
      },

      renderEmptyState: function renderEmptyState() {
        return null;
      },

      getModelsToRender: function getModelsToRender() {
        return this.state.models.list.models;
      },

      renderBalanceForModel: function renderBalanceForModel(model) {
        // Balance can be zero
        if (typeof model.attributes.balance === 'undefined') {
          return null;
        }

        return React.createElement(UIAmount, {amount: model.attributes.balance});
      },

      renderNameForModel: function renderNameForModel(model) {
        return model.attributes.name;
      },

      renderModel: function renderModel(model) {
        var ui = this.renderNameForModel(model);

        var balanceUI = this.renderBalanceForModel(model);
        if (balanceUI) {
          ui = (
            React.createElement(UILeftRight, null, 
              React.createElement(UIText, {maxLength: "20"}, ui), 
              balanceUI
            )
          );
        } else {
          ui = React.createElement(UIText, {maxLength: "30"}, ui);
        }

        var href = model.getNavUrl ? model.getNavUrl() : null;
        return (
          React.createElement(UISectionRow, {href: href}, 
            ui
          )
        );
      },

      _renderListItems: function _renderListItems() {
        var childNodes = [];
        var index = 0;
        ArrayUtils.each(this.getModelsToRender(), FunctionUtils.bind(this, function(junk, model) {
          var modelUI = React.cloneElement(this.renderModel(model), {
            key: 'key-' + (model.id || index)
          });
          childNodes.push(modelUI);
          ++index;
        }));


        return (
          React.createElement("div", {className: "listContainer"}, 
            childNodes, 
            this._maybeRenderNavigation()
          )
        );
      },

      _maybeRenderNavigation: function _maybeRenderNavigation() {
        if (!this.state.models.list.hasMoreItems()) {
          return null;
        }

        var title = UIStaticText('SEE MORE');
        if (this.state.isTailLoading) {
          title = UIStaticText('LOADING ...');
        }

        return (
          React.createElement(UISectionRow, null, 
            React.createElement(UIButton, {ref: "seeMoreButton", title: title, isFullWidth: true, className: "ttMargin", onClick: this._didClickNavigation})
          )
        );
      },

      _didClickNavigation: function _didClickNavigation() {
        this._loadMoreItems();
      },

      _loadMoreItems: function _loadMoreItems() {
        if (this.state.isTailLoading) {
          return;
        }

        this._setTailLoading(true);

        this.state.models.list.fetch({
          isTailLoad: true,
          success: FunctionUtils.bind(this, this._didLoadTail),
          error: FunctionUtils.bind(this, this._didFailLoadTail)
        });
      },

      _didLoadTail: function _didLoadTail(model) {
        this._setTailLoading(false);

        var models = this.state.models;
        models.listModel = model;
        this.setState({
          models: models
        });
      },

      _didFailLoadTail: function _didFailLoadTail() {
        this._setTailLoading(false);
      },

      _setTailLoading: function _setTailLoading(isLoading) {
        this.setState({
          isTailLoading: isLoading
        });
        this.refs.seeMoreButton && DOM.setLoading(this.refs.seeMoreButton.getDOMNode(), isLoading);
      }
    });
  }
);



define(
  'touch/utils/Date',[
    ''
  ],
  function DateUtils(
    junk
  ) {
    return {

      monthNames:     ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
      numDaysInMonth: [ 31  ,  28  ,  31  ,  30  ,  31  ,  30  ,  31  ,  31  ,  30  ,  31  ,  30  ,  31  ],

      getMonthName: function getMonthName(month) {
        return this.monthNames [month - 1];
      },

      getPreviousDate: function getPreviousDate(date) {
        return this._stepDate(date, -1);
      },

      getNextDate: function(date) {
        return this._stepDate(date, 1);
      },

      getDateAtOffset: function getDateAtOffset(date, offset) {
        return this._stepDate(date, offset);
      },

      _stepDate: function _stepDate(date, step) {
        var d = this.fromString(date);
        var newDate = new Date();
        newDate.setFullYear(d.getFullYear(), d.getMonth(), d.getDate() + step);
        return this.toString(newDate);
      },

      today: function today() {
        return this.toString(new Date());
      },

      getCurrentUnixTime: function getCurrentUnixTime() {
        var d = new Date();
        return Math.round(d.getTime() / 1000);
      },

      isGreaterThan: function isGreaterThan(date1String, date2String) {
        return this.fromString(date1String).getTime() > this.fromString(date2String).getTime();
      },

      isLessThan: function isLessThan(date1String, date2String) {
        return this.fromString(date1String).getTime() < this.fromString(date2String).getTime();
      },

      fromString: function fromString(date) {
        var utcDate = new Date(Date.parse(date+'T00:00:00Z'));
        var localDate = new Date();
        var timezoneOffset = localDate.getTimezoneOffset();
        localDate.setFullYear(utcDate.getUTCFullYear(), utcDate.getUTCMonth(), utcDate.getUTCDate());
        localDate.setHours(0);
        localDate.setMinutes(0);
        localDate.setSeconds(0);
        localDate.setMilliseconds(0);
        return localDate;
      },

      toString: function toString(d) {
        var month = d.getMonth() + 1;
        if (month < 10) {
          month = '0'+month;
        }
        var day = d.getDate();
        if (day < 10) {
          day = '0'+day;
        }
        return d.getFullYear()+'-'+month+'-'+day;
      }
    }
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIDate',[
    'touch/utils/Date',
    ''
  ],
  function UIDate(
    DateUtils,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var date = ASSERT_TRUTHY(this.props.date, '[UIDate::render] Expected non-empty date');

        var todayDate = new Date();
        var today = DateUtils.toString(todayDate);

        var formattedDate;
        if (today === date) {
          formattedDate = 'Today';
        } else {
          var parsedDate = DateUtils.fromString(date);
          var defaultShowYear = (parsedDate.getFullYear() != todayDate.getFullYear());

          var shortenYear = this.props.shortenYear || false;

          formattedDate = parsedDate.getDate();
          formattedDate += ' '+DateUtils.getMonthName(parsedDate.getMonth() + 1);

          if (this.props.showYear || defaultShowYear) {
            var year = parsedDate.getFullYear();
            if (shortenYear) {
              year = year.substr(-2);
            }
            formattedDate += ' '+year;
          }
        }

        return (
          React.createElement("div", {className: "UIDate"}, 
            formattedDate
          )
        );
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/UIAccount',[
    'touch/ui/UIText',
    ''
  ],
  function UITransactionAccount(
    UIText,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement("div", {className: "UITransactionAccount"}, 
            this._renderAccount()
          )
        );
      },

      _renderAccount: function _renderAccount() {
        var transaction = this.props.transaction;
        ASSERT_TRUTHY(transaction, '[UITransactionAccount] Expected non-empty transaction');

        var maxLength = this.props.maxLength || 20;

        if (transaction.attributes.transactionType !== 'transfer') {
          if (!transaction.attributes.account.name) {
            return (
              React.createElement(UIText, {className: "emptyStateContent"}, 
                UIStaticText('no account')
              )
            );
          }

          return React.createElement(UIText, {maxLength: maxLength}, transaction.attributes.account.name);
        }

        var fromAccount = transaction.attributes.fromAccount;
        var toAccount = transaction.attributes.toAccount;

        if (!fromAccount) {
          return React.createElement(UIText, {maxLength: maxLength}, toAccount.name);
        }

        if (!toAccount) {
          return React.createElement(UIText, {maxLength: maxLength}, fromAccount.name);
        }

        maxLength /= 2;

        return (
          React.createElement("div", null, 
            React.createElement(UIText, {className: "fromAccount", maxLength: maxLength}, fromAccount.name), 
            React.createElement("span", {className: "transferArrowContainer"}, " "), 
            React.createElement(UIText, {className: "toAccount", maxLength: maxLength}, toAccount.name)
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/UITags',[
    'touch/ui/UIText',
    ''
  ],
  function UITransactionTags(
    UIText,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var transaction = this.props.transaction;
        ASSERT_TRUTHY(transaction, '[UITransactionAccount] Expected non-empty transaction');

        if (!transaction.attributes.tagNames.length) {
          return (
            React.createElement("div", {className: "UITransactionTags emptyStateContent"}, 
              UIStaticText('no tags')
            )
          );
        }

        var tagNames = transaction.attributes.tagNames.join(', ');
        var maxLength = this.props.maxLength || 20;

        return (
          React.createElement("div", {className: "UITransactionTags"}, 
            React.createElement(UIText, {maxLength: maxLength}, 
              tagNames
            )
          )
        );
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/UITransaction',[
    'touch/ui/UIPopup',
    'touch/ui/UILeftRight',
    'touch/ui/UIDate',
    'touch/ui/UIAmount',
    'touch/ui/UIText',
    'touch/ui/UIHorizontalList',
    'touch/app/NavigationController',
    'touch/transactions/UIAccount',
    'touch/transactions/UITags',
    'touch/app/UISectionRow',
    ''
  ],
  function (
    UIPopup,
    UILeftRight,
    UIDate,
    UIAmount,
    UIText,
    UIHorizontalList,
    AppNavigationController,
    UITransactionAccount,
    UITransactionTags,
    UISectionRow,
    junk
  ) {

    var lhsMaxLength = 20;

    return React.createClass({

      compose: function compose() {
        var transaction = this.props.transaction;
        // Offline transactions don't have an id
        var href = transaction.id ? '/transaction?id='+transaction.id : null;
        var description = transaction.attributes.description || '<No description>';
        return (
          React.createElement(UISectionRow, {className: "UITransaction", key: 'transaction-'+transaction.id, href: href}, 
            React.createElement(UILeftRight, null, 
              React.createElement(UIText, {maxLength: lhsMaxLength}, description), 
              React.createElement(UIAmount, {amount: transaction.attributes.amount, amountType: transaction.attributes.amountType})
            ), 
            React.createElement(UILeftRight, {className: "metadataContainer smallFont"}, 
              this._renderAccountAndTags(), 
              React.createElement(UIDate, {className: "dateContainer", date: transaction.attributes.date})
            ), 
            this.props.children
          )
        );
      },

      _renderAccountAndTags: function _renderAccountAndTags() {
        var ui = React.createElement(UITransactionAccount, {transaction: this.props.transaction, maxLength: "40"});

        if (!this.props.transaction.attributes.tagNames.length) {
          return ui;
      }
      
        return (
          React.createElement("div", null, 
            ui, 
            React.createElement(UITransactionTags, {transaction: this.props.transaction, maxLength: "40"})
          )
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/list/UIList',[
    'touch/transactions/list/Model',
    'touch/lists/UIList',
    'touch/transactions/UITransaction',
    ''
  ],
  function UITransactionList(
    TransactionListModel,
    UIModelList,
    UITransaction,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        if (this.props.listModel) {
          return this.props.listModel;
        }

        if (this.props.extension && this.props.extension.createListModel) {
          return this.props.extension.createListModel();
        }

        var model = new TransactionListModel();
        model.filters = this.props.filters;
        return model;
      },

      getClassNames: function getClassNames() {
        return ['UITransactionList'];
      },

      renderModel: function renderModel(transaction) {
        return React.createElement(UITransaction, {transaction: transaction});
      },

      renderCardLabel: function renderCardLabel() {
        return this.props.title || UIStaticText('TRANSACTIONS');
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO TRANSACTIONS')
          )
        );
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/offline/Manager',[
    ''
  ],
  function OfflineTransactionsManager(
    junk
  ) {

    var storageKey = 'offline-transactions-v14';

    return {

      isAvailable: function isAvailable() {
        return BrowserStore.isAvailable();
      },

      maybeGetTransactions: function maybeGetTransactions() {
        if (!this.isAvailable()) {
          return null;
        }

        var offlineTransactions = BrowserStore.getItem(storageKey);
        if (!offlineTransactions || !offlineTransactions.length) {
          return null;
        }
        var allTransactions = Object.fromString(offlineTransactions);
        var validTransactions = [];
        ArrayUtils.each(allTransactions, function(index, transaction) {
          // Filter transactions in bad formats
          if (!transaction.date) {
            return true;
          }
          validTransactions.push(transaction);
        });
        return validTransactions.length ? validTransactions : null;
      },

      saveTransaction: function saveTransaction(transaction) {
        var offlineTransactions = this.maybeGetTransactions() || [];

        if (transaction.offlineId) {
          var foundOfflineId = false;
          var newTransactionList = [];
          ArrayUtils.each(offlineTransactions, function(index, existingModel) {
            if (existingModel.offlineId === transaction.offlineId) {
              foundOfflineId = true;
              newTransactionList.push(transaction);
            } else {
              newTransactionList.push(existingModel);
            }
          });
          if (!foundOfflineId) {
            newTransactionList.push(transaction);
          }
          offlineTransactions = newTransactionList;
        } else {
          var offlineId = 'offline-txn-'+DateUtils.getCurrentUnixTime();
          transaction.offlineId = offlineId;
          offlineTransactions.push(transaction);
        }
        this._saveTransactions(offlineTransactions);
      },

      deleteTransaction: function deleteTransaction(transaction) {
        var offlineTransactions = ASSERT_TRUTHY(this.maybeGetTransactions(), 'Expected non-empty offline transactions');
        ASSERT_TRUTHY(transaction.offlineId, 'Expected non-empty offline transaction id');

        var newTransactionList = [];
        var foundOfflineId = false;
        ArrayUtils.each(offlineTransactions, function(index, model) {
          if (model.offlineId === transaction.offlineId) {
            foundOfflineId = true;
            return true;
          }
          newTransactionList.push(model);
        });
        ASSERT_TRUE(foundOfflineId, 'Failed to locate offline transaction id');
        this._saveTransactions(newTransactionList);
      },

      _saveTransactions: function _saveTransactions(offlineTransactions) {
        if (!offlineTransactions.length) {
          BrowserStore.deleteItem(storageKey);
        } else {
          BrowserStore.addItem(storageKey, Object.toString(offlineTransactions));
        }
      }
    }
  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/offline/UIList',[
    'touch/app/NavigationController',
    'touch/app/Runtime',
    'touch/app/UISection',
    'touch/app/UICard',
    'touch/transactions/Model',
    'touch/transactions/UITransaction',
    'touch/transactions/offline/Manager',
    ''
  ],
  function UIOfflineTransactionList(
    AppNavigationController,
    Runtime,
    UISection,
    UICard,
    TransactionModel,
    UITransaction,
    OfflineTransactionsManager,
    junk
  ) {

    return React.createClass({

      mixins: [UICard],

      canWorkOffline: function canWorkOffline() {
        return true;
      },

      componentDidMount: function componentDidMount() {
        if (!Runtime.isOffline) {
          this._syncTransactions();
        }
      },

      _syncTransactions: function _syncTransactions() {
        var offlineTransactions = OfflineTransactionsManager.maybeGetTransactions();
        if (!offlineTransactions) {
          return;
        }

        this.setState({
          isSyncing: true
        });
        var numTransactions = offlineTransactions.length;
        var _didCompleteSyncRequestAtIndex = FunctionUtils.bind(this, function _didCompleteSyncRequestAtIndex(index) {
          if (index === numTransactions - 1) {
            this.setState({
              isSyncing: false
            });
          }
        });

        ArrayUtils.each(offlineTransactions, function(index, model) {
          // Don't sync again if we previously encountered a system or user
          // error.
          if (model.syncError) {
            _didCompleteSyncRequestAtIndex(index);
            return true;
          }

          var _didSync = function() {
            OfflineTransactionsManager.deleteTransaction(model);
            _didCompleteSyncRequestAtIndex(index);
          };

          var _didFailSync = function(_backboneModel, request) {
            if (request.responseJSON && request.responseJSON.error) {
              model.syncError = request.responseJSON.error;
              OfflineTransactionsManager.saveTransaction(model);
            }
            _didCompleteSyncRequestAtIndex(index);
          };

          var _model = _.clone(model);
          delete _model.offlineId;
          var transaction = new TransactionModel(model);
          transaction.save({}, {
            success: _didSync,
            error: _didFailSync
          });
        });

      },

      renderCard: function renderCard() {
        var offlineTransactions = OfflineTransactionsManager.maybeGetTransactions();

        if (!offlineTransactions) {
          return null;
        }

        // Sort by date of txn, not order of entry
        offlineTransactions = offlineTransactions.reverse();

        var childNodes = [];
        ArrayUtils.each(offlineTransactions, function(index, model) {
          var transaction = new TransactionModel(model);

          var transactionUI = React.createElement(UITransaction, {transaction: transaction});
          if (model.syncError) {
            transactionUI.props.children = (
              React.createElement("div", {className: "syncErrorMessage red smallFont tMargin"}, 
                UIStaticText('Error saving to server. Try again manually.')
              )
            );
          }
          childNodes.push(transactionUI);
        });

        return (
          React.createElement(UISection, {isLoading: this.state.isSyncing}, 
            childNodes
          )
        );

      },

      renderCardLabel: function renderCardLabel() {
        return UIStaticText('OFFLINE TRANSACTIONS');
      }

    });

  }
);



define(
  'touch/alerts/Model',[
    'touch/core/Model',
    ''
  ],
  function AlertModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/alert?id='+this.id;
      }

    });

  }
);



define(
  'touch/alerts/ListModel',[
    'touch/lists/Model',
    'touch/alerts/Model',
    ''
  ],
  function AlertListModel(
    ListModel,
    AlertModel,
    junk
  ) {

    return ListModel.createClass({
      url: '/apix/alerts',
      model: AlertModel
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/alerts/UIPopup',[
    'touch/alerts/ListModel',
    'touch/ui/UIPopup',
    'touch/app/NavigationController',
    'touch/app/UISectionRow',
    ''
  ],
  function UIAlertPopup(
    AlertListModel,
    UIPopup,
    AppNavigationController,
    UISectionRow,
    junk
  ) {

    return React.createClass({

      getModelsToFetch: function getModelsToFetch() {
        return {
          alertList: new AlertListModel()
        };
      },

      compose: function compose() {
        if (!this.state.models.alertList || this.state.models.alertList.isEmpty()) {
          return React.createElement("div", null);
        }

        return (
          React.createElement("div", {className: "UIAlertPopup clickable", onClick: this._didClickPopup}, 
            React.createElement("div", {className: "popupHandleContainer"}, 
              React.createElement("div", {className: "badgeContainer"}, 
                this.state.models.alertList.attributes.numTotalItems
              )
            )
          )
        );
      },

      _didClickPopup: function _didClickPopup() {
        var cells = [];
        ArrayUtils.each(this.state.models.alertList.models, function(index, alertModel) {
          cells.push(
            React.createElement(UISectionRow, null, 
              "Alert of type ", alertModel.attributes.type
            )
          );
        });

        AppNavigationController.presentPopup(
          React.createElement(UIPopup, {title: UIStaticText('ALERTS')}, 
            cells
          )
        );
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/dashboard/UIDashboard',[
    'touch/core/Model',
    'touch/app/UIPage',
    'touch/UIQuickSummary',
    'touch/transactions/list/UIList',
    'touch/transactions/offline/UIList',
    'touch/alerts/UIPopup',
    'touch/ui/UIHorizontalList',
    ''
  ],
  function UIDashboard(
    Model,
    UIAppPage,
    UIQuickSummary,
    UITransactionList,
    UIOfflineTransactionList,
    UIAlertPopup,
    UIHorizontalList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageContent: function renderPageContent() {
        return (
          React.createElement("div", {className: "UIDashboard"}, 
            this._renderSummary(), 
            React.createElement(UIOfflineTransactionList, null), 
            React.createElement(UITransactionList, null)
          )
        );
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('DASHBOARD');
      },

      renderCustomActions: function renderCustomActions() {
        return (
          React.createElement(UIHorizontalList, null, 
            React.createElement(UIAlertPopup, null), 
            React.createElement("a", {className: "searchActionContainer block", href: "/search"})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        var labels = [
          UIStaticText('NET WORTH'),
          UIStaticText('INCOME'),
          UIStaticText('EXPENSE')
        ];
        var hrefs = {
          networth: '/reports?reportType=networth',
          netloaned: '/loans',
          budget: '/budgets',
          income: '/reports?reportType=income',
          expense: '/reports?reportType=expense'
        };
        return React.createElement(UIQuickSummary, {labels: labels, hrefs: hrefs, dataUrl: "/apix/dashboard/summary"});
      }

    });
  }
);



define(
  'touch/accounts/list/Model',[
    'touch/lists/Model',
    'touch/accounts/Model',
    ''
  ],
  function AccountListModel(
    ListModel,
    AccountModel,
    junk
  ) {

    return ListModel.createClass({
      url: '/apix/accounts',
      model: AccountModel
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/accounts/list/UIList',[
    'touch/accounts/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UIAccountList(
    AccountListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        return new AccountListModel();
      },

      getClassNames: function getClassNames() {
        return ['UIAccountList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO ACCOUNTS')
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/accounts/list/UIPage',[
    'touch/app/UIPage',
    'touch/accounts/list/UIList',
  ],
  function UIAccountListPage(
    UIAppPage,
    UIAccountList
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('ACCOUNTS');
      },

      renderPageContent: function renderPageContent() {
        return React.createElement(UIAccountList, null);
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/charts/UIChart',[
    'touch/core/Model',
    ''
  ],
  function UIChart(
    Model,
    junk
  ) {

    return React.createClass({

      getModelsToFetch: function getModelsToFetch() {
        if (!this.props.dataUrl) {
          return {};
        }

        return {
          chart: Model.create({
            url: this.props.dataUrl
          })
        };
      },

      componentDidMount: function componentDidMount() {
        this._refreshPlot();
      },

      componentDidUpdate: function componentDidUpdate() {
        this._refreshPlot();
      },

      _refreshPlot: function _refreshPlot() {
        if (!this.props.data && !this.state.models.chart) {
          return;
        }

        var chartData = this.props.data || this.state.models.chart.attributes.chartData;

        this.numSeries = ArrayUtils.count(chartData);

        var firstSeries = chartData[0];
        this.startDate = firstSeries.data[0][0];
        this.endDate = firstSeries.data[firstSeries.data.length-1][0];
        this.numTicks = firstSeries.data.length;

        var container = DOM.findById(this.refs.chartContainer.getDOMNode());
        container.plot(chartData, this._getPlotOptions());
      },

      compose: function compose() {
        if (!this.props.data && !this.state.models.chart) {
          return React.createElement("div", null);
        }

        return (
          React.createElement("div", {className: "aaPadding UIChart"}, 
            React.createElement("div", {ref: "chartContainer", className: "chartContainer"})
          )
        );
      },

      getDefaultProps: function getDefaultProps() {
        return {
          chartType: 'line',
          pointRadius: 0,
          fill: 0,
          shouldShowZero: false
        };
      },

      _getPlotOptions: function _getPlotOptions() {
        var axisColor = '#dcdce2';
        var tickDecimals = 0;
        var tickLength = 5;
        var fontSpec = {
          size: 11,
          family: 'Proxima Nova Thin',
          color: '#606472'
        };

        var options = {
          series: {},
          xaxis: {
            mode: 'time',
            tickLength: tickLength,
            min: this.startDate,
            max: this.endDate,
            font: fontSpec
          },
          yaxis: {
            tickColor: '#f2f2f2',
            tickLength: tickLength,
            tickDecimals: tickDecimals,
            font: fontSpec
          },
          grid: {
            show: true,
            clickable: false,
            hoverable: false,
            mouseActiveRadius: 0,
            borderColor: axisColor,
            borderWidth: {
              left: 1,
              bottom: 1,
              top: 0,
              right: 0
            }
          },
          tooltip: {
            contextual: false
          },
          legend: {
            show: false
          }
        };

        if (this.props.chartType === 'line') {
          options.series.lines = {
            show: true,
            fill: this.props.fill,
            fillColor: false,
            zero: this.props.shouldShowZero ? 1 : 0
          };
          options.series.points = {
            show: true,
            radius: this.props.pointRadius,
            fill: 1,
            fillColor: false
          };
          options.series.shadowSize = this.numSeries == 1 ? 3 : 0;
        }

        if (this.props.chartType === 'bar') {
          var barWidth = (this.endDate - this.startDate) / (this.numTicks * 4);
          options.series.bars = {
            show: true,
            fill: true,
            fillColor: false,
            zero: this.props.shouldShowZero ? 1 : 0,
            order: 1,
            lineWidth: 1,
            barWidth: barWidth,
            align: 'left'
          };
        }

        return options;
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/forecast/UIChart',[
    'touch/core/Model',
    'touch/charts/UIChart',
    'touch/app/UISection',
    'touch/app/UICard',
    ''
  ],
  function UIForecastChart(
    Model,
    UIChart,
    UISection,
    UICard,
    junk
  ) {

    return React.createClass({

      mixins: [UICard],

      getModelsToFetch: function getModelsToFetch() {
        var chartDataUrl = Url.createWithPath('/apix/forecast/balanceChart');
        if (this.props.filters) {
          chartDataUrl.addQueryParams(this.props.filters);
        }

        return {
          chart: Model.create({
            url: chartDataUrl.toString()
          })
        };
      },

      renderCard: function renderCard() {
        if (!this.state.models.chart) {
          return React.createElement(UISection, {className: "UIForecastChart", isLoading: true});
        }

        return (
          React.createElement(UISection, {className: "UIForecastChart"}, 
            React.createElement(UIChart, {data: this.state.models.chart.attributes.chartData})
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
        var filters = this.props.filters || {};
        if (this.props.title) {
          return this.props.title;
        } else if (filters.accountId) {
          return UIStaticText('BALANCE');
        } else {
          return UIStaticText('NET WORTH');
        }
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/accounts/UIAccount',[
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    'touch/accounts/Model',
    'touch/UIQuickSummary',
    'touch/charts/UIChart',
    'touch/forecast/UIChart',
    'touch/app/UISection',
    'touch/core/Model',
    ''
  ],
  function UIAccount(
    UIAppPage,
    UITransactionList,
    AccountModel,
    UIQuickSummary,
    UIChart,
    UIForecastChart,
    UISection,
    Model,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        var accountId = this.getRequestParam('id');

        var balanceChartUrl = Url.createWithPath('/apix/account/balanceChart');
        balanceChartUrl.addQueryParam('id', accountId);

        return {
          account: new AccountModel({
            id: accountId
          }),
          balanceChart: Model.create({
            url: balanceChartUrl.toString()
          })
        };
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          accountId: this.getRequestParam('id')
        };
      },

      hasDynamicPageTitle: function hasDynamicPageTitle() {
        return true;
      },

      renderPageTitle: function renderPageTitle() {
        if (this.state.models.account) {
          return this.state.models.account.attributes.name;
        } else {
          return UIStaticText('ACCOUNTS');
        }
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement("div", null, 
            this._renderSummary(), 
            this._renderBalanceChart(), 
            React.createElement(UIForecastChart, {filters: this.getFilters(), title: UIStaticText('FORECAST')}), 
            React.createElement(UITransactionList, {filters: this.getFilters()})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        if (!this.state.models.account) {
          var labels = [
            UIStaticText('BALANCE')
          ];
          return React.createElement(UIQuickSummary, {labels: labels});
        }

        var account = this.state.models.account;

        var summaryItems = [];
        summaryItems.push({
          title: UIStaticText('BALANCE'),
          amount: account.attributes.balance,
          amountType: 'balance'
        });

        if (account.attributes.forecastBalance) {
          summaryItems.push({
            title: account.attributes.forecastBalance.title,
            amount: account.attributes.forecastBalance.balance,
            amountType: 'balance',
            href: '/forecast?accountId='+account.id
          });
        } else if (account.attributes.projectedBalance) {
          summaryItems.push({
            title: account.attributes.projectedBalance.title,
            amount: account.attributes.projectedBalance.balance,
            amountType: 'balance',
          });
        }

        return (
          React.createElement(UIQuickSummary, {items: summaryItems})
        );
      },

      _renderBalanceChart: function _renderBalanceChart() {
        var isLoading = true, content = null;
        if (this.state.models.balanceChart) {
          isLoading = false;
          content = React.createElement(UIChart, {data: this.state.models.balanceChart.attributes.chartData});
        }

        return (
          React.createElement(UISection, {label: UIStaticText('BALANCE'), isLoading: isLoading}, 
            content
          )
        );
      }
    });

  }
);



define(
  'touch/contacts/Model',[
    'touch/core/Model',
    ''
  ],
  function ContactModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/contact?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        return '/contact?id='+this.id;
      }

    });

  }
);



define(
  'touch/contacts/list/Model',[
    'touch/lists/Model',
    'touch/contacts/Model',
    ''
  ],
  function ContactListModel(
    ListModel,
    ContactModel,
    junk
  ) {

    return ListModel.createClass({
      url: '/apix/contacts',
      model: ContactModel
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/contacts/list/UIList',[
    'touch/contacts/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UIContactList(
    ContactListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        return new ContactListModel();
      },

      getClassNames: function getClassNames() {
        return ['UIContactList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO CONTACTS')
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/contacts/list/UIPage',[
    'touch/app/UIPage',
    'touch/contacts/list/UIList',
    ''
  ],
  function UIContactListPage(
    UIAppPage,
    UIContactList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('CONTACTS');
      },

      renderPageContent: function renderPageContent() {
        return React.createElement(UIContactList, null);
      }

    });

  }
);


/** @jsx React.DOM */


define(
  'touch/contacts/UIContact',[
    'touch/contacts/Model',
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    'touch/UIQuickSummary',
    ''
  ],
  function UIContactPage(
    ContactModel,
    UIAppPage,
    UITransactionList,
    UIQuickSummary,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        var contactModel = new ContactModel({
          id: this.getRequestParam('id')
        });
        return {
          contact: contactModel
        };
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          contacts: this.getRequestParam('id')
        };
      },

      hasDynamicPageTitle: function hasDynamicPageTitle() {
        return true;
      },

      renderPageTitle: function renderPageTitle() {
        if (this.state.models.contact) {
          return this.state.models.contact.attributes.name;
        } else {
          return UIStaticText('CONTACTS');
        }
      },

      renderPageContent: function renderPageContent() {
        var filters = this.getFilters();
        var summaryUrl = Url.createWithPath('/apix/contact/summary');
        summaryUrl.addQueryParams(filters);
        summaryUrl.addQueryParams({
          id: filters.contacts
        });

        var labels = [
          UIStaticText('BALANCE')
        ];
        return (
          React.createElement("div", null, 
            React.createElement(UIQuickSummary, {dataUrl: summaryUrl, labels: labels}), 
            React.createElement(UITransactionList, {filters: filters})
          )
        );
      }
    });


  }
);



define(
  'touch/tags/Model',[
    'touch/core/Model',
    ''
  ],
  function TagModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/tag?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        return '/tag?id='+this.id;
      }

    });

  }
);



define(
  'touch/tags/list/Model',[
    'touch/lists/Model',
    'touch/tags/Model',
    ''
  ],
  function TagListModel(
    ListModel,
    TagModel,
    junk
  ) {

    return ListModel.createClass({
      url: '/apix/tags',
      model: TagModel
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/tags/list/UIList',[
    'touch/tags/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UITagList(
    TagListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        var model = new TagListModel();
        model.sortField = 'name';
        model.sortOrder = 'ascending';
        return model;
      },

      getClassNames: function getClassNames() {
        return ['UITagList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO TAGS')
          )
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/tags/list/UIPage',[
    'touch/app/UIPage',
    'touch/tags/list/UIList',
    ''
  ],
  function UITagListPage(
    UIAppPage,
    UITagList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('TAGS');
      },

      renderPageContent: function renderPageContent() {
        return React.createElement(UITagList, null);
      }

    });

  }
);


/** @jsx React.DOM */


define(
  'touch/timeline/UITimeline',[
    'touch/core/Model',
    'touch/charts/UIChart',
    'touch/core/Period',
    'touch/app/UISection',
    'touch/app/UICard',
    ''
  ],
  function UITimeline(
    Model,
    UIChart,
    Period,
    UISection,
    UICard,
    junk
  ) {

    return React.createClass({

      mixins: [UICard],

      getModelsToFetch: function getModelsToFetch() {
        var dataUrl = Url.createWithPath('/apix/timeline/chart');
        dataUrl.addQueryParams(this.props.filters);
        dataUrl.addQueryParams({
          reportType: this.props.reportType
        });

        return {
          chart: Model.create({
            url: dataUrl.toString()
          })
        };
      },

      renderCard: function renderCard() {
        if (!this.state.models.chart) {
          return React.createElement(UISection, {className: "UITimeline", isLoading: true});
        }

        return (
          React.createElement(UISection, {className: "UITimeline"}, 
            React.createElement(UIChart, {data: this.state.models.chart.attributes.chartData})
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
          return UIStaticText('TIMELINE');
      },
     
      renderCardLabelMetadata: function renderCardLabelMetadata() {
        if (!this.state.models.chart) {
          return null;
        }

        var currentPeriodClassNames = ['lllMargin'];
        currentPeriodClassNames.push(this.props.reportType === 'expense' ? 'red' : 'green');

        return (
          React.createElement("span", null, 
            React.createElement("span", {className: currentPeriodClassNames.join(' ')}, 
              this.state.models.chart.attributes.selectedPeriodDescription
            ), 
            React.createElement("span", {className: "llMargin bgText"}, 
              UIStaticText('vs')
            ), 
            React.createElement("span", {className: "llMargin pastPeriodTitle"}, 
              this.state.models.chart.attributes.previousPeriodDescription
            )
          )
        );
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/compare/UICompare',[
    'touch/charts/UIChart',
    'touch/core/Model',
    'touch/app/UISection',
    'touch/app/UICard',
    ''
  ],
  function UICompare(
    UIChart,
    Model,
    UISection,
    UICard,
    junk
  ) {

    return React.createClass({

      mixins: [UICard],

      getModelsToFetch: function getModelsToFetch() {
        var dataUrl = Url.createWithPath('/apix/compare/chart');
        dataUrl.addQueryParams(this.props.filters);
        return {
          chart: Model.create({
            url: dataUrl.toString()
          })
        };
      },

      renderCard: function renderCard() {
        var isLoading = true, content = null;
        if (this.state.models.chart) {
          isLoading = false;
          content = React.createElement(UIChart, {chartType: "bar", data: this.state.models.chart.attributes.chartData});
        }

        return (
          React.createElement(UISection, {className: "UICompare", isLoading: isLoading}, 
            content
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
        return UIStaticText('COMPARE');
      },

      renderCardLabelMetadata: function renderCardLabelMetadata() {
        if (!this.state.models.chart) {
          return null;
        }

        return (
          React.createElement("span", null, 
            React.createElement("span", {className: "lllMargin green"}, 
              UIStaticText('INCOME')
            ), 
            React.createElement("span", {className: "llMargin bgText"}, 
              UIStaticText('vs')
            ), 
            React.createElement("span", {className: "llMargin red"}, 
              UIStaticText('EXPENSE')
            )
          )
        );
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/tags/UITag',[
    'touch/tags/Model',
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    'touch/UIQuickSummary',
    'touch/timeline/UITimeline',
    'touch/compare/UICompare',
    ''
  ],
  function UITagPage(
    TagModel,
    UIAppPage,
    UITransactionList,
    UIQuickSummary,
    UITimeline,
    UICompare,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        var tagModel = new TagModel({
          id: this.getRequestParam('id')
        });
        return {
          tag: tagModel
        };
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          tags: this.getRequestParam('id')
        };
      },

      hasDynamicPageTitle: function hasDynamicPageTitle() {
        return true;
      },

      renderPageTitle: function renderPageTitle() {
        if (this.state.models.tag) {
          return this.state.models.tag.attributes.name;
        } else {
          return UIStaticText('TAGS');
        }
      },

      renderPageContent: function renderPageContent() {
        var filters = this.getFilters();
        var chartFilters = filters;
        if (!filters.startDate && !filters.endDate && !filters.periodType) {
          chartFilters = _.clone(filters);
          chartFilters.periodType = 'month_current';
        }

        return (
          React.createElement("div", null, 
            this._renderSummary(), 
            React.createElement(UITimeline, {filters: chartFilters, reportType: "expense"}), 
            React.createElement(UICompare, {filters: chartFilters}), 
            React.createElement(UITransactionList, {filters: filters})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        var labels = [
          UIStaticText('SPENT'),
          UIStaticText('AVG. SPENT PER MONTH')
        ];

        var summaryUrl = Url.createWithPath('/apix/tag/summary');
        var params = this.getFilters();
        params.id = params.tags;
        delete params.tags;
        summaryUrl.addQueryParams(params);

        return React.createElement(UIQuickSummary, {dataUrl: summaryUrl, labels: labels});
      }

    });


  }
);



define(
  'touch/budgets/Model',[
    'touch/core/Model',
    ''
  ],
  function BudgetModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/budget?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        if (this.attributes.tagId === 0) {
          return null;
        }
        return '/budgets?tags='+this.attributes.tagId;
      }

    });

  }
);



define(
  'touch/budgets/list/Model',[
    'touch/lists/Model',
    'touch/budgets/Model',
    ''
  ],
  function BudgetListModel(
    ListModel,
    BudgetModel,
    junk
  ) {

    return ListModel.createClass({

      url: function() {
        var url = Url.createWithPath('/apix/budgets');
        if (this.filters) {
          url.addQueryParams(this.filters);
        }
        return url.toString();
      },

      model: BudgetModel,

      parseCustomResponse: function parseCustomResponse(response) {
        this.tagName = response.tagName;
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/budgets/list/UIList',[
    'touch/budgets/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UIBudgetList(
    BudgetListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        var model = new BudgetListModel();
        model.disablePagination = true;
        model.filters = _.clone(this.props.filters);
        model.filters.sortField = 0; // SORT_FIELD::BALANCE
        return model;
      },

      getClassNames: function getClassNames() {
        return ['UIBudgetList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO BUDGETS')
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
        return UIStaticText('BUDGETS');
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/budgets/list/UIPage',[
    'touch/app/UIPage',
    'touch/budgets/list/UIList',
    'touch/UIQuickSummary',
    'touch/transactions/list/UIList',
    'touch/timeline/UITimeline',
    'touch/compare/UICompare',
    ''
  ],
  function UIBudgetListPage(
    UIAppPage,
    UIBudgetList,
    UIQuickSummary,
    UITransactionList,
    UITimeline,
    UICompare,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('BUDGETS');
      },

      getPageFiltersToRender: function getPageFiltersToRender() {
        return ['tagId', 'period'];
      },

      renderPageContent: function renderPageContent() {
        var filters = this.getFilters();
        return (
          React.createElement("div", null, 
            this._renderSummary(), 
            React.createElement(UIBudgetList, {filters: filters}), 
            React.createElement(UITimeline, {filters: filters, reportType: "expense"}), 
            React.createElement(UICompare, {filters: filters}), 
            React.createElement(UITransactionList, {filters: filters})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        var labels = [
          UIStaticText('BUDGET BALANCE'),
          UIStaticText('BUDGET LIMIT'),
          UIStaticText('SPENT'),
          UIStaticText('AVG. SPENT PER MONTH')
        ];

        var summaryUrl = Url.createWithPath('/apix/budgets/summary');
        summaryUrl.addQueryParams(this.getFilters());

        return React.createElement(UIQuickSummary, {dataUrl: summaryUrl, labels: labels});
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          transactionType: -4, // ListFilters::TRANSACTION_TYPE_EXPENSE_ANY
          sortField: 6, // SORT_FIELD::ABSOLUTE_AMOUNT
          amountType: 'expense',
          periodType: 'month_current'
        };
      }
    });

  }
);


/** @jsx React.DOM */


define(
  'touch/budgets/UIBudget',[
    'touch/budgets/Model',
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    'touch/UIQuickSummary',
    ''
  ],
  function UIBudgetPage(
    BudgetModel,
    UIAppPage,
    UITransactionList,
    UIQuickSummary,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        var model = new BudgetModel({
          id: this.getRequestParam('id')
        });
        return {
          budget: model
        };
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        if (!this.state.models) {
          return {};
        }

        return {
          tags: this.state.models.budget.attributes.tagId
        };
      },

      hasDynamicPageTitle: function hasDynamicPageTitle() {
        return true;
      },

      renderPageTitle: function renderPageTitle() {
        if (this.state.models.budget) {
          return this.state.models.budget.attributes.name;
        } else {
          return UIStaticText('BUDGETS');
        }
      },

      renderPageContent: function renderPageContent() {
        if (!this.state.models) {
          return;
        }

        return (
          React.createElement("div", null, 
          this._renderSummary(), 
          React.createElement(UITransactionList, {filters: this.getFilters()})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        if (!this.state.models.budget) {
          var labels = [
            UIStaticText('BALANCE'),
            UIStaticText('LIMIT'),
            UIStaticText('SPENT')
          ];
          return React.createElement(UIQuickSummary, {labels: labels});
        }

        var budget = this.state.models.budget;

        var summaryItems = [
          {
            title: UIStaticText('BALANCE'),
            amount: budget.attributes.balance,
            amountType: 'balance'
          },
          {
            title: UIStaticText('LIMIT'),
            amount: budget.attributes.limit,
            amountType: 'none',
            shouldDisplaySign: false
          },
          {
            title: UIStaticText('SPENT'),
            amount: budget.attributes.spent,
            amountType: 'expense'
          }
        ];

        return React.createElement(UIQuickSummary, {items: summaryItems});
      }


    });


  }
);



define(
  'touch/groups/Model',[
    'touch/core/Model',
    ''
  ],
  function GroupModel(
    Model,
    junk
  ) {

    return Model.createClass({

      url: function url() {
        return '/apix/group?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        return '/group?id='+this.id;
      }

    });

  }
);



define(
  'touch/groups/list/Model',[
    'touch/lists/Model',
    'touch/groups/Model',
    ''
  ],
  function GroupListModel(
    ListModel,
    GroupModel,
    junk
  ) {

    return ListModel.createClass({
      url: '/apix/groups',
      model: GroupModel
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/groups/list/UIList',[
    'touch/groups/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UIGroupList(
    GroupListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        return new GroupListModel();
      },

      getClassNames: function getClassNames() {
        return ['UIGroupList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO GROUPS')
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/groups/list/UIPage',[
    'touch/app/UIPage',
    'touch/groups/list/UIList',
    ''
  ],
  function UIGroupListPage(
    UIAppPage,
    UIGroupList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('GROUPS');
      },

      renderPageContent: function renderPageContent() {
        return React.createElement(UIGroupList, null);
      }

    });

  }
);


/** @jsx React.DOM */


define(
  'touch/groups/UIBalances',[
    'touch/core/Model',
    'touch/app/UICard',
    'touch/app/UISection',
    'touch/app/UISectionRow',
    'touch/ui/UILeftRight',
    'touch/ui/UIAmount',
    'touch/ui/UIText',
    'touch/auth/Session',
    ''
  ],
  function UIGroupBalances(
    Model,
    UICard,
    UISection,
    UISectionRow,
    UILeftRight,
    UIAmount,
    UIText,
    Session,
    junk
  ) {

    return React.createClass({

      mixins: [UICard],

      getModelsToFetch: function getModelsToFetch() {
        ASSERT_TRUTHY(this.props.groupId, '[UIGroupBalances] Expected non-empty group id');
        return {
          balances: Model.create({
            url: '/apix/group/balances?id='+this.props.groupId
          })
        };
      },

      renderCard: function renderCard() {
        if (!this.state.models.balances) {
          return React.createElement(UISection, {className: "UIGroupBalances", isLoading: true});
        }

        var memberBalances = this.state.models.balances.attributes.balances;

        if (!memberBalances.length) {
          return (
            React.createElement(UISection, {className: "UIGroupBalances"}, 
              React.createElement("div", {className: "emptyMessageContainer bgText"}, 
                UIStaticText('NO PENDING LOANS')
              )
            )
          );
        }

        var balanceRows = [];
        ArrayUtils.each(memberBalances, function(index, balanceInfo) {
          var contactUrl = null;
          if (balanceInfo.contactId !== Session.getLoggedInUid()) {
            contactUrl = '/contact?id='+balanceInfo.contactId;
          }
          balanceRows.push(
            React.createElement(UISectionRow, {href: contactUrl}, 
              React.createElement(UILeftRight, null, 
                React.createElement(UIText, {maxLength: "20"}, balanceInfo.name), 
                React.createElement(UIAmount, {amount: balanceInfo.balance})
              )
            )
          );
        });

        return (
          React.createElement(UISection, {className: "UIGroupBalances"}, 
            balanceRows
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
        if (this.state.models.balances && !this.state.models.balances.attributes.balances.length) {
          return null;
        }
        return UIStaticText('BALANCES');
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/groups/UIGroup',[
    'touch/groups/Model',
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    'touch/groups/UIBalances',
    ''
  ],
  function UIGroupPage(
    GroupModel,
    UIAppPage,
    UITransactionList,
    UIGroupBalanceList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        return {
          group: new GroupModel({
            id: this.getRequestParam('id')
          })
        };
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          groupId: this.getRequestParam('id')
        };
      },

      hasDynamicPageTitle: function hasDynamicPageTitle() {
        return true;
      },

      renderPageTitle: function renderPageTitle() {
        if (this.state.models.group) {
          return this.state.models.group.attributes.name;
        } else {
          return UIStaticText('GROUPS');
        }
      },

      renderPageContent: function renderPageContent() {
        var filters = this.getFilters();
        return (
          React.createElement("div", null, 
            React.createElement(UIGroupBalanceList, {groupId: filters.groupId}), 
            React.createElement(UITransactionList, {filters: filters})
          )
        );
      }
    });
  }
);



define(
  'touch/loans/list/Model',[
    'touch/lists/Model',
    'touch/contacts/Model',
    'touch/groups/Model',
    ''
  ],
  function LoanListModel(
    ListModel,
    ContactModel,
    GroupModel,
    junk
  ) {

    return ListModel.createClass({

      url: '/apix/loans',

      model: function(attributes, options) {
        if (attributes.type == 'contact') {
          return new ContactModel(attributes, options);
        }

        if (attributes.type == 'group') {
          return new GroupModel(attributes, options);
        }

        BUG('Invalid loan model type ['+attributes.type+']');
      }
    
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/loans/list/UIList',[
    'touch/loans/list/Model',
    'touch/lists/UIList',
    ''
  ],
  function UILoanList(
    LoanListModel,
    UIModelList,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        return new LoanListModel();
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/loans/list/UIPage',[
    'touch/app/UIPage',
    'touch/loans/list/UIList',
    'touch/transactions/list/UIList',
    ''
  ],
  function UILoanListPage(
    UIAppPage,
    UILoanList,
    UITransactionList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('LOANS');
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement("div", null, 
            React.createElement(UILoanList, null), 
            React.createElement(UITransactionList, {filters: this.getFilters()})
          )
        );
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          transactionType: -3, // ListFilters::TRANSACTION_TYPE_SHARED
        };
      }
    });

  }
);


/** @jsx React.DOM */


define(
  'touch/forecast/UITransactionList',[
    'touch/lists/Model',
    'touch/transactions/Model',
    'touch/transactions/list/UIList',
    ''
  ],
  function UIForecastTransactionList(
    ListModel,
    TransactionModel,
    UITransactionList,
    junk
  ) {

    return React.createClass({

      createListModel: function createListModel() {
        var filters = this.props.filters;

        var model = ListModel.createClass({

          url: function() {
            var url = Url.createWithPath('/apix/forecast/transactions');
            if (filters) {
              url.addQueryParams(filters);
            }
            return url.toString();
          },

          model: TransactionModel
        });
        return new model;
      },

      compose: function compose() {
        return (
          React.createElement(UITransactionList, {filters: this.filters, extension: this})
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/forecast/UIForecast',[
    'touch/core/Model',
    'touch/app/UIPage',
    'touch/UIQuickSummary',
    'touch/forecast/UITransactionList',
    'touch/forecast/UIChart',
    ''
  ],
  function UIForecast(
    Model,
    UIAppPage,
    UIQuickSummary,
    UIForecastTransactionList,
    UIForecastChart,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getPageFiltersToRender: function getPageFiltersToRender() {
        return ['accountId', 'period'];
      },

      shouldSelectFuturePeriod: function shouldSelectFuturePeriod() {
        return true;
      },

      hasTopLevelAccessControl: function hasTopLevelAccessControl() {
        return true;
      },

      getPageContentCards: function getPageContentCards() {
        var filters = this.getFilters();
        return [
          this._renderSummary(),
          React.createElement(UIForecastChart, {filters: filters}),
          React.createElement(UIForecastTransactionList, {filters: filters})
        ];
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('FORECAST');
      },

      _renderSummary: function _renderSummary() {
        var url = Url.createWithPath('/apix/forecast/summary');
        url.addQueryParams(this.getFilters());

        var labels = [
          UIStaticText('NET WORTH TODAY'),
          UIStaticText('INCOME FORECAST'),
          UIStaticText('SCHEDULED BILLS'),
          UIStaticText('BUDGETED EXPENSES'),
          UIStaticText('NET WORTH FORECAST')
        ];

        return React.createElement(UIQuickSummary, {labels: labels, dataUrl: url});
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          periodType: 'month_next_3'
        };
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/reminders/list/UIList',[
    'touch/core/Model',
    'touch/lists/Model',
    'touch/lists/UIList',
    ''
  ],
  function UIReminderList(
    Model,
    ListModel,
    UIModelList,
    junk
  ) {

    var ReminderModel = Model.createClass({
      url: function url() {
        return '/apix/reminder?id='+this.id;
      },

      getNavUrl: function getNavUrl() {
        return '/reminder?id='+this.id;
      }
    });

    var ReminderListModel = ListModel.createClass({
      url: '/apix/reminders',
      model: ReminderModel
    });

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        return new ReminderListModel();
      },

      getClassNames: function getClassNames() {
        return ['UIReminderList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO REMINDERS')
          )
        );
      },

      renderNameForModel: function renderNameForModel(reminder) {
        return reminder.attributes.description;
      },

      renderBalanceForModel: function renderBalanceForModel(reminder) {
        return reminder.attributes.dueDateDescription;
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/reminders/list/UIPage',[
    'touch/app/UIPage',
    'touch/reminders/list/UIList',
    ''
  ],
  function UIReminderListPage(
    UIAppPage,
    UIReminderList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('REMINDERS');
      },

      renderPageContent: function renderPageContent() {
        return React.createElement(UIReminderList, null);
      }

    });

  }
);


/** @jsx React.DOM */


define(
  'touch/charts/UIPieChart',[
    'touch/core/Model',
    'touch/ui/UIGrid',
    'touch/ui/UIAmount',
    'touch/app/UISection',
    'touch/app/UISectionRow',
    'touch/app/UICard',
    ''
  ],
  function UIPieChart(
    Model,
    UIGrid,
    UIAmount,
    UISection,
    UISectionRow,
    UICard,
    junk
  ) {

    var colors = [
      '#FD9E41', // Orange
      '#AE7DCB', // Purple
      '#3997D5', // Blue
      '#399639', // Green
      '#333F46', // Black
      '#F39DFD', // Pink
      '#FEE533'  // Yellow
    ];
    var numColors = colors.length;

    return React.createClass({

      mixins: [UICard],

      getDefaultProps: function getDefaultProps() {
        return {
          strokeWidth: 3,
          innerRadius: 0.7
        };
      },

      getModelsToFetch: function getModelsToFetch() {
        var chartModel = Model.create({
          url: this.props.dataUrl
        });
        return {
          chart: chartModel
        };
      },

      componentDidMount: function componentDidMount() {
        this._refreshPlot();
      },

      componentDidUpdate: function componentDidUpdate() {
        this._refreshPlot();
      },

      _refreshPlot: function _refreshPlot() {
        if (
          !this.state.models.chart
          || !this.state.models.chart.attributes.chartData.length
        ) {
          return;
        }

        var container = DOM.findById(this.refs.chartContainer.getDOMNode());

        var chartData = this.state.models.chart.attributes.chartData;
        var plotData = [];
        ArrayUtils.each(chartData, FunctionUtils.bind(this, function(index, data) {
          data.color = this._getColorAtIndex(index);
          plotData.push(data);
        }));

        container.plot(plotData.reverse(), this._getPlotOptions());

        container.bind('click', function(event) {
          Event.stop(event);
        });

        if (this.props.onClick) {
          container.bind('plotclick', FunctionUtils.bind(this, function(event, pos, obj) {
            var sliceIndex = obj.seriesIndex;
            var sliceData = plotData[sliceIndex];
            this.props.onClick(sliceData);
          }));
        }
      },

      _renderLegend: function _renderLegend() {
        var chartData = this.state.models.chart.attributes.chartData;
        if (!chartData.length) {
          return;
        }

        var cells = [];
        var sliceUrlGenerator = this.props.extension.getUrlForSlice;
        ArrayUtils.each(chartData, FunctionUtils.bind(this, function(index, legendItem) {
          var itemUrl = sliceUrlGenerator && sliceUrlGenerator(legendItem);
          var iconStyle = {
            backgroundColor: this._getColorAtIndex(index)
          };
          cells.push(
            React.createElement(UISectionRow, {href: itemUrl, key: 'legend-'+index}, 
              React.createElement("div", {className: "lF legendIconContainer", style: iconStyle}), 
              React.createElement("div", {className: "lF"}, legendItem.label), 
              React.createElement(UIAmount, {className: "rF", amount: legendItem.data, amountType: "expense"}), 
              React.createElement("div", {className: "cF"})
            )
          );
        }));

        return cells;
      },

      renderCard: function renderCard() {
        var isLoading = true, content = null;
        if (this.state.models.chart) {
          if (!this.state.models.chart.attributes.chartData.length) {
            return null;
          }

          isLoading = false;
          var chartContainerClassNames = ['chartContainer'];
          if (this.props.onClick) {
            chartContainerClassNames.push('clickable');
          }
          content = (
            React.createElement("div", null, 
              React.createElement("div", {className: "aaPadding"}, 
                React.createElement("div", {className: chartContainerClassNames.join(' '), ref: "chartContainer"})
              ), 
              this._renderLegend()
            )
          );
        }

        return (
          React.createElement(UISection, {className: "UIPieChart", isLoading: isLoading}, 
            content
          )
        );
      },

      renderCardLabel: function renderCardLabel() {
        return this.props.title;
      },

      _getPlotOptions: function _getPlotOptions() {
        return {
          series: {
            pie: {
              show: true,
              radius: 1,
              innerRadius: this.props.innerRadius,
              stroke: {
                width: this.props.strokeWidth
              },
              label: {
                show: false
              }
            }
          },
          grid: {
            clickable: this.props.onClick ? true : false
          },
          legend: {
            show: false
          }
        };
      },

      _getColorAtIndex: function _getColorAtIndex(index) {
        if (index > 0) {
          index = ((index - 1) % (numColors - 1)) + 1;
        }
        return colors[index];
      },

      getUrlForSlice: function getUrlForSlice(sliceData) {
        return null;
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/categories/UIChart',[
    'touch/charts/UIPieChart',
    'touch/app/NavigationController',
    ''
  ],
  function UICategoriesChart(
    UIPieChart,
    AppNavigationController,
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        var dataUrl = Url.createWithPath('/apix/categories/chart');
        dataUrl.addQueryParams(this.props.filters);
        dataUrl.addQueryParam('reportType', this.props.reportType);

        var chartTitle = (this.props.reportType == 'income') ? UIStaticText('INCOME') : UIStaticText('EXPENSE');
        return (
          React.createElement(UIPieChart, {dataUrl: dataUrl, extension: this, title: chartTitle, onClick: this._didClickChart})
        );
      },

      _didClickChart: function _didClickChart(sliceData) {
        AppNavigationController.openUrl(this.getUrlForSlice(sliceData));
      },

      getUrlForSlice: function getUrlForSlice(sliceData) {
        ASSERT_TRUTHY(sliceData.tagId, '[UICategoriesChart] Expected non-empty tagId for slice');
        var url = Url.createWithPath('/reports');
        url.addQueryParams(this.props.filters);
        url.addQueryParams({
          reportType: this.props.reportType,
          tags: sliceData.tagId
        });
        return url.toString();
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/reports/UIReport',[
    'touch/core/Model',
    'touch/app/UIPage',
    'touch/UIQuickSummary',
    'touch/transactions/list/UIList',
    'touch/categories/UIChart',
    'touch/timeline/UITimeline',
    'touch/compare/UICompare',
    'touch/charts/UIChart',
    'touch/ui/UIPicker',
    'touch/app/UISectionRow',
    'touch/ui/UIPopup',
    ''
  ],
  function UIReport(
    Model,
    UIAppPage,
    UIQuickSummary,
    UITransactionList,
    UICategoriesChart,
    UITimeline,
    UICompare,
    UIChart,
    UIPicker,
    UISectionRow,
    UIPopup,
    junk
  ) {

    var reportTypeLabels = {
      networth: UIStaticText('NET WORTH'),
      expense: UIStaticText('EXPENSE'),
      income: UIStaticText('INCOME')
    };

    return React.createClass({

      mixins: [UIAppPage],

      getInitialState: function getInitialState() {
        return {
          reportType: this.getRequestParam('reportType')
        };
      },

      getPageFiltersToRender: function getPageFiltersToRender() {
        return ['tagId', 'period'];
      },

      renderPageContent: function renderPageContent() {
        var filters = this.getFilters();
        var charts;
        if (this.state.reportType == 'networth') {
          var dataUrl = Url.createWithPath('/apix/reports/netWorthChart');
          dataUrl.addQueryParams(filters);
          charts = (
            React.createElement("div", {className: "UISection"}, 
              React.createElement("div", {className: "sectionLabel bFontWeight"}, 
                UIStaticText('NET WORTH')
              ), 
              React.createElement(UIChart, {dataUrl: dataUrl})
            )
          );
        } else {
          charts = [
            React.createElement(UICategoriesChart, {filters: filters, reportType: this.state.reportType, key: "chart-categories"}),
            React.createElement(UITimeline, {filters: filters, reportType: this.state.reportType, key: "chart-timeline"}),
            React.createElement(UICompare, {filters: filters, key: "chart-compare"})
          ];
        }

        return (
          React.createElement("div", null, 
            this._renderSummary(), 
            charts, 
            React.createElement(UITransactionList, {filters: this._getTransactionListFilters()})
          )
        );
      },

      _renderSummary: function _renderSummary() {
        var filters = this.getFilters();

        var labels;
        if (this.state.reportType == 'networth') {
          labels = [
            UIStaticText('NET WORTH'),
            UIStaticText('THIS MONTH'),
          ];
        } else if (this.state.reportType === 'income') {
          labels = [
            UIStaticText('INCOME'),
          ];
        } else {
          labels = [
            UIStaticText('EXPENSE'),
          ];
        }

        var reportTypes = ['networth', 'income', 'expense'];
        var hrefs = {};
        var url = Url.createWithPath('/reports');
        url.addQueryParams(filters);
        ArrayUtils.each(reportTypes, function(index, reportType) {
          url.addQueryParam('reportType', reportType);
          hrefs[reportType] = url.toString();
        });

        var summaryUrl = Url.createWithPath('/apix/reports/summary');
        summaryUrl.addQueryParam('reportType', this.state.reportType);
        summaryUrl.addQueryParams(filters);

        return React.createElement(UIQuickSummary, {labels: labels, hrefs: hrefs, dataUrl: summaryUrl});
      },

      getBrowserPageTitle: function getBrowserPageTitle() {
        return 'REPORTS';
      },

      renderPageTitle: function renderPageTitle() {
        var label = ArrayUtils.get(reportTypeLabels, this.state.reportType);
        return (
          React.createElement(UIPicker, {isCentered: true, onClick: this._didClickReportTypePicker}, 
            'REPORTS / '+label
          )
        );
      },

      _didClickReportTypePicker: function _didClickReportTypePicker() {
        var url = Url.createWithPath('/reports');
        url.addQueryParams(this.getFilters());

        var children = [];
        ArrayUtils.each(reportTypeLabels, FunctionUtils.bind(this, function(reportType, label) {
          var didClick = FunctionUtils.bind(this, function() {
            url.addQueryParam('reportType', reportType);
            AppNavigationController.openUrl(url.toString());
          })
          var classNames = [];
          if (reportType === this.state.reportType) {
            classNames.push('selected');
          }
          children.push(
            React.createElement(UISectionRow, {className: classNames.join(' '), onClick: didClick, key: 'type-'+reportType}, 
              label
            )
            );
        }));
        AppNavigationController.presentPopup(
          React.createElement(UIPopup, {title: UIStaticText('SELECT REPORT TYPE')}, 
            children
          )
        );
      },

      getDefaultParamValues: function getDefaultParamValues() {
        return {
          reportType: 'expense'
        };
      },

      _getTransactionListFilters: function _getTransactionListFilters() {
        var transactionType, amountType, sortField;
        switch (this.state.reportType) {
          case 'expense':
            transactionType = -4; // ListFilters::TRANSACTION_TYPE_EXPENSE_ANY
            amountType = 'expense';
            sortField = 6; // SORT_FIELD::ABSOLUTE_AMOUNT
            break;
          case 'income':
            transactionType = 4; // Transaction::TYPE_INCOME
            amountType = 'expense';
            sortField = 6; // SORT_FIELD::ABSOLUTE_AMOUNT
          default:
            transactionType = -1; // ListFilters::TRANSACTION_TYPE_ANY
            amountType = 'paid';
            sortField = 2; // SORT_FIELD::DATE
        }
        var filters = _.clone(this.getFilters());
        return Object.extend(filters, {
          sortField: sortField,
          amountType: amountType,
          transactionType: transactionType
        });
      },

      getImplicitFilterParams: function getImplicitFilterParams() {
        return {
          periodType: 'month_current'
        };
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIGridCell',[
    ''
  ],
  function UIGridCell(
    junk
  ) {

    return React.createClass({

      displayName: 'UIGridCell',

      compose: function compose() {
        return (
          React.createElement("td", {isTableCell: true}, 
            this.props.children
          )
        );
      }

    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UICalendar',[
    ''
  ],
  function UICalendar(
    junk
  ) {

    return React.createClass({

      compose: function compose() {
        return (
          React.createElement("div", {className: "UICalendar"})
        );
      },

      componentDidMount: function componentDidMount() {
        var container = DOM.findById(this.getDOMNode());

        var options = {
          dateFormat: 'yy-mm-dd',
          showAnim: 'slideDown',
          showOtherMonths: true,
          selectOtherMonths: true,
          defaultDate: this.props.date,
          dayNamesMin: [ 'SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA' ]
        };

        var onSelect = this.props.onSelect;
        if (onSelect) {
          options.onSelect = function(date, datePicker) {
            onSelect(date);
          };
        }

        container.datepicker(options);
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIDatePicker',[
    'touch/app/NavigationController',
    'touch/ui/UIPopup',
    'touch/ui/UICalendar',
    'touch/ui/UIDate',
    ''
  ],
  function UIDatePicker(
    AppNavigationController,
    UIPopup,
    UICalendar,
    UIDate,
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          date: this.props.date || DateUtils.today(),
        };
      },

      compose: function compose() {
        return (
          React.createElement("div", {className: "UIDatePicker", onClick: this._didClickPicker}, 
            this._maybeRenderDate(), 
            React.createElement("input", {type: "hidden", name: "date", value: this.state.date})
          )
        );
      },

      _maybeRenderDate: function _maybeRenderDate() {
        if (this.props.showSelectedDate === false) {
          return;
        }

        return React.createElement(UIDate, {date: this.state.date});
      },

      _didClickPicker: function _didClickPicker() {
        AppNavigationController.presentPopup(this._renderPopup());
      },

      _renderPopup: function _renderPopup() {
        return (
          React.createElement(UIPopup, {title: UIStaticText('SELECT DATE')}, 
            React.createElement(UICalendar, {date: this.state.date, onSelect: this._didSelectDate})
          )
        );
      },

      _didSelectDate: function _didSelectDate(date) {
        AppNavigationController.dismissPopup();
        this.setState({
          date: date
        });

        if (this.props.onSelect) {
          this.props.onSelect(date);
        }
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/accounts/picker/UIList',[
    'touch/accounts/list/Model',
    'touch/lists/UIList',
    'touch/app/NavigationController',
    'touch/app/UISectionRow',
    ''
  ],
  function UIAccountPickerList(
    AccountListModel,
    UIModelList,
    AppNavigationController,
    UISectionRow,
    junk
  ) {

    return React.createClass({

      mixins: [UIModelList],

      createListModel: function createListModel() {
        var model = new AccountListModel();
        model.disablePagination = true;
        model.sortField = 'name';
        model.sortOrder = 'ascending';
        return model;
      },

      getModelsToRender: function getModelsToRender() {
        var models = this.state.models.list.models;

        if (this.state.models.list.isEmpty()) {
          return models;
        }

        if (this.props.enableNoAccountOption) {
          models = ArrayUtils.prepend(models, {
            id: 0,
            attributes: {
              id: 0,
              name: UIStaticText('NO ACCOUNT')
            }
          });
        }

        if (this.props.enableAllAccountsOption) {
          models = ArrayUtils.prepend(models, {
            id: null,
            attributes: {
              id: null,
              name: UIStaticText('ALL ACCOUNTS')
            }
          });
        }

        return models;
      },

      getClassNames: function getClassNames() {
        return ['UIAccountList'];
      },

      renderEmptyState: function renderEmptyState() {
        return (
          React.createElement("div", {className: "emptyStateContainer"}, 
            UIStaticText('NO ACCOUNTS')
          )
        );
      },

      renderModel: function renderModel(account) {
        var _didSelectAccount = FunctionUtils.bind(this, function _didSelectAccount() {
          AppNavigationController.dismissModal();
          this.props.onSelect(account);
        });
        return (
          React.createElement(UISectionRow, {isClickable: true, onClick: _didSelectAccount}, 
            account.attributes.name
          )
        );
      },

    });

  }
);


/** @jsx React.DOM */


define(
  'touch/accounts/picker/UIPage',[
    'touch/app/UIPage',
    'touch/accounts/picker/UIList',
    ''
  ],
  function UIAccountPickerListPage(
    UIAppPage,
    UIAccountPickerList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('SELECT ACCOUNT');
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement(UIAccountPickerList, {
            enableNoAccountOption: this.props.enableNoAccountOption, 
            enableAllAccountsOption: this.props.enableAllAccountsOption, 
            onSelect: this.props.onSelect}
          )
        );
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIAmountInput',[
    ''
  ],
  function UIAmountInput(
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          amount: this.props.defaultValue || 0,
          isFocused: false
        };
      },

      compose: function compose() {
        var value = '';
        if (this.state.amount) {
          value = this.state.amount.toFixed(2);
        } else {
          value = '0.00';
        }

        return (
          React.createElement("input", {
            className: "UIAmountInput fixedWidthFont", 
            type: "tel", 
            value: value, 
            onChange: this._didChange, 
            onFocus: this._didFocus, 
            onBlur: this._didBlur}
          )
        );
      },

      _didChange: function _didChange(event) {
        var value = event.target.value || '';
        if (!value.length) {
          value = 0;
        } else {
          value = value.replace(/[^0-9]/g, '');
          value = parseInt(value) / 100;
        }

        this.setState({
          amount: value
        });
      },

      _didBlur: function _didBlur(event) {
        this.setState({
          isFocused: false
        });
      },

      _didFocus: function _didFocus(event) {
        if (this.state.isFocused) {
          return;
        }

        this.setState({
          isFocused: true
        });

        var elem = event.target;
        var originalValue = elem.value || '';
        var selectionIndex = originalValue.length;

        if (selectionIndex == 0) {
          return;
        }

        setTimeout(
          function() {
            elem.selectionStart = selectionIndex;
            elem.selectionEnd = selectionIndex;
          },
          0
        );
      }

    });

  }
);



define(
  'touch/search/ResultsModel',[
    'touch/core/Model',
    ''
  ],
  function SearchResultsModel(
    Model,
    junk
  ) {

    return {

      create: function create(params) {
        var model = Model.createCollection({

          url: function() {
            var url = Url.createWithPath('/apix/search');
            url.addQueryParams({
              query: params.query,
              type: params.type
            });
            return url.toString();
          }

        });
        model.query = params.query;
        return model;
      }
    }
  }
);

/** @jsx React.DOM */


define(
  'touch/ui/UIAutocompleteInput',[
    'touch/app/Runtime',
    'touch/search/ResultsModel',
    'touch/ui/UIHorizontalList',
    ''
  ],
  function UIAutocompleteInput(
    Runtime,
    SearchResultsModel,
    UIHorizontalList,
    junk
  ) {

    return React.createClass({

      getInitialState: function getInitialState() {
        return {
          value: this.props.value || this.props.defaultValue || '',
          showSuggestions: false
        };
      },

      render: function render() {
        ASSERT_TRUTHY(this.props.autocompleteType, '[UIAutocompleteInput] Expected non-empty type');

        var inputField =
          React.createElement("input", {
            className: "UIAutocompleteInput", 
            type: "text", 
            autoCorrect: "off", 
            autoCapitalize: "off", 
            value: this.state.value, 
            onChange: this._didChangeValue, 
            // onBlur={this._didBlur} Blur conflicts with click on suggestion
            name: this.props.name, 
            placeholder: this.props.placeholder, 
            ref: "inputField"}
          );
        inputField = this.transferPropsTo(inputField);

        return (
          React.createElement("div", null, 
            inputField, 
            this._maybeRenderSuggestions()
          )
        );
      },

      focusInput: function focusInput() {
        this.focus(this.refs.inputField);
      },

      _didBlur: function _didBlur(event) {
        this._setLoading(false);
        this.setState({
          showSuggestions: false,
          searchResults: []
        });
      },

      _didChangeValue: function _didChangeValue(event) {
        var rawValue = event.target.value || '';

        if (this.props.onChange) {
          this.props.onChange(rawValue);
        }

        this._setQuery(rawValue);
      },

      _setQuery: function _setQuery(rawValue) {
        rawValue = rawValue || '';
        var query;
        if (!this.props.allowMultipleValues || !rawValue.length) {
          query = rawValue;
        } else {
          var values = StringUtils.explodeCSV(rawValue);
          query = values[values.length - 1];
        }

        this.setState({
          value: rawValue,
          query: query,
          searchResults: [],
          showSuggestions: query.length > 0
        });

        if (!query.length) {
          this._setLoading(false);
          return;
        }

        var resultsModel = SearchResultsModel.create({
          type: this.props.autocompleteType,
          query: query
        });

        var _didFailFetch = FunctionUtils.bind(this, function(model) {
          if (model.query !== this.state.query) {
            return;
          }
          this._setLoading(false);
        });

        var _didFetch = FunctionUtils.bind(this, function(model) {
          if (model.query !== this.state.query) {
            return;
          }

          this.setState({
            searchResults: resultsModel.models
          });
          this._setLoading(false);
        });

        resultsModel.fetch({
          success: _didFetch,
          error: _didFailFetch
        });
        this._setLoading(true);
      },

      _setLoading: function _setLoading(isLoading) {
        DOM.setLoading(this.refs.inputField.getDOMNode(), isLoading);
      },

      _maybeRenderSuggestions: function _maybeRenderSuggestions() {
        if (!this.state.showSuggestions) {
          return;
        }

        var lowerCaseQuery = this.state.query.toLowerCase();
        var resultsToRender = [];
        var matchForQuery;
        ArrayUtils.each(this.state.searchResults, function(index, result) {
          var resultName = result.attributes.name;

          if (resultName.toLowerCase() === lowerCaseQuery) {
            matchForQuery = resultName;
            return true;
          }

          resultsToRender.push(resultName);
        });

        if (!matchForQuery) {
          matchForQuery = this.state.query;
        }
        resultsToRender = ArrayUtils.prepend(resultsToRender, matchForQuery);

        var cells = [];
        ArrayUtils.each(resultsToRender, FunctionUtils.bind(this, function(index, resultName) {
          var displayName = resultName;
          // Android Chrome annoyingly traps all clicks on a clickable email
          // container. You can get precedence using onTouchStart, but then you
          // lose horizontal scroll.
          if (Runtime.isAndroid) {
            displayName = resultName.replace('@', '@ ');
          }
          cells.push(
            React.createElement("div", {onClick: this._didClickSuggestion, "data-suggestion": resultName, className: "suggestionContainer"}, 
              displayName
            )
          );
        }));

        return (
          React.createElement("div", {className: "UIAutocompleteSuggestions layerOverlay"}, 
            React.createElement(UIHorizontalList, null, 
              cells
            )
          )
        );
      },

      _didClickSuggestion: function _didClickSuggestion(event) {
        Event.stop(event);

        var suggestionValue = DOM.getAttribute(event.target, 'data-suggestion');
        var inputValue;

        if (!this.props.allowMultipleValues) {
          inputValue = suggestionValue;
        } else {
          var values = StringUtils.explodeCSV(this.state.value);
          values[values.length - 1] = suggestionValue;
          inputValue = values.join(', ')+', ';
        }

        this.setState({
          value: inputValue,
          showSuggestions: false
        });

        if (this.props.onChange) {
          this.props.onChange(inputValue);
        }

        this.focusInput();
        return false;
      }
    });
  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/input/UIInput',[
    'touch/transactions/Model',
    'touch/app/UIPage',
    'touch/ui/UILeftRight',
    'touch/ui/UIForm',
    'touch/app/NavigationController',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIGrid',
    'touch/ui/UIGridCell',
    'touch/ui/UIDatePicker',
    'touch/accounts/picker/UIPage',
    'touch/ui/UIPicker',
    'touch/ui/UIPopup',
    'touch/ui/UIAmountInput',
    'touch/ui/UIAutocompleteInput',
    'touch/app/UISectionRow',
    'touch/transactions/offline/Manager',
    'touch/ui/UIButton',
    'touch/auth/Session',
    ''
  ],
  function UITransactionInput(
    TransactionModel,
    UIAppPage,
    UILeftRight,
    UIForm,
    AppNavigationController,
    UIHorizontalList,
    UIGrid,
    UIGridCell,
    UIDatePicker,
    UIAccountPickerListPage,
    UIPicker,
    UIPopup,
    UIAmountInput,
    UIAutocompleteInput,
    UISectionRow,
    OfflineTransactionsManager,
    UIButton,
    Session,
    junk
  ) {

    var transactionTypeLabels = {
      expense: UIStaticText('EXPENSE'),
      income: UIStaticText('INCOME'),
      transfer: UIStaticText('TRANSFER'),
      refund: UIStaticText('REFUND'),
      sharedBill: UIStaticText('SHARED BILL'),
      paidForFriend: UIStaticText('PAID FOR FRIEND'),
      loan: UIStaticText('LOAN'),
      settlement: UIStaticText('SETTLEMENT')
    };

    return React.createClass({

      mixins: [UIAppPage],

      canWorkOffline: function canWorkOffline() {
        return true;
      },

      getInitialState: function getInitialState() {
        var areSecondaryInputsExpanded = false;
        var defaultAccount = Session.getPreference('defaultAccount');
        if (!defaultAccount) {
          defaultAccount = {
            id: 0,
            name: UIStaticText('NO ACCOUNT')
          };
        } else {
          areSecondaryInputsExpanded = true;
        }
        var state = {
          transactionType: this.props.transactionType || 'expense',
          tagNames: '',
          amount: 0,
          account: defaultAccount,
          date: DateUtils.today(),
          payerNames: [UIStaticText('Me')],
          payerAmounts: [],
          sharerNames: [UIStaticText('Me')],
          sharerAmounts: [],
          hasMultiplePayers: false,
          splitType: 'even'
        };
        if (this.props.transaction) {
          state.amount = parseFloat(this.props.transaction.attributes.totalAmount);
          state.transactionType = this.props.transaction.attributes.transactionType;
          state.date = this.props.transaction.attributes.date;

          var accountFields = ['fromAccount', 'toAccount', 'account'];
          ArrayUtils.each(accountFields, FunctionUtils.bind(this, function(fieldIndex, fieldName) {
            var fieldValue = this.props.transaction.attributes[fieldName];
            if (!fieldValue) {
              fieldValue = {
                id: 0,
                name: UIStaticText('NO ACCOUNT')
              };
            } else {
              areSecondaryInputsExpanded = true;
            }
            state[fieldName] = fieldValue;
          }));

          state.tagNames = this.props.transaction.attributes.tagNames.join(', ');
          if (state.tagNames.length > 0) {
            areSecondaryInputsExpanded = true;
          }

          if (this.props.transaction.attributes.sharers) {
            var payerNames = [];
            var payerAmounts = [];
            var sharerNames = [];
            var sharerAmounts = [];
            ArrayUtils.each(this.props.transaction.attributes.sharers, FunctionUtils.bind(this, function(index, sharerInfo) {
              if (sharerInfo.paidAmount > 0) {
                payerNames.push(sharerInfo.name);
                payerAmounts.push(sharerInfo.paidAmount);
              }

              sharerNames.push(sharerInfo.name);
              sharerAmounts.push(sharerInfo.spentAmount);
            }));
            state.payerNames = payerNames;
            state.payerAmounts = payerAmounts;
            state.sharerNames = sharerNames;
            state.sharerAmounts = sharerAmounts;
            state.hasMultiplePayers = payerNames.length > 1;
            state.splitType = 'uneven';
          }
        };

        state.areSecondaryInputsExpanded = areSecondaryInputsExpanded;
        return state;
      },

      didModalPresent: function didModalPresent() {
        this.focus(this.refs.descriptionInput);
      },

      getBrowserPageTitle: function getBrowserPageTitle() {
        return this.props.transaction ? UIStaticText('EDIT TRANSACTION') : UIStaticText('ADD TRANSACTION');
      },

      renderPageTitle: function renderPageTitle() {
        var label = transactionTypeLabels[this.state.transactionType];
        ASSERT_TRUTHY(label, '[UITransactionInput] Invalid transaction type ['+this.state.transactionType+']');
        return (
          React.createElement(UIPicker, {isCentered: true, onClick: this._didClickTransactionTypePicker}, 
            label
          )
        );
      },

      _didClickTransactionTypePicker: function _didClickTransactionTypePicker() {
        var children = [];
        ArrayUtils.each(transactionTypeLabels, FunctionUtils.bind(this, function(transactionType, label) {
          var didClickTransactionType = FunctionUtils.bind(this, function() {
            AppNavigationController.dismissPopup();
            this.setState({
              areSecondaryInputsExpanded: transactionType === 'transfer',
              transactionType: transactionType
            });
          })
          var classNames = [];
          if (transactionType === this.state.transactionType) {
            classNames.push('selected');
          }
          children.push(
            React.createElement(UISectionRow, {className: classNames.join(' '), onClick: didClickTransactionType, key: 'type-'+transactionType}, 
              label
            )
            );
        }));
        AppNavigationController.presentPopup(
          React.createElement(UIPopup, {title: UIStaticText('SELECT TYPE')}, 
            children
          )
        );
      },

      renderCustomActions: function renderCustomActions() {
        if (this.state.isLoading) {
          return  React.createElement("div", {className: "submitButton smallerFont"}, UIStaticText('SAVING ...'));
        }

        var submitForm = FunctionUtils.bind(this, function() {
          this.refs.inputForm.submit();
        });
        return (
          React.createElement("div", {className: "submitButton smallFont", onClick: submitForm}, 
            UIStaticText('SAVE')
          )
        );
      },

      renderPageContent: function renderPageContent() {
        var parentTidInput = null;
        if (this.props.transaction) {
          if (this.props.transaction.id) {
            parentTidInput = React.createElement("input", {type: "hidden", name: "parentTid", value: this.props.transaction.id});
          } else if (this.props.transaction.attributes.offlineId) {
            parentTidInput = React.createElement("input", {type: "hidden", name: "offlineId", value: this.props.transaction.attributes.offlineId});
          }
        }

        var transaction = new TransactionModel();
        return (
          React.createElement("div", {className: "UITransactionInput"}, 
            React.createElement(UIForm, {ref: "inputForm", model: transaction, delegate: this}, 
              this._renderPrimaryInputs(), 
              this._renderTransactionTypeSpecificInputs(), 
              this._renderSecondaryInputs(), 
              React.createElement("input", {type: "hidden", name: "transactionType", value: this.state.transactionType}), 
              parentTidInput, 
              this._renderSubmitButton()
            )
          )
        );
      },

      _shouldSubmitForm: function _shouldSubmitForm() {
        var refsToCheck = ['descriptionInput', 'amountInput'];

        var shouldSubmit = true;
        ArrayUtils.each(refsToCheck, FunctionUtils.bind(this, function(index, refName) {
          var input = DOM.findById(this.refs[refName].getDOMNode());

          if (input.val().trim() == '') {
            input.focus();
            shouldSubmit = false;
            return false;
          }
        }));

        return shouldSubmit;
      },

      formWillSubmit: function formWillSubmit(form, params) {
        this.setState({
          isLoading: true
        });
      },

      formDidSubmit: function formDidSubmit(form, model) {
        this.setState({
          isLoading: false
        });
        if (model.attributes.offlineId) {
          OfflineTransactionsManager.deleteTransaction(model.attributes);
        }
        if (model.attributes.parentTid) {
          AppNavigationController.navigateBack();
        } else {
          AppNavigationController.reloadPage();
        }
      },

      formDidFailSubmit: function formDidFailSubmit(form, model, response) {
        this.setState({
          isLoading: false
        });

        if (response || !OfflineTransactionsManager.isAvailable()) {
          return;
        }

        var rawModel = model;
        model = model.attributes;

        model.amount = Math.abs(parseFloat(model.amount));
        model.tagNames = StringUtils.explodeCSV(model.tagNames);
        model.account = {
          id: model.accountId,
          name: model.accountName,
        };
        var fromAccountId = parseInt(model.fromAccountId);
        var toAccountId = parseInt(model.toAccountId);
        if (fromAccountId) {
          model.fromAccount = {
            id: fromAccountId,
            name: model.fromAccountName
          };
        }
        if (toAccountId) {
          model.toAccount = {
            id: toAccountId,
            name: model.toAccountName
          };
        }
        delete model.fromAccountId;
        delete model.fromAccountName;
        delete model.toAccountId;
        delete model.toAccountName;
        if (model.transactionType === 'income' || model.transactionType === 'refund') {
          model.amountType = 'income';
        } else if (model.transactionType === 'transfer') {
          if (model.fromAccount && model.toAccount) {
            model.amountType = 'transfer';
          } else {
            model.amountType = 'transfer_one_way';
            if (model.fromAccount) {
              model.amount = - model.amount;
            }
          }
        } else {
          model.amountType = 'expense';
        }
        model.canEdit = true;
        model.canDelete = true;
        delete model.accountId;
        delete model.accountName;

        OfflineTransactionsManager.saveTransaction(model);
        AppNavigationController.reloadPage();

        return EventDelegater.STOP;
      },

      getPageClasses: function getPageClasses() {
        var headerClassMapping = {
          income: 'greenHeader',
          refund: 'greenHeader',
        }
        var headerClass = headerClassMapping[this.state.transactionType] || 'orangeHeader';
        return ['UITransactionInput', headerClass];
      },

      _renderPrimaryInputs: function _renderPrimaryInputs() {
        var descriptionValue = this.props.transaction ? this.props.transaction.attributes.description : '';

        var inputsContainer = this._renderLabelAmountInputs(
          React.createElement("input", {
            type: "text", 
            autoCorrect: "off", 
            ref: "descriptionInput", 
            name: "description", 
            placeholder: UIStaticText('DESCRIPTION'), 
            defaultValue: descriptionValue}
          ),
          React.createElement(UIAmountInput, {
            ref: "amountInput", 
            name: "amount", 
            placeholder: UIStaticText('AMOUNT'), 
            defaultValue: this.state.amount}
          )
        );
        return (
          React.createElement("div", {className: "primaryInputsContainer"}, 
            inputsContainer
          )
        );
      },

      _renderSecondaryInputs: function _renderSecondaryInputs() {
        if (this.state.areSecondaryInputsExpanded) {
          return (
            React.createElement("div", {className: "secondaryInputsContainer expanded"}, 
              React.createElement(UIAutocompleteInput, {
                className: "inputSelector tagSelector", 
                ref: "tagInput", 
                name: "tagNames", 
                placeholder: UIStaticText('TAGS'), 
                defaultValue: this.state.tagNames, 
                autocompleteType: "tag", 
                allowMultipleValues: true}
              ), 
              this._renderAccountInput(), 
              React.createElement(UIDatePicker, {className: "inputSelector", date: this.state.date, onSelect: this._didSelectDate})
            )
          );
        }

        return (
          React.createElement("div", {className: "secondaryInputsContainer collapsed"}, 
            React.createElement("input", {type: "hidden", name: "accountId", value: this.state.account.id}), 
            React.createElement("input", {type: "hidden", name: "accountName", value: this.state.account.accountName}), 
            React.createElement("input", {type: "hidden", name: "tagNames", value: this.state.tagNames}), 
            React.createElement("input", {type: "hidden", name: "date", value: this.state.date}), 
            React.createElement(UIHorizontalList, {className: "fullWidth"}, 
              React.createElement("div", {className: "inputSelector accountSelector", onClick: this._didClickAccountSelector}), 
              React.createElement("div", {className: "inputSelector tagSelector", onClick: this._didClickTagSelector}), 
              React.createElement(UIDatePicker, {className: "inputSelector", showSelectedDate: false, date: this.state.date, onSelect: this._didSelectDate})
            )
          )
        );
      },

      _didClickTagSelector: function _didClickTagSelector() {
        this.setState({
          areSecondaryInputsExpanded: true
        });
      },

      _didSelectDate: function _didSelectDate(date) {
        this.setState({
          areSecondaryInputsExpanded: true,
          date: date
        });
      },

      _renderAccountInput: function _renderAccountInput() {
        if (this.state.transactionType === 'transfer') {
          return (
            React.createElement("div", {className: "inputSelector accountSelector"}, 
              React.createElement(UIHorizontalList, {isFullWidth: true, widths: ['50%', 1, '50%']}, 
                this._renderTransferAccountPicker('fromAccount', UIStaticText('FROM')), 
                React.createElement("div", {className: "transferArrowContainer"}), 
                this._renderTransferAccountPicker('toAccount', UIStaticText('TO'))
              )
            )
          );
        }

        var accountName = this.state.account.name;

        return (
          React.createElement("div", {className: "inputSelector accountSelector clickable", onClick: this._didClickAccountSelector}, 
            accountName, 
            React.createElement("input", {type: "hidden", name: "accountId", value: this.state.account.id}), 
            React.createElement("input", {type: "hidden", name: "accountName", value: accountName})
          )
        );
      },

      _didClickAccountSelector: function _didClickAccountSelector() {
        var onAccountSelect = FunctionUtils.bind(this, function onAccountSelect(account) {
          this.setState({
            areSecondaryInputsExpanded: true,
            account: account.attributes
          });
        });

        AppNavigationController.presentModal(
          React.createElement(UIAccountPickerListPage, {
            accountId: this.state.account.id, 
            enableNoAccountOption: true, 
            onSelect: onAccountSelect}
          )
        );
      },

      _renderTransferAccountPicker: function _renderTransferAccountPicker(accountType, placeholder) {
        var account = this.state[accountType];
        var accountUI, accountIdInput, accountNameInput;
        if (account) {
          accountUI = account.name;
          accountIdInput = React.createElement("input", {type: "hidden", name: accountType+'Id', value: account.id});
          accountNameInput = React.createElement("input", {type: "hidden", name: accountType+'Name', value: account.name});
        } else {
          accountUI = (
            React.createElement("div", {className: "accountPickerLabel"}, 
              placeholder
            )
          );
        }
        var didClickPicker = FunctionUtils.bind(this, this._didClickTransferAccountPicker, accountType);
        return (
          React.createElement("div", {onClick: didClickPicker, className: "clickable transferAccountPicker"}, 
            accountUI, 
            accountIdInput, 
            accountNameInput
          )
        );
      },

      _renderTransactionTypeSpecificInputs: function _renderTransactionTypeSpecificInputs() {
        switch (this.state.transactionType) {
          case 'sharedBill':
            return this._renderSharedBillInputs();
          case 'paidForFriend':
          case 'settlement':
          case 'loan':
            return this._renderTwoPersonSharedBillInputs();
        }
      },

      _didClickTransferAccountPicker: function _didClickTransferAccountPicker(accountType) {
        var account = this.state[accountType] || {};
        var didSelectAccount = FunctionUtils.bind(this, function(account) {
          var newState = {};
          newState[accountType] = account.attributes;
          this.setState(newState);
        });
        AppNavigationController.presentModal(
          React.createElement(UIAccountPickerListPage, {accountId: account.id, enableNoAccountOption: true, onSelect: didSelectAccount})
        );
      },

      _renderTwoPersonSharedBillInputs: function _renderTwoPersonSharedBillInputs() {
        var firstPersonTitleMapping = {
          paidForFriend: UIStaticText('PAID BY'),
          settlement: UIStaticText('PAID BY'),
          loan: UIStaticText('LOANED BY')
        };
        var firstPersonInputNameMapping = {
          paidForFriend: 'paidForPaidBy',
          settlement: 'settlePaidBy',
          loan: 'loanedBy'
        };
        var secondPersonTitleMapping = {
          paidForFriend: UIStaticText('PAID FOR'),
          settlement: UIStaticText('RECEIVED BY'),
          loan: UIStaticText('BORROWED BY')
        };
        var secondPersonInputNameMapping = {
          paidForFriend: 'paidForPaidFor',
          settlement: 'settlePaidFor',
          loan: 'borrowedBy'
        };

        var firstPersonTitle = ASSERT_TRUTHY(firstPersonTitleMapping[this.state.transactionType], '[UITransactionInput] Invalid transaction type ['+this.state.transactionType+']');
        var secondPersonTitle = ASSERT_TRUTHY(secondPersonTitleMapping[this.state.transactionType], '[UITransactionInput] Invalid transaction type ['+this.state.transactionType+']');
        var firstPersonInputName = ASSERT_TRUTHY(firstPersonInputNameMapping[this.state.transactionType], '[UITransactionInput] Invalid transaction type ['+this.state.transactionType+']');
        var secondPersonInputName = ASSERT_TRUTHY(secondPersonInputNameMapping[this.state.transactionType], '[UITransactionInput] Invalid transaction type ['+this.state.transactionType+']');

        var firstPersonDefaultValue = '', secondPersonDefaultValue = '';
        if (this.props.transaction) {
          if (this.state.payerNames.length > 0) {
            firstPersonDefaultValue = this.state.payerNames[0];
          }
          if (this.state.sharerNames.length > 0) {
            ArrayUtils.each(this.state.sharerNames, function(index, sharerName) {
              if (sharerName !== firstPersonDefaultValue) {
                secondPersonDefaultValue = sharerName;
                return false;
              }
            });
          }
        }

        return (
          React.createElement("div", null, 
            React.createElement("div", {className: "sectionHeader"}, 
              firstPersonTitle
            ), 
            React.createElement(UIAutocompleteInput, {
              name: firstPersonInputName, 
              defaultValue: firstPersonDefaultValue, 
              autocompleteType: "contact"}
            ), 
            React.createElement("div", {className: "sectionHeader"}, 
              secondPersonTitle
            ), 
            React.createElement(UIAutocompleteInput, {
              name: secondPersonInputName, 
              defaultValue: secondPersonDefaultValue, 
              autocompleteType: "contact"}
            )
          )
        );
      },

      _renderSharedBillInputs: function _renderSharedBillInputs() {
        return (
          React.createElement("div", {className: "sharedInputsContainer"}, 
            this._renderPayerInputs(), 
            this._renderSharerInputs(), 
            React.createElement("input", {type: "hidden", name: "numPayers", value: this.state.payerNames.length}), 
            React.createElement("input", {type: "hidden", name: "numSharers", value: this.state.sharerNames.length})
          )
        );
      },

      _renderPayerInputs: function _renderPayerInputs() {
        var didAddPayer = FunctionUtils.bind(this, function() {
          this.setState({
            hasMultiplePayers: true
          });
        });

        var payerInputs = this._renderParticipantInputList({
          inputNamePrefix: 'payer',
          names: this.state.payerNames,
          amounts: this.state.payerAmounts,
          addInputPlaceholder: UIStaticText('Add payer'),
          onChangeInput: FunctionUtils.bind(this, this._didChangeParticipantInput, 'payerNames'),
          shouldDisplayAddInput: this.state.hasMultiplePayers,
          shouldDisplayAmount: this.state.hasMultiplePayers
        });

        var addHeaderButton = null;
        if (!this.state.hasMultiplePayers) {
          addHeaderButton = React.createElement("div", {className: "sectionHeaderButton", onClick: didAddPayer});
        }

        return (
          React.createElement("div", null, 
            React.createElement("div", {className: "sectionHeader payerSectionHeader"}, 
              UIStaticText('PAYERS'), 
              addHeaderButton
            ), 
            payerInputs
          )
        );
      },

      _renderSharerInputs: function _renderSharerInputs() {
        var sharerInputs = this._renderParticipantInputList({
          inputNamePrefix: 'sharer',
          names: this.state.sharerNames,
          amounts: this.state.sharerAmounts,
          addInputPlaceholder: UIStaticText('Add sharer'),
          onChangeInput: FunctionUtils.bind(this, this._didChangeParticipantInput, 'sharerNames'),
          shouldDisplayAddInput: true,
          shouldDisplayAmount: this.state.splitType === 'uneven'
        });

        return (
          React.createElement("div", null, 
            React.createElement("input", {type: "hidden", name: "splitType", value: this.state.splitType}), 
            React.createElement("div", {className: "sectionHeader sharerSectionHeader"}, 
              React.createElement("div", {className: "sectionHeaderButton"}, 
                this._renderSplitTypeSelector()
              ), 
              UIStaticText('SHARERS')
            ), 
            sharerInputs
          )
        );
      },

      _renderSplitTypeSelector: function _renderSplitTypeSelector() {
        var splitTypes = {
          'even': UIStaticText('EVEN'),
          'uneven': UIStaticText('UNEVEN'),
        };
        var cells = [];
        ArrayUtils.each(splitTypes, FunctionUtils.bind(this, function(splitType, label) {
          var classNames = ['splitTypeOption'];
          if (splitType === this.state.splitType) {
            classNames.push('selected');
          }

          var didClickSplitTypeOption = FunctionUtils.bind(this, function() {
            this.setState({
              splitType: splitType
            });
          });
          cells.push(
            React.createElement("div", {className: classNames.join(' '), onClick: didClickSplitTypeOption}, 
              label
            )
          );
        }));

        return (
          React.createElement("div", {className: "splitTypeSelector"}, 
            cells
          )
        );
      },

      _didChangeParticipantInput: function _didChangeParticipantInput(key, inputIndex, inputValue) {
        var names = this.state[key];
        names[inputIndex] = inputValue;

        this.setState({
          key: names
        });
      },

      _renderLabelAmountInputs: function _renderLabelAmountInputs(labelInput, amountInput) {
        return (
          React.createElement(UILeftRight, {className: "labelAmountInputContainer fullWidth"}, 
            labelInput, 
            amountInput
          )
          );
      },

      _renderParticipantInputList: function _renderParticipantInputList(options) {
        var requiredInputs = ['names', 'inputNamePrefix', 'addInputPlaceholder', 'onChangeInput'];
        ArrayUtils.each(requiredInputs, function(index, inputName) {
          ASSERT_TRUTHY(options[inputName], '[UITransactionInput] Expected non-empty input ['+inputName+']');
        });

        var amounts = options.amounts || [];

        var numInputs = options.names.length;
        if (options.shouldDisplayAddInput) {
          numInputs++;
        }

        var participantInputs = [];
        for (var participantIndex = 0; participantIndex < numInputs; ++participantIndex) {
          var participantInputName = options.inputNamePrefix+'Name'+participantIndex;
          var participantInputValue = options.names[participantIndex] || '';

          var onChange = FunctionUtils.bind(this, options.onChangeInput, participantIndex);

          var placeholder = '';
          if (options.shouldDisplayAddInput && participantIndex == numInputs - 1) {
            placeholder = options.addInputPlaceholder;
          }

          var participantInput = (
            React.createElement(UIAutocompleteInput, {
              name: participantInputName, 
              defaultValue: participantInputValue, 
              "data-index": participantIndex, 
              onChange: onChange, 
              autocompleteType: "contact", 
              placeholder: placeholder}
            )
          );

          if (!options.shouldDisplayAmount) {
            participantInputs.push(participantInput);
            continue;
          }

          var amountInputName = options.inputNamePrefix+'Amount'+participantIndex;
          var amountInputValue = amounts[participantIndex] || '';
          participantInputs.push(
            this._renderLabelAmountInputs(
              participantInput,
              React.createElement(UIAmountInput, {
                name: amountInputName, 
                placeholder: UIStaticText('AMOUNT'), 
                defaultValue: amountInputValue}
              )
            )
          );
        }

        return participantInputs;
      },

      _renderSubmitButton: function _renderSubmitButton() {
        return React.createElement(UIButton, {className: "primarySubmitButton", isFullWidth: true, type: "submit", title: UIStaticText('SAVE TRANSACTION')});
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/list/UIPage',[
    'touch/app/UIPage',
    'touch/transactions/list/UIList',
    ''
  ],
  function UITransactionListPage(
    UIAppPage,
    UITransactionList,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('TRANSACTIONS');
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement(UITransactionList, {filters: this.getFilters()})
        );
      }

    });
  }
);

/** @jsx React.DOM */


define(
  'touch/transactions/UIPage',[
    'touch/app/UIPage',
    'touch/ui/UIGrid',
    'touch/ui/UIHorizontalList',
    'touch/ui/UIAmount',
    'touch/ui/UIText',
    'touch/ui/UIDate',
    'touch/ui/UIButton',
    'touch/transactions/UIAccount',
    'touch/transactions/UITags',
    'touch/transactions/input/UIInput',
    'touch/app/NavigationController',
    'touch/transactions/Model',
    'touch/transactions/offline/Manager',
    'touch/app/UISection',
    ''
  ],
  function UITransactionPage(
    UIAppPage,
    UIGrid,
    UIHorizontalList,
    UIAmount,
    UIText,
    UIDate,
    UIButton,
    UITransactionAccount,
    UITransactionTags,
    UITransactionInput,
    AppNavigationController,
    TransactionModel,
    OfflineTransactionsManager,
    UISection,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getModelsToFetch: function getModelsToFetch() {
        if (this.props.transaction && this.props.transaction.isPersonalTransaction()) {
          return {};
        }
        var transactionId = this.getRequestParam('id');
        return {
          transaction: new TransactionModel({
            id: transactionId
          })
        };
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('TRANSACTIONS');
      },

      getClassNames: function getClassNames() {
        var classes = ['UITransactionPage'];
        if (!this.state.models.transaction && !this.props.transaction) {
          classes.push('loading');
        }
        return classes;
      },

      renderPageContent: function renderPageContent() {
        var transaction = this.state.models.transaction || this.props.transaction;
        var descUI = ' ', amountUI = ' ', dateUI = ' ', tagsUI = ' ', accountUI = ' ';
        if (transaction) {
          descUI = transaction.attributes.description;
          amountUI = React.createElement(UIAmount, {amountType: transaction.attributes.amountType, amount: transaction.attributes.amount});
          dateUI = React.createElement(UIDate, {date: transaction.attributes.date});
          tagsUI = React.createElement(UITransactionTags, {transaction: transaction, maxLength: -1});
          accountUI = React.createElement(UITransactionAccount, {transaction: transaction, maxLength: -1});
        }

        return (
          React.createElement("div", null, 
            React.createElement(UISection, null, 
              React.createElement(UIGrid, {numColumns: "2", className: "detailsContainer fullWidth", widths: [1]}, 
                UIStaticText('DESC'), 
                descUI, 

                UIStaticText('AMOUNT'), 
                amountUI, 

                UIStaticText('DATE'), 
                dateUI, 

                UIStaticText('TAGS'), 
                tagsUI, 

                UIStaticText('ACCOUNT'), 
                accountUI
              )
            ), 

            this._maybeRenderSharers()
          )
        );
      },

      _maybeRenderSharers: function _maybeRenderSharers() {
        var transaction = this.state.models.transaction || this.props.transaction;
        if (!transaction || transaction.isPersonalTransaction() || !this.state.models.transaction) {
          return;
        }

        var cells = [
          React.createElement("div", {className: "columnLabel"}, UIStaticText('SHARER')),
          React.createElement("div", {className: "columnLabel"}, UIStaticText('PAID')),
          React.createElement("div", {className: "columnLabel"}, UIStaticText('SPENT'))
        ];
        ArrayUtils.each(transaction.attributes.sharers, function(index, sharerInfo) {
          cells.push(sharerInfo.name);
          cells.push(
            React.createElement(UIAmount, {amountType: "none", amount: sharerInfo.paidAmount})
          );
          cells.push(
            React.createElement(UIAmount, {amountType: "none", amount: sharerInfo.spentAmount})
          );
        });

        return (
          React.createElement(UISection, null, 
            React.createElement(UIGrid, {className: "fullWidth sharersContainer", numColumns: "3"}, 
              cells
            )
          )
        );

      },

      renderFixedActions: function renderFixedActions() {
        var transaction = this.state.models.transaction || this.props.transaction;
        if (!transaction || (!transaction.isPersonalTransaction() && !this.state.models.transaction)) {
          return;
        }

        var actionButtons = [];
        if (transaction.attributes.canEdit) {
          actionButtons.push(
            React.createElement(UIButton, {title: UIStaticText('EDIT'), isFullWidth: true, onClick: this._didClickEditButton, key: "button-edit"})
          );
        }
        if (transaction.attributes.canDelete) {
          actionButtons.push(
            React.createElement(UIButton, {
              isDestructive: true, 
              title: UIStaticText('DELETE'), 
              isFullWidth: true, 
              onClick: this._didClickDeleteButton, 
              key: "button-delete"}
            )
          );
        }

        if (!actionButtons.length) {
          return;
        }

        return (
          React.createElement(UIHorizontalList, {className: "actionsContainer fullWidth", hasEqualWidthCells: true}, 
            actionButtons
          )
        );
      },

      _didClickEditButton: function _didClickEditButton() {
        var transaction = this.state.models.transaction || this.props.transaction;
        AppNavigationController.presentModal(
          React.createElement(UITransactionInput, {transaction: transaction})
        );
      },

      _didClickDeleteButton: function _didClickDeleteButton() {
        var onDelete = function() {
          AppNavigationController.navigateBack();
        };
        var transaction = this.state.models.transaction || this.props.transaction;
        if (transaction.attributes.offlineId) {
          OfflineTransactionsManager.deleteTransaction(transaction.attributes);
          onDelete();
        } else {
          transaction.destroy({
            success: onDelete
          });
        }
      }

    });


  }
);


/** @jsx React.DOM */


define(
  'touch/membership/UIPlans',[
    'touch/app/UIPage',
    'touch/app/UISection',
    'touch/app/UISectionRow',
    'touch/ui/UIRadioSelector',
    ''
  ],
  function UIMembershipPlans(
    UIAppPage,
    UISection,
    UISectionRow,
    UIRadioSelector,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getInitialState: function getInitialState() {
        return {
          plan: this.props.plan || 'pro'
        };
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('MEMBERSHIP PLANS');
      },

      renderPageContent: function renderPageContent() {
        var allPlans = ['pro', 'plus', 'basic'];
        var options = {};
        ArrayUtils.each(allPlans, FunctionUtils.bind(this, function(index, plan) {
          options[plan] = this._renderOptionForPlan(plan);
        }));
        
        return (
          React.createElement("div", {className: "UIMembershipPlans"}, 
            React.createElement("div", {className: "UISection padded"}, 
              React.createElement(UIRadioSelector, {isFullWidth: true, value: this.state.plan, options: options, onSelect: this._didSelectPlan})
            ), 
            this._renderFeatures()
          )
        );
      },

      _didSelectPlan: function _didSelectPlan(plan) {
        this.setState({
          plan: plan
        });
      },

      _renderFeatures: function _renderFeatures() {
        var featuresByPlan = {
          basic: [
            UIStaticText('Unlimited Transactions'),
            UIStaticText('Shared Bills and IOUs'),
            UIStaticText('Basic Reports')
          ],
          plus: [
            UIStaticText('Unlimited Accounts'),
            UIStaticText('Unlimited Budgets'),
            UIStaticText('Unlimited Bill Reminders')
          ],
          pro: [
            UIStaticText('Advanced Reports'),
            UIStaticText('Forecast'),
            UIStaticText('Online Payments'),
            UIStaticText('Automatic Backups')
          ]
        };
        var proFeatureDescriptions = [
          UIStaticText("Advanced visual tools like Top Movers, Timeline and Allocation help surface deviations from normal spending (or earning) patterns."),
          UIStaticText("Predicts account balances based on past spending. Useful to prevent overdraft fees and going over credit card limits."),
          UIStaticText('Send money to any email address via Paypal. Useful to send gifts and settle debts.'),
          UIStaticText('Transactions are automatically uploaded to your Dropbox account everyday. Your data, safe with you!'),
        ];

        var allPlans = ['pro', 'plus', 'basic'];
        var sectionRows = [];
        var isFeatureAvailable = false;
        ArrayUtils.each(allPlans, FunctionUtils.bind(this, function(planIndex, plan) {
          var features = ASSERT_TRUTHY(featuresByPlan[plan], 'Invalid plan ['+plan+']');

          if (!isFeatureAvailable && plan == this.state.plan) {
            isFeatureAvailable = true;
          };

          var classNames = [];
          if (!isFeatureAvailable) {
            classNames.push('notAvailable');
          }

          ArrayUtils.each(features, function(featureIndex, feature) {
            sectionRows.push(
              React.createElement(UISectionRow, {className: classNames.join(' '), key: 'plan-'+planIndex+'-'+featureIndex}, 
                feature
              )
            );
          });
        }));

        return (
          React.createElement(UISection, {label: UIStaticText('FEATURES')}, 
            sectionRows
          )
        );

      },

      _renderOptionForPlan: function _renderOptionForPlan(plan) {
        var planLabels = {
          pro: UIStaticText('PRO'),
          plus: UIStaticText('PLUS'),
          basic: UIStaticText('BASIC')
        };
        var planPrices = {
          pro: 4.99,
          plus: 3.99,
          basic: 0
        };

        var label = ASSERT_TRUTHY(planLabels[plan], 'Invalid plan ['+plan+']');
        var price = planPrices[plan];

        var priceUI, monthUI;
        if (price == 0) {
          priceUI = UIStaticText('FREE');
          monthUI = React.createElement("span", null, " ");
        } else {
          priceUI = '$ '+price;
          monthUI = UIStaticText('PER MONTH');
        }

        return (
          React.createElement("div", null, 
            label, 
            React.createElement("div", {className: "priceContainer bigFont"}, 
              priceUI
            ), 
            React.createElement("div", {className: "smallerFont"}, 
              monthUI
            )
          )
        );
      }

    });


  }
);

/** @jsx React.DOM */


define(
  'touch/membership/UIPayment',[
    'touch/app/Runtime',
    'touch/app/UIPage',
    'touch/ui/UIRadioSelector',
    'touch/ui/UIButton',
    'touch/app/NavigationController',
    'touch/core/Model',
    'touch/auth/Session',
    ''
  ],
  function UIMembershipPayment(
    Runtime,
    UIAppPage,
    UIRadioSelector,
    UIButton,
    AppNavigationController,
    Model,
    Session,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      getInitialState: function getInitialState() {
        return {
          plan: this.props.plan || 'pro',
          period: this.props.period || '12'
        };
      },

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('UPGRADE MEMBERSHIP');
      },
      
      getPageClasses: function getPageClasses() {
        return ['greenHeader'];
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement("div", {className: "UIMembershipPayment UISection padded"}, 
            this._renderPlanSelector(), 
            this._renderPeriodSelector(), 
            this._renderPaymentMethodButtons(), 
            React.createElement("div", {className: "renewalInfoContainer smallFont"}, 
              "Your membership will renew automatically. By purchasing a membership, you agree to the Buxfer ", ' ', 
              React.createElement("a", {className: "hoverable", target: "_blank", href: "/pricingTerms"}, "Pricing Terms"), "."
            )
          )
        );
      },

      _renderPlanSelector: function _renderPlanSelector() {
        var planOptions = {
          plus: UIStaticText('PLUS'),
          pro: UIStaticText('PRO')
        };
        return (
          React.createElement("div", {className: "planSelectorContainer"}, 
            React.createElement("div", {className: "bFontWeight"}, 
              UIStaticText('SELECT PLAN')
            ), 
            React.createElement(UIRadioSelector, {className: "ttMargin", isFullWidth: true, name: "planType", options: planOptions, value: this.state.plan, onSelect: this._didSelectPlan})
          )
        );
      },

      _renderPeriodSelector: function _renderPeriodSelector() {
        var rateSchedule = {
          plus: {
            1: UIStaticText('$4.99 / month'),
            12: UIStaticText('$3.99 / month')
          },
          pro: {
            1: UIStaticText('$5.99 / month'),
            12: UIStaticText('$4.99 / month')
          }
        };
        var periodDescriptions = {
          1: UIStaticText('Monthly'),
          12: UIStaticText('Yearly')
        };

        var options = {};
        ArrayUtils.each(rateSchedule[this.state.plan], function(planPeriod, pricingDescription) {
          var periodDescription = periodDescriptions[planPeriod];
          var labelUI = (
            React.createElement("div", null, 
              periodDescription, 
              React.createElement("div", {className: "tMargin"}, 
                pricingDescription
              )
            )
          );
          options[planPeriod] = labelUI;
        });

        return (
          React.createElement("div", {className: "periodSelectorContainer"}, 
            React.createElement("div", {className: "bFontWeight"}, 
              UIStaticText('SELECT PERIOD')
            ), 
            React.createElement(UIRadioSelector, {className: "ttMargin", isFullWidth: true, name: "periodType", options: options, value: this.state.period, onSelect: this._didSelectPeriod})
          )
        );
      },

      _didSelectPlan: function _didSelectPlan(plan) {
        this.setState({
          plan: plan
        });
      },

      _didSelectPeriod: function _didSelectPeriod(period) {
        this.setState({
          period: period
        });
      },

      _renderPaymentMethodButtons: function _renderPaymentMethodButtons() {
        return (
          React.createElement("div", {className: "paymentMethodButtonsContainer"}, 
            React.createElement(UIButton, {
              title: UIStaticText('PAY WITH CREDIT CARD'), 
              isFullWidth: true, 
              onClick: this._didClickCreditCardButton, 
              ref: "creditCardSubmitButton"}
            ), 
            React.createElement(UIButton, {
              title: UIStaticText('PAY WITH PAYPAL'), 
              isFullWidth: true, 
              className: "ttMargin", 
              onClick: this._didClickPaypalButton, 
              ref: "paypalSubmitButton"}
            )
          )
        );
      },

      _didClickCreditCardButton: function _didClickCreditCardButton() {
        this.setLoading(this.refs.creditCardSubmitButton, true);

        var description;
        if (this.state.plan === 'plus') {
          description = UIStaticText('PLUS Membership');
        } else if (this.state.plan === 'pro') {
          description = UIStaticText('PRO Membership');
        } else {
          BUG('Invalid plan ['+this.state.plan+']');
        }

        var totalAmount = this._getTotalAmount() * 100;

        var stripeKey = Runtime.isDev ? 'pk_kU85fBDSBgA5UwZyJtWFkahJg6K2k' : 'pk_6ehWePRkAWTLz3PZTTAk8TMNm3pvZ';
        StripeCheckout.open({
          key: stripeKey,
          address: false,
          amount: totalAmount,
          currency: 'usd',
          email: Session.getEmail(),
          name: 'Buxfer',
          description: description,
          panelLabel: UIStaticText('PAY'),
          image: '/media/freebird/logo/logo-dark.png',
          token: FunctionUtils.bind(this, this._submitCreditCardPayment),
          opened: FunctionUtils.bind(this, function _didOpenStripeCheckout() {
            this.setLoading(this.refs.creditCardSubmitButton, false);
          })
        });
      },

      _submitCreditCardPayment: function _submitCreditCardPayment(response) {
        var url = Url.createWithPath('/apix/membership/chargeCreditCard');
        url.addQueryParams({
          stripeToken: response.id,
          plan: this.state.plan,
          period: this.state.period
        });
        var ref = this.getRequestParam('ref');
        if (ref) {
          url.addQueryParam('referer', ref);
        }

        var model = Model.create({
          url: url.toString()
        });
        model.fetch({
          success: FunctionUtils.bind(this, function(model) {
            this.setLoading(this.refs.creditCardSubmitButton, false);
            AppNavigationController.dismissModal();
            AppNavigationController.reloadPage();
          })
        });
      },

      _didClickPaypalButton: function _didClickPaypalButton() {
        this.setLoading(this.refs.paypalSubmitButton, true);

        var url = Url.createWithPath('/apix/membership/paypalRedirect');
        url.addQueryParams({
          plan: this.state.plan,
          period: this.state.period,
          returnUrl: AppNavigationController.getCurrentUrl()
        });
        var ref = this.getRequestParam('ref');
        if (ref) {
          url.addQueryParam('referer', ref);
        }

        var model = Model.create({
          url: url.toString()
        });
        model.fetch({
          success: FunctionUtils.bind(this, function(model) {
            // For apps, user can switch back to our app, and we need to reset
            // the loading state. For website, opening external URL can take a
            // while, so keep the loading indicator going.
            if (Runtime.isApp) {
              this.setLoading(this.refs.paypalSubmitButton, false);
            }
            AppNavigationController.openExternalUrl(model.attributes.redirectData.url);
          }),
          error: FunctionUtils.bind(this, function(model, request, options) {
            this.setLoading(this.refs.paypalSubmitButton, false);
            ErrorManager.handleAPIError(request.responseJSON);
          })
        });
      },

      _getTotalAmount: function _getTotalAmount() {
        var rateSchedule = {
          plus: {
            1: 4.99,
            12: 3.99
          },
          pro: {
            1: 5.99,
            12: 4.99
          }
        };
        return rateSchedule[this.state.plan][this.state.period]*this.state.period;
      }
    });


  }
);

/** @jsx React.DOM */


define(
  'touch/search/UIGlobalSearch',[
    'touch/app/NavigationController',
    'touch/ui/UIPopup',
    'touch/ui/UIHorizontalList',
    'touch/app/UIPage',
    'touch/search/ResultsModel',
    'touch/app/UISectionRow',
    'touch/ui/UIText',
    'touch/transactions/UITransaction',
    'touch/transactions/Model',
    ''
  ],
  function UIGlobalSearch(
    AppNavigationController,
    UIPopup,
    UIHorizontalList,
    UIAppPage,
    SearchResultsModel,
    UISectionRow,
    UIText,
    UITransaction,
    TransactionModel,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('SEARCH');
      },

      renderPageContent: function renderPageContent() {
        return (
          React.createElement("div", {className: "UIGlobalSearch"}, 
            React.createElement("input", {ref: "searchInput", className: "searchInput", name: "keywords", type: "text", onChange: this._didChangeQuery}), 
            this._renderResults()
          )
        );
      },

      _renderResults: function _renderResults() {
        if (!this.state.resultsModel) {
          return (
            React.createElement("div", {className: "hintContainer styleItalic"}, 
              UIStaticText('Search transactions, tags, accounts, contacts or groups')
            )
          );
        }

        if (!this.state.resultsModel.models.length) {
          return (
            React.createElement("div", {className: "hintContainer styleItalic"}, 
              UIStaticText('No results found')
            )
          );
        }

        var resultNodes = [];
        var currentCategory = null;
        var hasTransactionResults = false;
        ArrayUtils.each(this.state.resultsModel.models, FunctionUtils.bind(this, function(index, model) {
          if (model.attributes.type !== currentCategory) {
            currentCategory = model.attributes.type;
            if (currentCategory === 'transaction') {
              hasTransactionResults = true;
            }
            var categoryLabel = UIStaticText((currentCategory+'s').toUpperCase());
            resultNodes.push(
              React.createElement("div", {className: "sectionHeader", key: 'section-'+currentCategory}, 
                categoryLabel
              )
            );
          }
          
          var nameProps = {
            key: 'result-'+index,
            className: 'resultContainer',
            onClick: FunctionUtils.bind(this, this._didClickResult, model)
          };

          var key = 'result-'+model.attributes.type+'-'+model.id;
          var nameUI;
          if (model.attributes.type === 'transaction') {
            var transaction = new TransactionModel(model.attributes);
            nameUI = React.createElement(UITransaction, React.__spread({transaction: transaction},  nameProps, {key: key}));
          } else {
            nameUI = (
             React.createElement(UISectionRow, React.__spread({},  nameProps, {key: key}), 
               React.createElement(UIText, {maxLength: "30"}, model.attributes.name)
             )
           );
          }

          resultNodes.push(nameUI);
        }));

        if (hasTransactionResults) {
          var searchUrl = '/transactions?keywords='+this.state.query;
          resultNodes.push(
            React.createElement(UISectionRow, {className: "resultContainer", href: searchUrl, key: "see-all"}, 
              UIStaticText('SEE ALL MATCHING TRANSACTIONS')
            )
          );
        }

        return (
          React.createElement("div", {className: "resultListContainer"}, 
            resultNodes
          )
        );
      },

      _didClickResult: function _didClickResult(result) {
        var urlBasedTypes = {
          tag: 1,
          contact: 1,
          group: 1,
          account: 1
        };
        if (result.attributes.type in urlBasedTypes) {
          AppNavigationController.openUrl('/'+result.attributes.type+'?id='+result.id);
          return;
        }
      },

      _didCancelSearch: function _didCancelSearch() {
        AppNavigationController.dismissModal();
        this.setState({
          resultsModel: null
        });
      },

      didModalPresent: function didModalPresent() {
        this.focus(this.refs.searchInput);
      },

      _didChangeQuery: function _didChangeQuery(event) {
        DOM.setLoading(this.refs.searchInput.getDOMNode(), true);

        var inputField = event.target;
        var query = DOM.value(inputField);
        this.setState({
          query: query
        });

        if (!query.length) {
          DOM.setLoading(this.refs.searchInput.getDOMNode(), false);
          this.setState({
            query: null,
            resultsModel: null
          });
          return;
        }

        var resultsModel = SearchResultsModel.create({
          query: query,
          type: 'global'
        });
        var onFetchComplete = FunctionUtils.bind(this, function(model) {
          if (model.query != this.state.query) {
            return;
          }
          this.setState({
            resultsModel: model
          });
          DOM.setLoading(this.refs.searchInput.getDOMNode(), false);
        });
        resultsModel.fetch({
          success: onFetchComplete
        });
      }
    });

  }
);

/** @jsx React.DOM */


define(
  'touch/feedback/UIFeedback',[
    'touch/app/UIPage',
    'touch/app/UISection',
    'touch/ui/UIForm',
    'touch/ui/UIButton',
    'touch/app/NavigationController',
    ''
  ],
  function UIFeedback(
    UIAppPage,
    UISection,
    UIForm,
    UIButton,
    AppNavigationController,
    junk
  ) {

    return React.createClass({

      mixins: [UIAppPage],

      renderPageTitle: function renderPageTitle() {
        return UIStaticText('CONTACT US');
      },

      renderPageContent: function renderPageContent() {
        var placeholder = UIStaticText(
          'Report a problem, Suggest an idea or Provide feedback'
        );
        return (
          React.createElement(UISection, {className: "UIFeedback", isPadded: true}, 
            React.createElement(UIForm, {action: "/apix/feedback", delegate: this}, 
              React.createElement("textarea", {name: "message", placeholder: placeholder}), 
              React.createElement(UIButton, {type: "submit", isFullWidth: true, className: "ttMargin", title: UIStaticText('SUBMIT FEEDBACK')})
            )
          )
        );
      },

      formDidSubmit: function formDidSubmit(model) {
        alert(UIStaticText('Thank you for your valuable feedback!'));
        AppNavigationController.openUrl('/dashboard');
      }

    });


  }
);



define(
  'touch/app/RouteConfig',[
    'touch/auth/UILogin',
    'touch/auth/UISignup',
    'touch/dashboard/UIDashboard',
    'touch/accounts/list/UIPage',
    'touch/accounts/UIAccount',
    'touch/contacts/list/UIPage',
    'touch/contacts/UIContact',
    'touch/tags/list/UIPage',
    'touch/tags/UITag',
    'touch/budgets/list/UIPage',
    'touch/budgets/UIBudget',
    'touch/groups/list/UIPage',
    'touch/groups/UIGroup',
    'touch/loans/list/UIPage',
    'touch/forecast/UIForecast',
    'touch/reminders/list/UIPage',
    'touch/reports/UIReport',
    'touch/transactions/input/UIInput',
    'touch/transactions/list/UIPage',
    'touch/transactions/UIPage',
    'touch/membership/UIPlans',
    'touch/membership/UIPayment',
    'touch/search/UIGlobalSearch',
    'touch/feedback/UIFeedback',
    ''
  ],
  function LoggedInRouteConfig(
    UILogin,
    UISignup,
    UIDashboard,
    UIAccountListPage,
    UIAccount,
    UIContactListPage,
    UIContact,
    UITagListPage,
    UITag,
    UIBudgetListPage,
    UIBudget,
    UIGroupListPage,
    UIGroup,
    UILoanListPage,
    UIForecast,
    UIReminderListPage,
    UIReports,
    UITransactionInput,
    UITransactionListPage,
    UITransactionPage,
    UIMembershipPlans,
    UIMembershipPayment,
    UIGlobalSearch,
    UIFeedback,
    junk
  ) {

    return {
      routes: {
        '/dashboard': {
          component: UIDashboard,
          navigationCategory: 'dashboard'
        },
        '/accounts': {
          component: UIAccountListPage,
          navigationCategory: 'accounts'
        },
        '/account': {
          component: UIAccount,
          navigationCategory: 'accounts'
        },
        '/contacts': {
          component: UIContactListPage,
          navigationCategory: 'contacts'
        },
        '/contact': {
          component: UIContact,
          navigationCategory: 'contacts'
        },
        '/tags': {
          component: UITagListPage,
          navigationCategory: 'tags'
        },
        '/tag': {
          component: UITag,
          navigationCategory: 'tags'
        },
        '/budgets': {
          component: UIBudgetListPage,
          navigationCategory: 'budgets'
        },
        '/budget': {
          component: UIBudget,
          navigationCategory: 'budgets'
        },
        '/groups': {
          component: UIGroupListPage,
          navigationCategory: 'groups'
        },
        '/group': {
          component: UIGroup,
          navigationCategory: 'groups'
        },
        '/loans': {
          component: UILoanListPage,
          navigationCategory: 'contacts'
        },
        '/forecast': {
          component: UIForecast,
          navigationCategory: 'forecast'
        },
        '/reminders': {
          component: UIReminderListPage,
          navigationCategory: 'reminders'
        },
        '/reports': {
          component: UIReports,
          navigationCategory: 'reports'
        },
        '/transaction/add': {
          component: UITransactionInput,
          presentationMethod: 'modal'
        },
        '/transactions': {
          component: UITransactionListPage,
          navigationCategory: 'transactions'
        },
        '/transaction': {
          component: UITransactionPage,
          navigationCategory: 'transactions'
        },
        '/membership/plans': {
          component: UIMembershipPlans,
          presentationMethod: 'modal'
        },
        '/membership/payment': {
          component: UIMembershipPayment,
          presentationMethod: 'modal'
        },
        '/search': {
          component: UIGlobalSearch,
          presentationMethod: 'modal'
        },
        '/login': {
          component: UILogin
        },
        '/signup': {
          component: UISignup
        },
        '/feedback': {
          component: UIFeedback
        }
      }
    };

  }
);

/** @jsx React.DOM */


define(
  'touch/sidebar/UISidebar',[
    'touch/app/NavigationController',
    'touch/app/Runtime',
    'touch/core/Model',
    'touch/auth/Session',
    'touch/app/UISectionRow',
    ''
  ],
  function UISidebar(
    AppNavigationController,
    Runtime,
    Model,
    Session,
    UISectionRow,
    junk
  ) {

    var SidebarModel = Model.createClass({
      url: '/apix/sidebar/items'
    });

    return React.createClass({

      getModelsToFetch: function getModelsToFetch() {
        return {
          items: (new SidebarModel())
        };
      },

      compose: function compose() {
        return (
          React.createElement("div", {className: "UISidebar layerSidebar"}, 
            React.createElement("div", {className: "headerContainer"}, 
              React.createElement("div", {className: "closeButtonContainer clickable", onClick: this._didClickCloseButton})
            ), 
            React.createElement("div", {className: "navItemsContainer"}, 
              this._renderNavItems()
            )
          )
        );
      },

      _renderNavItems: function _renderNavItems() {
        var items;
        if (this.state.models.items) {
          items = this.state.models.items.attributes;
        } else {
          items = [
            {
              title: UIStaticText('DASHBOARD'),
              category: 'dashboard'
            },
            {
              title: UIStaticText('REPORTS'),
              category: 'reports'
            },
            {
              title: UIStaticText('BUDGETS'),
              category: 'budgets'
            },
            {
              title: UIStaticText('FORECAST'),
              category: 'forecast'
            }
          ];
        }

        var urlMap = {
          dashboard: '/dashboard',
          forecast: '/forecast',
          reports: '/reports',
          accounts: '/accounts',
          tags: '/tags',
          budgets: '/budgets',
          reminders: '/reminders',
          contacts: '/contacts',
          groups: '/groups'
        };
        var navItems = [];
        ArrayUtils.each(items, function(index, info) {
          navItems.push({
            category: info.category,
            className: 'itemType'+StringUtils.ucfirst(info.category),
            url: urlMap[info.category],
            title: info.title
          });
        });

        if (this.state.models.items) {
          navItems.push({
            url: '/feedback',
            title: UIStaticText('CONTACT US')
          });
        }

        var navItemContainers = [];
        ArrayUtils.each(navItems, FunctionUtils.bind(this, function(index, info) {
          var classNames = ['navItem'];
          if (info.className) {
            classNames.push(info.className);
          }
          classNames = classNames.join(' ');

          navItemContainers.push(
            React.createElement(UISectionRow, {className: classNames, key: 'sidebar-'+info.category, onClick: FunctionUtils.bind(this, this._didClickNavItem, info.url)}, 
              info.title
            )
          );
        }));

        if (this.state.models.items) {
          if (!Runtime.isApp) {
            navItemContainers.push(
              React.createElement(UISectionRow, {className: "navItem", key: "sidebar-desktop", href: "/w/runtime?type=desktop", target: "_top"}, 
                UIStaticText('DESKTOP SITE')
              )
            );
          }

          if (Runtime.isDemo) {
            if (Runtime.isApp) {
              navItemContainers.push(
                React.createElement(UISectionRow, {className: "navItem", key: "sidebar-exit-demo", onClick: this._didClickExitAppDemo}, 
                  UIStaticText('EXIT DEMO')
                )
              );
            } else {
              navItemContainers.push(
                React.createElement(UISectionRow, {className: "navItem", key: "sidebar-exit-demo", href: "https://www.buxfer.com/signup", target: "_top"}, 
                  UIStaticText('EXIT DEMO')
                )
              );
            }
          } else {
            navItemContainers.push(
              React.createElement(UISectionRow, {className: "navItem", key: "sidebar-logout", onClick: this._didClickLogout}, 
                UIStaticText('LOGOUT')
              )
            );
          }
        }

        return navItemContainers;
      },

      _didClickExitAppDemo: function _didClickExitAppDemo() {
        DOM.removeClass(DOM.findBySelector('body'), 'sidebarOpen');
        AppNavigationController.endInAppDemo();
        AppNavigationController.setUrl('/signup');
      },

      _didClickLogout: function _didClickLogout() {
        DOM.removeClass(DOM.findBySelector('body'), 'sidebarOpen');
        Session.logout();
        AppNavigationController.setUrl('/login');
      },

      _didClickNavItem: function _didClickNavItem(url) {
        AppNavigationController.setUrl(url);
        AppNavigationController.toggleSidebar();
      },

      _didClickCloseButton: function _didClickCloseButton() {
        AppNavigationController.toggleSidebar();
      }
    });

  }
);



define(
  'touch/events/Delegater',[
    'touch/app/NavigationController',
    'touch/app/Runtime',
    ''
  ],
  function EventDelegater(
    AppNavigationController,
    Runtime,
    junk
  ) {

    return {

      FORWARD: 'forward',
      STOP: 'stop',

      init: function init() {
        var jWindow = DOM.findBySelector(window);
        var jBody = DOM.findBySelector('body');
        
        jBody.on('click', FunctionUtils.bind(this, this._delegateClick));
        // TODO(perf): This messes up the performance
        // jWindow.scroll(FunctionUtils.bind(this, this._delegateScroll));
      },

      _delegateClick: function _delegateClick(event) {
        var clickTarget = Event.element(event);
        DOM.maybeFindParent(clickTarget, FunctionUtils.bind(this, function(elem) {
          try {
            var href = elem.attr('href');
            if (!href) {
              return false;
            }

            Event.stop(event);

            this._openUrl(href, elem);

            return true;
          } catch (e) {
            WARNING('[EventDelegater] Failed to delegate event ['+e.message+']', e);
            return true;
          }
        }));
      },

      _delegateScroll: function _delegateScroll(event) {
        var target = Event.element(event);
        if (!target.is(DOM.findBySelector(document))) {
          return;
        }
        EventNotificationManager.notify(EventType.PAGE_DID_SCROLL, {
          event: event,
          scrollTop: DOM.getPageScrollOffset()
        });
      },

      _openUrl: function _openUrl(url, elem) {
        var target = elem.attr('target');

        if (!target) {
          AppNavigationController.openUrl(url);
          return;
        }

        DOM.setLoading(elem, true);

        if (Runtime.baseUrl && !url.match(/^http(s)?:\/\//)) {
          url = Runtime.baseUrl + url;
        }

        if (Runtime.isApp) {
          window.open(url, '_system');
          return;
        }

        window.location.href = url;
      }
    };
  }
);

/** @jsx React.DOM */


define(
  'touch/app/Init',[
    'touch/network/Network',
    'touch/app/Runtime',
    'touch/auth/Session',
    'touch/app/NavigationController',
    'touch/app/RouteConfig',
    'touch/sidebar/UISidebar',
    'touch/events/Delegater',
    'touch/errors/Manager',
    ''
  ],
  function AppInit(
    Network,
    Runtime,
    Session,
    AppNavigationController,
    AppRouteConfig,
    UISidebar,
    EventDelegater,
    ErrorManager,
    junk
  ) {

    _setupGlobals();
    ErrorManager.init();

    var pageLoadUrl = AppNavigationController.getCurrentUrl();
    if (Runtime.isApp || pageLoadUrl === '/') {
      pageLoadUrl = '/dashboard';
    }

    var rootElement = DOM.findBySelector('.relativePageBodyContainer');
    var chromeContainer = rootElement[0];

    Network.init();
    _initSession();
    _initChrome();
    _initEventHandlers(); // Depends on chrome init
    _initNavigation(); // Depends on session init

    function _initNavigation() {
      AppNavigationController.initWithRouteConfig(AppRouteConfig);
      AppNavigationController.openUrl(pageLoadUrl);
    }

    function _initSession() {
      React.enableNetworking = false;
      Session.on('refresh', _didSessionRefresh);
      Session.on('didInit', _didSessionInit);
      Session.on('didFailInit', _didSessionFailInit);

      Session.init();
    }

    function _didSessionInit() {
      React.enableNetworking = true;
      Runtime.isOffline = false;
      _didSessionRefresh();
      var url = Session.isLoggedIn() ? pageLoadUrl : '/signup';
      AppNavigationController.openUrl(url);
    }

    function _didSessionFailInit() {
      Runtime.isOffline = true;
      AppNavigationController.reloadPage({
        avoidSessionInit: true
      });
    }

    function _didSessionRefresh() {
      React.unmountComponentAtNode(chromeContainer);
      _initChrome();
    }

    function _initChrome() {
      AppNavigationController.chromeDidInit();

      var content = null;
      if (
        Session.isLoggedIn()
        || (!Session.isInitialized && Runtime.canWorkOffline())
      ) {
        content = (
          React.createElement("div", null, 
            React.createElement(UISidebar, null), 
            React.createElement("div", {className: "preloader"}, 
              React.createElement("div", {className: "fixedWidthFont"}, "ABCD")
            )
          )
        );
      }

      var didClickPopupBackground = function didClickPopupBackground(event) {
        var clickTarget = Event.element(event);
        if (!DOM.maybeFindParentByClass(clickTarget, 'popupContainer')) {
          AppNavigationController.dismissPopup();
        }
      };

      var ui = (
        React.createElement("div", null, 
          content, 
          React.createElement("div", {className: "appContainer"}), 
          React.createElement("div", {className: "modalBackgroundContainer layerOverlayBg"}), 
          React.createElement("div", {className: "popupBackgroundContainer layerOverlayBg", onClick: didClickPopupBackground}), 
          React.createElement("div", {className: "popupContainer layerOverlay"})
        )
      );
      React.render(ui, chromeContainer);
    }
    
    function _initEventHandlers() {
      // Don't pass in document.body here, otherwise touch events on elements
      // outside the body's height (if set to 100% for example), will not receive
      // touchstart events.
      // Fastclick looks for the style.msTouchAction attribute, so copy it over
      // from the body.
      document.style = document.body.style;
      FastClick.attach(document);

      EventDelegater.init();
    }

    function _setupGlobals() {
      Object.define('AppNavigationController', AppNavigationController);
      Object.define('ErrorManager', ErrorManager);
      Object.define('EventDelegater', EventDelegater);
    }
  }
);


